@@ -9,26 +9,28 @@
 
 #include "system.h"
 
+#include <rpmio_internal.h>
+#include <rpmlib.h>
+
+#include "cpio.h"
 #include "fsm.h"
+
 #include "rpmerr.h"
 #include "debug.h"
 
 /*@access FSM_t @*/
 
-/*@-exportheadervar@*/
-extern int _fsm_debug;
-/*@=exportheadervar@*/
-
 /**
  * Convert string to unsigned integer (with buffer size check).
- * @param		input string
- * @retval		address of 1st character not processed
+ * @param str		input string
+ * @retval endptr	address of 1st character not processed
  * @param base		numerical conversion base
  * @param num		max no. of bytes to read
  * @return		converted integer
  */
 static int strntoul(const char *str, /*@out@*/char **endptr, int base, int num)
 	/*@modifies *endptr @*/
+	/*@requires maxSet(endptr) >= 0 @*/
 {
     char * buf, * end;
     unsigned long ret;
@@ -38,20 +40,26 @@ static int strntoul(const char *str, /*@out@*/char **endptr, int base, int num)
     buf[num] = '\0';
 
     ret = strtoul(buf, &end, base);
+/*@-boundsread@*/ /* LCL: strtoul annotations */
     if (*end != '\0')
 	*endptr = ((char *)str) + (end - buf);	/* XXX discards const */
     else
 	*endptr = ((char *)str) + strlen(buf);
+/*@=boundsread@*/
 
     return ret;
 }
 
 #define GET_NUM_FIELD(phys, log) \
+	/*@-boundswrite@*/ \
 	log = strntoul(phys, &end, 16, sizeof(phys)); \
+	/*@=boundswrite@*/ \
 	if ( (end - phys) != sizeof(phys) ) return CPIOERR_BAD_HEADER;
 #define SET_NUM_FIELD(phys, val, space) \
 	sprintf(space, "%8.8lx", (unsigned long) (val)); \
-	memcpy(phys, space, 8);
+	/*@-boundsread@*/ \
+	memcpy(phys, space, 8) \
+	/*@=boundsread@*/
 
 int cpioTrailerWrite(FSM_t fsm)
 {
@@ -59,22 +67,24 @@ int cpioTrailerWrite(FSM_t fsm)
 	(struct cpioCrcPhysicalHeader *)fsm->rdbuf;
     int rc;
 
+/*@-boundswrite@*/
     memset(hdr, '0', PHYS_HDR_SIZE);
     memcpy(hdr->magic, CPIO_NEWC_MAGIC, sizeof(hdr->magic));
     memcpy(hdr->nlink, "00000001", 8);
     memcpy(hdr->namesize, "0000000b", 8);
     memcpy(fsm->rdbuf + PHYS_HDR_SIZE, CPIO_TRAILER, sizeof(CPIO_TRAILER));
+/*@=boundswrite@*/
 
     /* XXX DWRITE uses rdnb for I/O length. */
     fsm->rdnb = PHYS_HDR_SIZE + sizeof(CPIO_TRAILER);
-    rc = fsmStage(fsm, FSM_DWRITE);
+    rc = fsmNext(fsm, FSM_DWRITE);
 
     /*
      * GNU cpio pads to 512 bytes here, but we don't. This may matter for
      * tape device(s) and/or concatenated cpio archives. <shrug>
      */
     if (!rc)
-	rc = fsmStage(fsm, FSM_PAD);
+	rc = fsmNext(fsm, FSM_PAD);
 
     return rc;
 }
@@ -103,15 +113,17 @@ int cpioHeaderWrite(FSM_t fsm, struct stat * st)
 
     len = strlen(fsm->path) + 1; SET_NUM_FIELD(hdr->namesize, len, field);
     memcpy(hdr->checksum, "00000000", 8);
+/*@-boundswrite@*/
     memcpy(fsm->rdbuf + PHYS_HDR_SIZE, fsm->path, len);
+/*@=boundswrite@*/
 
     /* XXX DWRITE uses rdnb for I/O length. */
     fsm->rdnb = PHYS_HDR_SIZE + len;
-    rc = fsmStage(fsm, FSM_DWRITE);
+    rc = fsmNext(fsm, FSM_DWRITE);
     if (!rc && fsm->rdnb != fsm->wrnb)
 	rc = CPIOERR_WRITE_FAILED;
     if (!rc)
-	rc = fsmStage(fsm, FSM_PAD);
+	rc = fsmNext(fsm, FSM_PAD);
     return rc;
 }
 
@@ -125,11 +137,13 @@ int cpioHeaderRead(FSM_t fsm, struct stat * st)
     int rc = 0;
 
     fsm->wrlen = PHYS_HDR_SIZE;
-    rc = fsmStage(fsm, FSM_DREAD);
+    rc = fsmNext(fsm, FSM_DREAD);
     if (!rc && fsm->rdnb != fsm->wrlen)
 	rc = CPIOERR_READ_FAILED;
     if (rc) return rc;
+/*@-boundswrite@*/
     memcpy(&hdr, fsm->wrbuf, fsm->rdnb);
+/*@=boundswrite@*/
 
     if (strncmp(CPIO_CRC_MAGIC, hdr.magic, sizeof(CPIO_CRC_MAGIC)-1) &&
 	strncmp(CPIO_NEWC_MAGIC, hdr.magic, sizeof(CPIO_NEWC_MAGIC)-1))
@@ -161,7 +175,7 @@ int cpioHeaderRead(FSM_t fsm, struct stat * st)
 
     {	char * t = xmalloc(nameSize + 1);
 	fsm->wrlen = nameSize;
-	rc = fsmStage(fsm, FSM_DREAD);
+	rc = fsmNext(fsm, FSM_DREAD);
 	if (!rc && fsm->rdnb != fsm->wrlen)
 	    rc = CPIOERR_BAD_HEADER;
 	if (rc) {
@@ -169,8 +183,10 @@ int cpioHeaderRead(FSM_t fsm, struct stat * st)
 	    fsm->path = NULL;
 	    return rc;
 	}
+/*@-boundswrite@*/
 	memcpy(t, fsm->wrbuf, fsm->rdnb);
 	t[nameSize] = '\0';
+/*@=boundswrite@*/
 	fsm->path = t;
     }
 
@@ -212,6 +228,7 @@ const char *const cpioStrerror(int rc)
     case CPIOERR_READLINK_FAILED: s = "readlink";	break;
     case CPIOERR_READ_FAILED:	s = "read";	break;
     case CPIOERR_COPY_FAILED:	s = "copy";	break;
+    case CPIOERR_LSETFCON_FAILED: s = "lsetfilecon";	break;
 
     case CPIOERR_HDR_SIZE:	s = _("Header size too big");	break;
     case CPIOERR_UNKNOWN_FILETYPE: s = _("Unknown file type");	break;
@@ -219,6 +236,8 @@ const char *const cpioStrerror(int rc)
     case CPIOERR_MD5SUM_MISMATCH: s = _("MD5 sum mismatch");	break;
     case CPIOERR_INTERNAL:	s = _("Internal error");	break;
     case CPIOERR_UNMAPPED_FILE:	s = _("Archive file not in header"); break;
+    case CPIOERR_ENOENT:	s = strerror(ENOENT); break;
+    case CPIOERR_ENOTEMPTY:	s = strerror(ENOTEMPTY); break;
     }
     /*@=branchstate@*/
 
