@@ -78,8 +78,8 @@ static int intcmp(const void * a, const void * b)
  */
 static int removePackage(rpmts ts, Header h, int dboffset,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey depends)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
-	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmte p;
 
@@ -135,11 +135,11 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     uint_32 ohcolor;
     int isSource;
     int duplicate = 0;
-    rpmtsi pi; rpmte p;
+    rpmtsi pi = NULL; rpmte p;
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
     const char * arch;
     const char * os;
-    rpmds add;
+    rpmds oldChk, newChk;
     rpmds obsoletes;
     alKey pkgKey;	/* addedPackages key */
     int xx;
@@ -151,24 +151,38 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
      * Check for previously added versions with the same name and arch/os.
      * FIXME: only catches previously added, older packages.
      */
-    add = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_EQUAL|RPMSENSE_LESS));
     arch = NULL;
     xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
     os = NULL;
     xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
     hcolor = hGetColor(h);
-
     pkgKey = RPMAL_NOMATCH;
+
+    /* XXX Always add source headers. */
+    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    if (isSource) {
+	oc = ts->orderCount;
+	goto addheader;
+    }
+
+    oldChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_LESS));
+    newChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_EQUAL|RPMSENSE_GREATER));
+    /* XXX can't use rpmtsiNext() filter or oc will have wrong value. */
     for (pi = rpmtsiInit(ts), oc = 0; (p = rpmtsiNext(pi, 0)) != NULL; oc++) {
-	const char * parch;
-	const char * pos;
 	rpmds this;
 
 	/* XXX Only added packages need be checked for dupes. */
 	if (rpmteType(p) == TR_REMOVED)
 	    continue;
 
+	/* XXX Never check source headers. */
+	if (rpmteIsSource(p))
+	    continue;
+
 	if (tscolor) {
+	    const char * parch;
+	    const char * pos;
+
 	    if (arch == NULL || (parch = rpmteA(p)) == NULL)
 		continue;
 	    if (os == NULL || (pos = rpmteO(p)) == NULL)
@@ -177,27 +191,48 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 		continue;
 	}
 
+	/* OK, binary rpm's with same arch and os.  Check NEVR. */
 	if ((this = rpmteDS(p, RPMTAG_NAME)) == NULL)
 	    continue;	/* XXX can't happen */
 
-	rc = rpmdsCompare(add, this);
+	/* If newer NEVR was previously added, then skip adding older. */
+	rc = rpmdsCompare(newChk, this);
 	if (rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
-	    const char * addNEVR = rpmdsDNEVR(add);
-	    rpmMessage(RPMMESS_WARNING,
-		_("package %s was already added, replacing with %s\n"),
-		(pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
-		(addNEVR ? addNEVR + 2 : "?addNEVR?"));
+	    const char * addNEVR = rpmdsDNEVR(oldChk);
+	    if (rpmIsVerbose())
+		rpmMessage(RPMMESS_WARNING,
+		    _("package %s was already added, skipping %s\n"),
+		    (pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
+		    (addNEVR ? addNEVR + 2 : "?addNEVR?"));
+	    ec = 1;
+	    break;
+	}
+
+	/* If older NEVR was previously added, then replace old with new. */
+	rc = rpmdsCompare(oldChk, this);
+	if (rc != 0) {
+	    const char * pkgNEVR = rpmdsDNEVR(this);
+	    const char * addNEVR = rpmdsDNEVR(newChk);
+	    if (rpmIsVerbose())
+		rpmMessage(RPMMESS_WARNING,
+		    _("package %s was already added, replacing with %s\n"),
+		    (pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
+		    (addNEVR ? addNEVR + 2 : "?addNEVR?"));
 	    duplicate = 1;
 	    pkgKey = rpmteAddedKey(p);
 	    break;
 	}
     }
     pi = rpmtsiFree(pi);
-    add = rpmdsFree(add);
+    oldChk = rpmdsFree(oldChk);
+    newChk = rpmdsFree(newChk);
 
-    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    /* If newer NEVR was already added, exit now. */
+    if (ec)
+	goto exit;
 
+addheader:
     if (oc >= ts->orderAlloced) {
 	ts->orderAlloced += (oc - ts->orderAlloced) + ts->delta;
 /*@-type +voidabstract @*/
@@ -239,7 +274,12 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	ts->numAddedPackages++;
     }
 
-    if (!upgrade)
+    /* XXX rpmgi hack: Save header in transaction element if requested. */
+    if (upgrade & 0x2)
+	(void) rpmteSetHeader(p, h);
+
+    /* If not upgrading, then we're done. */
+    if (!(upgrade & 0x1))
 	goto exit;
 
     /* XXX binary rpms always have RPMTAG_SOURCERPM, source rpms do not */
@@ -280,7 +320,11 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	    continue;	/* XXX can't happen */
 
 	/* Ignore colored obsoletes not in our rainbow. */
+#if 0
 	dscolor = rpmdsColor(obsoletes);
+#else
+	dscolor = hcolor;
+#endif
 	/* XXX obsoletes are never colored, so this is for future devel. */
 	if (tscolor && dscolor && !(tscolor & dscolor))
 	    continue;
@@ -289,7 +333,10 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if (!strcmp(rpmteN(p), Name))
 	    continue;
 
-	mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, Name, 0);
+	if (Name[0] == '/')
+	    mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, Name, 0);
+	else
+	    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, Name, 0);
 
 	xx = rpmdbPruneIterator(mi,
 	    ts->removedPackages, ts->numRemovedPackages, 1);
@@ -307,11 +354,18 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	     * If no obsoletes version info is available, match all names.
 	     */
 	    if (rpmdsEVR(obsoletes) == NULL
-	     || rpmdsAnyMatchesDep(oh, obsoletes, _rpmds_nopromote))
+	     || rpmdsAnyMatchesDep(oh, obsoletes, _rpmds_nopromote)) {
+		const char * ohNEVRA = hGetNEVRA(oh, NULL);
 #ifdef	DYING	/* XXX see http://bugzilla.redhat.com #134497 */
 		if (rpmVersionCompare(h, oh))
 #endif
 		    xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), pkgKey);
+/*@-nullptrarith@*/
+		rpmMessage(RPMMESS_DEBUG, _("  Obsoletes: %s\t\terases %s\n"),
+			rpmdsDNEVR(obsoletes)+2, ohNEVRA);
+/*@=nullptrarith@*/
+		ohNEVRA = _free(ohNEVRA);
+	    }
 	}
 	mi = rpmdbFreeIterator(mi);
     }
@@ -936,6 +990,7 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
  * @param requires	relation
  * @param zap		max. no. of co-requisites to remove (-1 is all)?
  * @retval nzaps	address of no. of relations removed
+ * @param msglvl	message level at which to spew
  * @return		(possibly NULL) formatted "q <- p" releation (malloc'ed)
  */
 /*@-boundswrite@*/
@@ -979,7 +1034,7 @@ zapRelation(rpmte q, rpmte p,
 	if (zap && !(Flags & RPMSENSE_PREREQ)) {
 	    rpmMessage(msglvl,
 			_("removing %s \"%s\" from tsort relations.\n"),
-			(rpmteNEVR(p) ?  rpmteNEVR(p) : "???"), dp);
+			(rpmteNEVRA(p) ?  rpmteNEVRA(p) : "???"), dp);
 	    rpmteTSI(p)->tsi_count--;
 	    if (tsi_prev) tsi_prev->tsi_next = tsi->tsi_next;
 	    tsi->tsi_next = NULL;
@@ -1075,6 +1130,8 @@ static inline int addRelation(rpmts ts,
 
     if (rpmteDepth(p) <= rpmteDepth(q))	/* Save max. depth in dependency tree */
 	(void) rpmteSetDepth(p, (rpmteDepth(q) + 1));
+    if (rpmteDepth(p) > ts->maxDepth)
+	ts->maxDepth = rpmteDepth(p);
 
     tsi = xcalloc(1, sizeof(*tsi));
     tsi->tsi_suc = p;
@@ -1174,6 +1231,8 @@ int rpmtsOrder(rpmts ts)
     int newOrderCount = 0;
     orderListIndex orderList;
     int numOrderList;
+    int npeer = 128;	/* XXX more than deep enough for now. */
+    int * peer = memset(alloca(npeer*sizeof(*peer)), 0, (npeer*sizeof(*peer)));
     int nrescans = 10;
     int _printed = 0;
     char deptypechar;
@@ -1181,6 +1240,7 @@ int rpmtsOrder(rpmts ts)
     int oType = 0;
     int treex;
     int depth;
+    int breadth;
     int qlen;
     int i, j;
 
@@ -1289,6 +1349,7 @@ int rpmtsOrder(rpmts ts)
 	npreds = rpmteTSI(p)->tsi_count;
 
 	(void) rpmteSetNpreds(p, npreds);
+	(void) rpmteSetDepth(p, 1);
 
 	if (npreds == 0)
 	    (void) rpmteSetTree(p, treex++);
@@ -1300,9 +1361,10 @@ int rpmtsOrder(rpmts ts)
 
     }
     pi = rpmtsiFree(pi);
+    ts->ntrees = treex;
 
     /* T4. Scan for zeroes. */
-    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth)\n"));
+    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth, breadth)\n"));
 
 rescan:
     if (pi != NULL) pi = rpmtsiFree(pi);
@@ -1345,15 +1407,19 @@ rescan:
 	}
 	deptypechar = (rpmteType(q) == TR_REMOVED ? '-' : '+');
 
-	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%5d%5d %*s%c%s\n",
-			orderingCount, rpmteNpreds(q),
-			rpmteTSI(q)->tsi_qcnt, rpmteTree(q), rpmteDepth(q),
-			(2 * rpmteDepth(q)), "",
-			deptypechar,
-			(rpmteNEVR(q) ? rpmteNEVR(q) : "???"));
-
 	treex = rpmteTree(q);
 	depth = rpmteDepth(q);
+	breadth = ((depth < npeer) ? peer[depth]++ : 0);
+	(void) rpmteSetBreadth(q, breadth);
+
+	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%5d%5d%5d %*s%c%s\n",
+			orderingCount, rpmteNpreds(q),
+			rpmteTSI(q)->tsi_qcnt,
+			treex, depth, breadth,
+			(2 * depth), "",
+			deptypechar,
+			(rpmteNEVRA(q) ? rpmteNEVRA(q) : "???"));
+
 	(void) rpmteSetDegree(q, 0);
 	tsbytes += rpmtePkgFileSize(q);
 
@@ -1473,8 +1539,8 @@ rescan:
 
 		/* Print next member of loop. */
 		buf[0] = '\0';
-		if (rpmteNEVR(p) != NULL)
-		    (void) stpcpy(buf, rpmteNEVR(p));
+		if (rpmteNEVRA(p) != NULL)
+		    (void) stpcpy(buf, rpmteNEVRA(p));
 		rpmMessage(msglvl, "    %-40s %s\n", buf,
 			(dp ? dp : "not found!?!"));
 
