@@ -9,6 +9,7 @@
 #include <rpmlib.h>
 #include <rpmmacro.h>
 #include <rpmurl.h>
+#include <rpmlua.h>
 
 #include "cpio.h"
 #include "fsm.h"		/* XXX CPIO_FOO/FSM_FOO constants */
@@ -38,6 +39,14 @@ int _psm_debug = _PSM_DEBUG;
 /*@unchecked@*/
 int _psm_threads = 0;
 
+/* Give access to the rpmte global tracking the last instance added
+ * to the database.
+ */
+/*@-exportheadervar@*/
+/*@unchecked@*/
+extern unsigned int myinstall_instance;
+/*@=exportheadervar@*/
+
 /*@access FD_t @*/		/* XXX void ptr args */
 /*@access rpmpsm @*/
 
@@ -45,6 +54,10 @@ int _psm_threads = 0;
 /*@access rpmte @*/	/* XXX rpmInstallSourcePackage */
 /*@access rpmts @*/	/* XXX ts->notify */
 
+/*@access rpmluav @*/
+/*@access rpmtsScore @*/
+/*@access rpmtsScoreEntry @*/
+
 int rpmVersionCompare(Header first, Header second)
 {
     const char * one, * two;
@@ -477,6 +490,89 @@ static pid_t psmWait(rpmpsm psm)
     return psm->sq.reaped;
 }
 
+/**
+ * Run internal Lua script.
+ */
+static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
+		   int progArgc, const char **progArgv,
+		   const char *script, int arg1, int arg2)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies psm, fileSystem, internalState @*/
+{
+    const rpmts ts = psm->ts;
+    int rootFd = -1;
+    const char *n, *v, *r;
+    rpmRC rc = RPMRC_OK;
+    int i;
+    int xx;
+    rpmlua lua = NULL; /* Global state. */
+    rpmluav var;
+
+    xx = headerNVR(h, &n, &v, &r);
+
+    if (!rpmtsChrootDone(ts)) {
+	const char *rootDir = rpmtsRootDir(ts);
+	xx = chdir("/");
+/*@-nullpass@*/
+	rootFd = open(".", O_RDONLY, 0);
+/*@=nullpass@*/
+	if (rootFd >= 0) {
+	    /*@-superuser -noeffect @*/
+	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+		xx = chroot(rootDir);
+	    /*@=superuser =noeffect @*/
+	    xx = rpmtsSetChrootDone(ts, 1);
+	}
+    }
+
+    /* Create arg variable */
+    rpmluaPushTable(lua, "arg");
+    var = rpmluavNew();
+    rpmluavSetListMode(var, 1);
+/*@+relaxtypes@*/
+    if (progArgv) {
+	for (i = 0; i < progArgc && progArgv[i]; i++) {
+	    rpmluavSetValue(var, RPMLUAV_STRING, progArgv[i]);
+	    rpmluaSetVar(lua, var);
+	}
+    }
+    if (arg1 >= 0) {
+	rpmluavSetValueNum(var, arg1);
+	rpmluaSetVar(lua, var);
+    }
+    if (arg2 >= 0) {
+	rpmluavSetValueNum(var, arg2);
+	rpmluaSetVar(lua, var);
+    }
+/*@=relaxtypes@*/
+/*@-moduncon@*/
+    var = rpmluavFree(var);
+/*@=moduncon@*/
+    rpmluaPop(lua);
+
+    {
+	char buf[BUFSIZ];
+	xx = snprintf(buf, BUFSIZ, "%s(%s-%s-%s)", sln, n, v, r);
+	if (rpmluaRunScript(lua, script, buf) == -1)
+	    rc = RPMRC_FAIL;
+    }
+
+    rpmluaDelVar(lua, "arg");
+
+    if (rootFd >= 0) {
+	const char *rootDir = rpmtsRootDir(ts);
+	xx = fchdir(rootFd);
+	xx = close(rootFd);
+	/*@-superuser -noeffect @*/
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+	    xx = chroot(".");
+	/*@=superuser =noeffect @*/
+	xx = rpmtsSetChrootDone(ts, 0);
+    }
+
+    return rc;
+}
+
 /**
  */
 /*@unchecked@*/
@@ -491,7 +587,7 @@ static const char * ldconfig_path = "/sbin/ldconfig";
  * Run a script with an interpreter. If the interpreter is not specified,
  * /bin/sh will be used. If the interpreter is /bin/sh, then the args from
  * the header will be ignored, passing instead arg1 and arg2.
- * 
+ *
  * @param psm		package state machine data
  * @param h		header
  * @param sln		name of scriptlet section
@@ -504,7 +600,7 @@ static const char * ldconfig_path = "/sbin/ldconfig";
  * @return		0 on success
  */
 static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
-		int progArgc, const char ** progArgv, 
+		int progArgc, const char ** progArgv,
 		const char * script, int arg1, int arg2)
 	/*@globals ldconfig_done, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState@*/
@@ -536,6 +632,14 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     if (progArgv == NULL && script == NULL)
 	return rc;
 
+    if (progArgv && strcmp(progArgv[0], "<lua>") == 0) {
+	rpmMessage(RPMMESS_DEBUG,
+		_("%s: %s(%s-%s-%s.%s) running <lua> scriptlet.\n"),
+		psm->stepName, tag2sln(psm->scriptTag), n, v, r, a);
+	return runLuaScript(psm, h, sln, progArgc, progArgv,
+			    script, arg1, arg2);
+    }
+
     psm->sq.reaper = 1;
 
     /* XXX FIXME: except for %verifyscript, rpmteNEVR can be used. */
@@ -549,7 +653,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     /*
      * If a successor node, and ldconfig was just run, don't bother.
      */
-    if (ldconfig_path && progArgv && psm->unorderedSuccessor) {
+    if (ldconfig_path && progArgv != NULL && psm->unorderedSuccessor) {
  	if (ldconfig_done && !strcmp(progArgv[0], ldconfig_path)) {
 	    rpmMessage(RPMMESS_DEBUG,
 		_("%s: %s(%s-%s-%s.%s) skipping redundant \"%s\".\n"),
@@ -688,7 +792,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	out = fdDup(STDOUT_FILENO);
     }
     if (out == NULL) return RPMRC_FAIL;	/* XXX can't happen */
-    
+
     /*@-branchstate@*/
     xx = rpmsqFork(&psm->sq);
     if (psm->sq.child == 0) {
@@ -756,7 +860,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    }
 	}
 
-	rootDir = rpmtsRootDir(ts);
+	rootDir = ts->rootDir;	/* HACK: rootDir = rpmtsRootDir(ts); instead */
 	if (rootDir  != NULL)	/* XXX can't happen */
 	switch(urlIsURL(rootDir)) {
 	case URL_IS_PATH:
@@ -780,8 +884,10 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    unsetenv("MALLOC_CHECK_");
 
 	    /* Permit libselinux to do the scriptlet exec. */
-	    if (rpmtsSELinuxEnabled(ts) == 1) {
+	    if (rpmtsSELinuxEnabled(ts) == 1) {	
+/*@-moduncon@*/
 		xx = rpm_execcon(0, argv[0], argv, environ);
+/*@=moduncon@*/
 		if (xx != 0)
 		    break;
 	    }
@@ -790,6 +896,11 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    xx = execv(argv[0], (char *const *)argv);
 /*@=nullstate@*/
 	    break;
+	case URL_IS_HTTPS:
+	case URL_IS_HTTP:
+	case URL_IS_FTP:
+	case URL_IS_DASH:
+	case URL_IS_HKP:
 	default:
 	    break;
 	}
@@ -820,7 +931,7 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     if (freePrefixes) prefixes = hfd(prefixes, ipt);
 
     xx = Fclose(out);	/* XXX dup'd STDOUT_FILENO */
-    
+
     /*@-branchstate@*/
     if (script) {
 	if (!rpmIsDebug())
@@ -963,7 +1074,7 @@ static rpmRC handleOneTrigger(const rpmpsm psm,
 		    triggersAlreadyRun[index] == 0)
 		{
 		    rc = runScript(psm, triggeredH, "%trigger", 1,
-			    triggerProgs + index, triggerScripts[index], 
+			    triggerProgs + index, triggerScripts[index],
 			    arg1, arg2);
 		    if (triggersAlreadyRun != NULL)
 			triggersAlreadyRun[index] = 1;
@@ -1006,6 +1117,7 @@ static rpmRC runTriggers(rpmpsm psm)
 
     if (psm->te) 	/* XXX can't happen */
 	N = rpmteN(psm->te);
+/* XXX: Might need to adjust instance counts four autorollback. */
     if (N) 		/* XXX can't happen */
 	numPackage = rpmdbCountPackages(rpmtsGetRdb(ts), N)
 				+ psm->countCorrection;
@@ -1074,7 +1186,7 @@ static rpmRC runImmedTriggers(rpmpsm psm)
 	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, triggerNames[i], 0);
 
 	    while((sourceH = rpmdbNextIterator(mi)) != NULL) {
-		rc |= handleOneTrigger(psm, sourceH, fi->h, 
+		rc |= handleOneTrigger(psm, sourceH, fi->h,
 				rpmdbGetIteratorCount(mi),
 				triggersRun);
 	    }
@@ -1248,7 +1360,7 @@ rpmRC rpmpsmStage(rpmpsm psm, pkgStage stage)
 		rpmfiFC(fi), (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST));
 
 	/*
-	 * When we run scripts, we pass an argument which is the number of 
+	 * When we run scripts, we pass an argument which is the number of
 	 * versions of this package that will be installed when we are
 	 * finished.
 	 */
@@ -1258,6 +1370,48 @@ rpmRC rpmpsmStage(rpmpsm psm, pkgStage stage)
 	    break;
 	}
 
+	/* If we have a score then autorollback is enabled.  If autorollback is
+ 	 * enabled, and this is an autorollback transaction, then we may need to
+	 * adjust the pkgs installed count.
+	 *
+	 * If all this is true, this adjustment should only be made if the PSM goal
+	 * is an install.  No need to make this adjustment on the erase
+	 * component of the upgrade, or even more absurd to do this when doing a
+	 * PKGSAVE.
+	 */
+	if (rpmtsGetScore(ts) != NULL &&
+	    rpmtsGetType(ts) == RPMTRANS_TYPE_AUTOROLLBACK &&
+	    (psm->goal & ~(PSM_PKGSAVE|PSM_PKGERASE))) {
+	    /* Get the score, if its not NULL, get the appropriate
+ 	     * score entry.
+	     */
+	    rpmtsScore score = rpmtsGetScore(ts);
+	    if (score != NULL) {
+		/* OK, we got a real score so lets get the appropriate
+		 * score entry.
+		 */
+		rpmtsScoreEntry se;
+		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
+
+		/* IF the header for the install element has been installed,
+		 * but the header for the erase element has not been erased,
+		 * then decrement the instance count.  This is because in an
+		 * autorollback, if the header was added in the initial transaction
+		 * then in the case of an upgrade the instance count will be
+		 * 2 instead of one when re-installing the old package, and 3 when
+		 * erasing the new package.
+		 *
+		 * Another wrinkle is we only want to make this adjustement
+		 * if the thing we are rollback was an upgrade of package.  A pure
+		 * install or erase does not need the adjustment
+		 */
+		if (se && se->installed &&
+	 	    !se->erased &&
+		    (se->te_types & (TR_ADDED|TR_REMOVED)))
+		    psm->npkgs_installed--;
+	   }
+	}
+
 	if (psm->goal == PSM_PKGINSTALL) {
 	    int fc = rpmfiFC(fi);
 
@@ -1302,7 +1456,7 @@ assert(psm->mi == NULL);
 	     */
 	    {   const char * p;
 		xx = hge(fi->h, RPMTAG_DEFAULTPREFIX, NULL, (void **) &p, NULL);
-		fi->striplen = (xx ? strlen(p) + 1 : 1); 
+		fi->striplen = (xx ? strlen(p) + 1 : 1);
 	    }
 	    fi->mapflags =
 		CPIO_MAP_PATH | CPIO_MAP_MODE | CPIO_MAP_UID | CPIO_MAP_GID;
@@ -1709,7 +1863,7 @@ psm->te->h = headerLink(fi->h);
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPOSTUN)) {
 		rc = rpmpsmNext(psm, PSM_SCRIPT);
-		/* XXX WTFO? postun failures don't cause erasure failure. */
+		if (rc) break;
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERPOSTUN)) {
@@ -1844,7 +1998,8 @@ psm->te->h = headerFree(psm->te->h);
 
 	    xx = chdir("/");
 	    /*@-superuser@*/
-	    rc = chroot(rootDir);
+	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+		rc = chroot(rootDir);
 	    /*@=superuser@*/
 	    psm->chrootDone = 1;
 	    (void) rpmtsSetChrootDone(ts, 1);
@@ -1853,9 +2008,11 @@ psm->te->h = headerFree(psm->te->h);
     case PSM_CHROOT_OUT:
 	/* Restore root directory if changed. */
 	if (psm->chrootDone) {
+	    const char * rootDir = rpmtsRootDir(ts);
 	    const char * currDir = rpmtsCurrDir(ts);
 	    /*@-superuser@*/
-	    rc = chroot(".");
+	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+		rc = chroot(".");
 	    /*@=superuser@*/
 	    psm->chrootDone = 0;
 	    (void) rpmtsSetChrootDone(ts, 0);
@@ -1916,6 +2073,36 @@ assert(psm->mi == NULL);
 	else
 	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
 				NULL, NULL);
+
+	/* Set the database instance so consumers (i.e. rpmtsRun())
+	 * can add this to a rollback transaction.
+	 */
+	rpmteSetDBInstance(psm->te, myinstall_instance);
+
+	/*
+	 * If the score exists and this is not a rollback or autorollback
+	 * then lets check off installed for this package.
+	 */
+	if (rpmtsGetScore(ts) != NULL &&
+	    rpmtsGetType(ts) != RPMTRANS_TYPE_ROLLBACK &&
+	    rpmtsGetType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
+	{
+	    /* Get the score, if its not NULL, get the appropriate
+ 	     * score entry.
+	     */
+	    rpmtsScore score = rpmtsGetScore(ts);
+	    if (score != NULL) {
+		rpmtsScoreEntry se;
+		/* OK, we got a real score so lets get the appropriate
+		 * score entry.
+		 */
+		rpmMessage(RPMMESS_DEBUG,
+		    _("Attempting to mark %s as installed in score board(%u).\n"),
+		    rpmteN(psm->te), (unsigned) score);
+		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
+		if (se != NULL) se->installed = 1;
+	    }
+	}
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
 	break;
     case PSM_RPMDB_REMOVE:
@@ -1923,6 +2110,33 @@ assert(psm->mi == NULL);
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
 	rc = rpmdbRemove(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->record,
 				NULL, NULL);
+
+	/*
+	 * If the score exists and this is not a rollback or autorollback
+	 * then lets check off erased for this package.
+	 */
+	if (rpmtsGetScore(ts) != NULL &&
+	   rpmtsGetType(ts) != RPMTRANS_TYPE_ROLLBACK &&
+	   rpmtsGetType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
+	{
+	    /* Get the score, if its not NULL, get the appropriate
+	     * score entry.
+	     */
+	    rpmtsScore score = rpmtsGetScore(ts);
+
+	    if (score != NULL) { /* XXX: Can't happen */
+		rpmtsScoreEntry se;
+		/* OK, we got a real score so lets get the appropriate
+		 * score entry.
+		 */
+		rpmMessage(RPMMESS_DEBUG,
+		    _("Attempting to mark %s as erased in score board(0x%x).\n"),
+		    rpmteN(psm->te), (unsigned) score);
+		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
+		if (se != NULL) se->erased = 1;
+	    }
+	}
+
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
 	break;
 
