@@ -15,6 +15,9 @@
 
 #include "rpmdb.h"
 #include "rpmfi.h"
+
+#define	_RPMGI_INTERNAL	/* XXX for gi->flags */
+#include "rpmgi.h"
 #include "rpmts.h"
 
 #include "manifest.h"
@@ -22,6 +25,8 @@
 
 #include "debug.h"
 
+/*@access rpmgi @*/
+
 /**
  */
 static void printFileInfo(char * te, const char * name,
@@ -357,6 +362,28 @@ void rpmDisplayQueryTags(FILE * fp)
     }
 }
 
+static int rpmgiShowMatches(QVA_t qva, rpmts ts)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+        /*@modifies qva, rpmGlobalMacroContext, h_errno, internalState @*/
+{
+    rpmgi gi = qva->qva_gi;
+    int ec = 0;
+
+    while (rpmgiNext(gi) == RPMRC_OK) {
+	Header h;
+	int rc;
+
+	h = rpmgiHeader(gi);
+	if (h == NULL)		/* XXX perhaps stricter break instead? */
+	    continue;
+	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
+	    ec = rc;
+	if (qva->qva_source == RPMQV_DBOFFSET)
+	    break;
+    }
+    return ec;
+}
+
 int rpmcliShowMatches(QVA_t qva, rpmts ts)
 {
     Header h;
@@ -393,10 +420,7 @@ static inline unsigned char nibble(char c)
 /*@-bounds@*/ /* LCL: segfault (realpath annotation?) */
 int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 {
-    const char ** av = NULL;
     int res = 0;
-    Header h;
-    int rc;
     const char * s;
     int i;
     int provides_checked = 0;
@@ -409,153 +433,26 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     /*@-branchstate@*/
     switch (qva->qva_source) {
     case RPMQV_RPM:
-    {	int ac = 0;
-	const char * fileURL = NULL;
-	rpmRC rpmrc;
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-	rc = rpmGlob(arg, &ac, &av);
-	if (rc) return 1;
+    case RPMQV_ALL:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-restart:
-	for (i = 0; i < ac; i++) {
-	    FD_t fd;
+    case RPMQV_HDLIST:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
-	    fileURL = _free(fileURL);
-	    fileURL = av[i];
-	    av[i] = NULL;
-
-	    /* Try to read the header from a package file. */
-	    fd = Fopen(fileURL, "r.ufdio");
-	    if (fd == NULL || Ferror(fd)) {
-		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
-			Fstrerror(fd));
-		if (fd != NULL) (void) Fclose(fd);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-
-    	    rpmrc = rpmReadPackageFile(ts, fd, fileURL, &h);
-
-	    (void) Fclose(fd);
-
-	    res = 0;
-	    switch (rpmrc) {
-	    default:
-#ifdef	DYING
-		rpmError(RPMERR_QUERY, _("query of %s failed\n"), fileURL);
-#endif
-		res = 1;
-		/*@switchbreak@*/ break;
-	    case RPMRC_NOTTRUSTED:
-	    case RPMRC_NOKEY:
-	    case RPMRC_OK:
-		if (h == NULL) {
-#ifdef	DYING
-		    rpmError(RPMERR_QUERY,
-			_("old format source packages cannot be queried\n"));
-#endif
-		    res = 1;
-		    /*@switchbreak@*/ break;
-		}
-
-		/* Query a package file. */
-		res = qva->qva_showPackage(qva, ts, h);
-		h = headerFree(h);
-		rpmtsEmpty(ts);
-		continue;
-		/*@notreached@*/ /*@switchbreak@*/ break;
-	    case RPMRC_NOTFOUND:
-		res = 0;
-		/*@switchbreak@*/ break;
-	    }
-	    if (res)
-		/*@loopbreak@*/ break;
-
-	    /* Try to read a package manifest. */
-	    fd = Fopen(fileURL, "r.fpio");
-	    if (fd == NULL || Ferror(fd)) {
-		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
-			Fstrerror(fd));
-		if (fd != NULL) (void) Fclose(fd);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-	    
-	    /* Read list of packages from manifest. */
-/*@-nullstate@*/ /* FIX: *av may be NULL */
-	    res = rpmReadPackageManifest(fd, &ac, &av);
-/*@=nullstate@*/
-	    if (res != RPMRC_OK) {
-		rpmError(RPMERR_MANIFEST,
-			_("%s: not an rpm package (or package manifest): %s\n"),
-			fileURL, Fstrerror(fd));
-		res = 1;
-	    }
-	    (void) Fclose(fd);
-
-	    /* If successful, restart the query loop. */
-	    if (res == 0)
-		goto restart;
-
-	    /*@loopbreak@*/ break;
-	}
-
-	fileURL = _free(fileURL);
-	if (av) {
-	    for (i = 0; i < ac; i++)
-		av[i] = _free(av[i]);
-	    av = _free(av);
-	}
-    }	break;
+    case RPMQV_FTSWALK:
+	res = rpmgiShowMatches(qva, ts);
+	break;
 
     case RPMQV_SPECFILE:
 	res = ((qva->qva_specQuery != NULL)
 		? qva->qva_specQuery(ts, qva, arg) : 1);
 	break;
 
-    case RPMQV_ALL:
-	qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, NULL, 0);
-	if (qva->qva_mi == NULL) {
-	    rpmError(RPMERR_QUERYINFO, _("no packages\n"));
-	    res = 1;
-	} else {
-	    if (arg != NULL)
-	    for (av = (const char **) arg; *av != NULL; av++) {
-		int tag = RPMTAG_NAME;
-		const char * pat;
-		char * a, * ae;
-
-		pat = a = xstrdup(*av);
-		tag = RPMTAG_NAME;
-
-		/* Parse for "tag=pattern" args. */
-		if ((ae = strchr(a, '=')) != NULL) {
-		    *ae++ = '\0';
-		    tag = tagValue(a);
-		    if (tag < 0) {
-			rpmError(RPMERR_QUERYINFO,
-				_("unknown tag: \"%s\"\n"), a);
-			res = 1;
-		    }
-		    pat = ae;
-		}
-
-		if (!res)
-		    res = rpmdbSetIteratorRE(qva->qva_mi, tag, RPMMIRE_DEFAULT, pat);
-		a = _free(a);
-
-		if (res == 0)
-		    continue;
-
-		qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
-		res = 1;
-		/*@loopbreak@*/ break;
-	    }
-	    if (!res)
-		res = rpmcliShowMatches(qva, ts);
-	}
-	break;
-
     case RPMQV_GROUP:
 	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_GROUP, arg, 0);
 	if (qva->qva_mi == NULL) {
@@ -705,6 +602,10 @@ restart:
 		fn = xstrdup(fn);
 	    else
 		fn = xstrdup(arg);
+	} else if (*arg != '/') {
+	    const char *curDir = currentDirectory();
+	    fn = (char *) rpmGetPath(curDir, "/", arg, NULL);
+	    curDir = _free(curDir);
 	} else
 	    fn = xstrdup(arg);
 	(void) rpmCleanPath(fn);
@@ -777,9 +678,92 @@ restart:
 }
 /*@=bounds@*/
 
+int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
+{
+    rpmRC rpmrc = RPMRC_NOTFOUND;
+    int ec = 0;
+
+    switch (qva->qva_source) {
+    case RPMQV_ALL:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_PACKAGES, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, RPMGI_NONE);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, (const char *) argv);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_RPM:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_HDLIST:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_HDLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    case RPMQV_FTSWALK:
+	if (ftsOpts == 0)
+	    ftsOpts = (FTS_COMFOLLOW | FTS_LOGICAL | FTS_NOSTAT);
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_FTSWALK, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
+
+	if (qva->qva_gi != NULL && (qva->qva_gi->flags & RPMGI_TSADD))	/* Load the ts with headers. */
+	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+	if (rpmrc != RPMRC_NOTFOUND)
+	    return 1;	/* XXX should be no. of failures. */
+	
+	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
+	ec = rpmQueryVerify(qva, ts, NULL);
+	/*@=nullpass@*/
+	rpmtsEmpty(ts);
+	break;
+    default:
+	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
+	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts,
+		(giFlags | (RPMGI_NOGLOB|RPMGI_NOHEADER)));
+	while (rpmgiNext(qva->qva_gi) == RPMRC_OK) {
+	    ec += rpmQueryVerify(qva, ts, rpmgiHdrPath(qva->qva_gi));
+	    rpmtsEmpty(ts);
+	}
+	break;
+    }
+
+    qva->qva_gi = rpmgiFree(qva->qva_gi);
+
+    return ec;
+}
+
 int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
 {
-    const char * arg;
     rpmVSFlags vsflags, ovsflags;
     int ec = 0;
 
@@ -820,19 +804,7 @@ int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
 #endif
 
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
-    if (qva->qva_source == RPMQV_ALL) {
-	/*@-nullpass@*/ /* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, (const char *) argv);
-	/*@=nullpass@*/
-    } else {
-/*@-boundsread@*/
-	if (argv != NULL)
-	while ((arg = *argv++) != NULL) {
-	    ec += rpmQueryVerify(qva, ts, arg);
-	    rpmtsEmpty(ts);
-	}
-/*@=boundsread@*/
-    }
+    ec = rpmcliArgIter(ts, qva, argv);
     vsflags = rpmtsSetVSFlags(ts, ovsflags);
 
     if (qva->qva_showPackage == showQueryPackage)
