@@ -78,7 +78,7 @@ typedef /*@abstract@*/ struct fileIndexEntry_s *	fileIndexEntry;
  * A file to be installed/removed.
  */
 struct fileIndexEntry_s {
-/*@dependent@*/ /*@null@*/
+/*@dependent@*/ /*@relnull@*/
     const char * baseName;	/*!< File basename. */
     int baseNameLen;
     alNum pkgNum;		/*!< Containing package index. */
@@ -92,7 +92,7 @@ typedef /*@abstract@*/ struct dirInfo_s *		dirInfo;
  * A directory to be installed/removed.
  */
 struct dirInfo_s {
-/*@owned@*/ /*@null@*/
+/*@owned@*/ /*@relnull@*/
     const char * dirName;	/*!< Directory path (+ trailing '/'). */
     int dirNameLen;		/*!< No. bytes in directory path. */
 /*@owned@*/
@@ -278,7 +278,19 @@ static int fieCompare(const void * one, const void * two)
     if (a->baseName == NULL || b->baseName == NULL)
 	return lenchk;
 
-    /* XXX FIXME: this might do "backward" strcmp for speed */
+#ifdef	NOISY
+/*@-modfilesys@*/
+if (_rpmal_debug) {
+fprintf(stderr, "\t\tstrcmp(%p:%p, %p:%p)", a, a->baseName, b, b->baseName);
+#if 0
+fprintf(stderr, " a %s", a->baseName);
+#endif
+fprintf(stderr, " b %s", a->baseName);
+fprintf(stderr, "\n");
+}
+/*@=modfilesys@*/
+#endif
+
     return strcmp(a->baseName, b->baseName);
 }
 
@@ -330,17 +342,34 @@ fprintf(stderr, "*** del %p[%d]\n", al->list, pkgNum);
 	    if (die == NULL)
 		continue;
 
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "--- die[%5d] %p [%3d] %s\n", (die - al->dirs), die, die->dirNameLen, die->dirName);
+/*@=modfilesys@*/
+
 	    last = die->numFiles;
 	    fie = die->files + last - 1;
 	    for (i = last - 1; i >= 0; i--, fie--) {
 		if (fie->pkgNum != pkgNum)
 		    /*@innercontinue@*/ continue;
 		die->numFiles--;
-		if (i > die->numFiles)
-		    /*@innercontinue@*/ continue;
+
+		if (i < die->numFiles) {
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "\t%p[%3d] memmove(%p:%p,%p:%p,0x%x) %s <- %s\n", die->files, die->numFiles, fie, fie->baseName, fie+1, (fie+1)->baseName, ((die->numFiles - i) * sizeof(*fie)), fie->baseName, (fie+1)->baseName);
+/*@=modfilesys@*/
+
 /*@-bounds@*/
-		memmove(fie, fie+1, (die->numFiles - i) * sizeof(*fie));
+		    memmove(fie, fie+1, (die->numFiles - i) * sizeof(*fie));
 /*@=bounds@*/
+		}
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "\t%p[%3d] memset(%p,0,0x%x) %p [%3d] %s\n", die->files, die->numFiles, die->files + die->numFiles, sizeof(*fie), fie->baseName, fie->baseNameLen, fie->baseName);
+/*@=modfilesys@*/
+		memset(die->files + die->numFiles, 0, sizeof(*fie)); /* overkill */
+
 	    }
 	    if (die->numFiles > 0) {
 		if (last > i)
@@ -351,11 +380,22 @@ fprintf(stderr, "*** del %p[%d]\n", al->list, pkgNum);
 	    die->files = _free(die->files);
 	    die->dirName = _free(die->dirName);
 	    al->numDirs--;
-	    if ((die - al->dirs) > al->numDirs)
-		continue;
+	    if ((die - al->dirs) < al->numDirs) {
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "    die[%5d] memmove(%p,%p,0x%x)\n", (die - al->dirs), die, die+1, ((al->numDirs - (die - al->dirs)) * sizeof(*die)));
+/*@=modfilesys@*/
+
 /*@-bounds@*/
-	    memmove(die, die+1, (al->numDirs - (die - al->dirs)) * sizeof(*die));
+		memmove(die, die+1, (al->numDirs - (die - al->dirs)) * sizeof(*die));
 /*@=bounds@*/
+	    }
+
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "    die[%5d] memset(%p,0,0x%x)\n", al->numDirs, al->dirs + al->numDirs, sizeof(*die));
+/*@=modfilesys@*/
+	    memset(al->dirs + al->numDirs, 0, sizeof(*al->dirs)); /* overkill */
 	}
 
 	if (origNumDirs > al->numDirs) {
@@ -418,34 +458,55 @@ fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
     fi = rpmfiLink(alp->fi, "Files index (rpmalAdd)");
     fi = rpmfiInit(fi, 0);
     if (rpmfiFC(fi) > 0) {
-	int * dirMapping;
 	dirInfo dieNeedle =
 		memset(alloca(sizeof(*dieNeedle)), 0, sizeof(*dieNeedle));
 	dirInfo die;
-	int first;
-	int origNumDirs;
+	int dc = rpmfiDC(fi);
 	int dx;
-	int dc;
-
-	dc = rpmfiDC(fi);
+	int * dirMapping = alloca(sizeof(*dirMapping) * dc);
+	int * dirUnique = alloca(sizeof(*dirUnique) * dc);
+	const char * DN;
+	int origNumDirs;
+	int first;
+	int i;
 
 	/* XXX FIXME: We ought to relocate the directory list here */
 
-	dirMapping = alloca(sizeof(*dirMapping) * dc);
-
-	/*
-	 * Allocated enough space for all the directories we could possible
-	 * need to add
-	 */
+	/* XXX enough space for all directories, late realloc to truncate. */
 	al->dirs = xrealloc(al->dirs, (al->numDirs + dc) * sizeof(*al->dirs));
+
+	/* Only previously allocated dirInfo is sorted and bsearch'able. */
 	origNumDirs = al->numDirs;
 
+	/* Package dirnames are not currently unique. Create unique mapping. */
+	for (dx = 0; dx < dc; dx++) {
+	    (void) rpmfiSetDX(fi, dx);
+	    DN = rpmfiDN(fi);
+	    if (DN != NULL)
+	    for (i = 0; i < dx; i++) {
+		const char * iDN;
+		(void) rpmfiSetDX(fi, i);
+		iDN = rpmfiDN(fi);
+		if (iDN != NULL && !strcmp(DN, iDN))
+		    /*@innerbreak@*/ break;
+	    }
+	    dirUnique[dx] = i;
+	}
+
+	/* Map package dirs into transaction dirInfo index. */
 	for (dx = 0; dx < dc; dx++) {
 
+	    /* Non-unique package dirs use the 1st entry mapping. */
+	    if (dirUnique[dx] < dx) {
+		dirMapping[dx] = dirMapping[dirUnique[dx]];
+		continue;
+	    }
+
+	    /* Find global dirInfo mapping for first encounter. */
 	    (void) rpmfiSetDX(fi, dx);
 
 	    /*@-assignexpose -dependenttrans -observertrans@*/
-	    {   const char * DN = rpmfiDN(fi);
+	    {   DN = rpmfiDN(fi);
 
 #if defined(__ia64__)
 /* XXX Make sure that autorelocated file dependencies are satisfied. */
@@ -473,7 +534,7 @@ fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
 		die->numFiles = 0;
 /*@-modfilesys@*/
 if (_rpmal_debug)
-fprintf(stderr, "+++ die[%3d] %p [%d] %s\n", al->numDirs, die, die->dirNameLen, die->dirName);
+fprintf(stderr, "+++ die[%5d] %p [%3d] %s\n", al->numDirs, die, die->dirNameLen, die->dirName);
 /*@=modfilesys@*/
 
 		al->numDirs++;
@@ -495,8 +556,16 @@ fprintf(stderr, "+++ die[%3d] %p [%d] %s\n", al->numDirs, die, die->dirNameLen,
 	    die = al->dirs + dirMapping[dx];
 	    die->files = xrealloc(die->files,
 			(die->numFiles + next - first) * sizeof(*die->files));
+
 	    fie = die->files + die->numFiles;
 
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "    die[%5d] %p->files [%p[%d],%p) -> [%p[%d],%p)\n", dirMapping[dx], die,
+die->files, die->numFiles, die->files+die->numFiles,
+fie, (next - first), fie + (next - first));
+/*@=modfilesys@*/
+
 	    /* Rewind to first file, generate file index entry for each file. */
 	    fi = rpmfiInit(fi, first);
 	    while ((first = rpmfiNext(fi)) >= 0 && first < next) {
@@ -506,6 +575,11 @@ fprintf(stderr, "+++ die[%3d] %p [%d] %s\n", al->numDirs, die, die->dirNameLen,
 		fie->baseNameLen = (fie->baseName ? strlen(fie->baseName) : 0);
 		fie->pkgNum = pkgNum;
 		fie->ficolor = rpmfiFColor(fi);
+/*@-modfilesys@*/
+if (_rpmal_debug)
+fprintf(stderr, "\t%p[%3d] %p:%p[%2d] %s\n", die->files, die->numFiles, fie, fie->baseName, fie->baseNameLen, rpmfiFN(fi));
+/*@=modfilesys@*/
+
 		die->numFiles++;
 		fie++;
 	    }
