@@ -184,6 +184,7 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
     void * uh = NULL;
     int_32 uht, uhc;
     int res = EXIT_FAILURE;
+    int deleting = (qva->qva_mode == RPMSIGN_DEL_SIGNATURE);
     rpmRC rc;
     int xx;
     
@@ -290,7 +291,13 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
 	xx = headerRemoveEntry(sigh, RPMSIGTAG_SHA1);
 	xx = rpmAddSignature(sigh, sigtarget, RPMSIGTAG_SHA1, qva->passPhrase);
 
-	/* If gpg/pgp is configured, replace the signature. */
+	if (deleting) {	/* Nuke all the signature tags. */
+	    xx = headerRemoveEntry(sigh, RPMSIGTAG_GPG);
+	    xx = headerRemoveEntry(sigh, RPMSIGTAG_DSA);
+	    xx = headerRemoveEntry(sigh, RPMSIGTAG_PGP5);
+	    xx = headerRemoveEntry(sigh, RPMSIGTAG_PGP);
+	    xx = headerRemoveEntry(sigh, RPMSIGTAG_RSA);
+	} else		/* If gpg/pgp is configured, replace the signature. */
 	if ((sigtag = rpmLookupSignatureType(RPMLOOKUPSIG_QUERY)) > 0) {
 	    unsigned char oldsignid[8], newsignid[8];
 
@@ -331,7 +338,6 @@ static int rpmReSign(/*@unused@*/ rpmts ts,
 		    continue;
 		}
 	    }
-
 	}
 
 	/* Reallocate the signature into one contiguous region. */
@@ -556,6 +562,7 @@ static int rpmcliImportPubkeys(const rpmts ts,
     const char * fn;
     const unsigned char * pkt = NULL;
     ssize_t pktlen = 0;
+    char * t = NULL;
     int res = 0;
     rpmRC rpmrc;
     int rc;
@@ -567,12 +574,26 @@ static int rpmcliImportPubkeys(const rpmts ts,
     while ((fn = *argv++) != NULL) {
 /*@=boundsread@*/
 
-rpmtsClean(ts);
+	rpmtsClean(ts);
 	pkt = _free(pkt);
+	t = _free(t);
+
+	/* If arg looks like a keyid, then attempt keyserver retrieve. */
+	if (fn[0] == '0' && fn[1] == 'x') {
+	    const char * s;
+	    int i;
+	    for (i = 0, s = fn+2; *s && isxdigit(*s); s++, i++)
+		{};
+	    if (i == 8 || i == 16) {
+		t = rpmExpand("%{_hkp_keyserver_query}", fn+2, NULL);
+		if (t && *t != '%')
+		    fn = t;
+	    }
+	}
 
 	/* Read pgp packet. */
 	if ((rc = pgpReadPkts(fn, &pkt, &pktlen)) <= 0) {
-	    rpmError(RPMERR_IMPORT, _("%s: import read failed.\n"), fn);
+	    rpmError(RPMERR_IMPORT, _("%s: import read failed(%d).\n"), fn, rc);
 	    res++;
 	    continue;
 	}
@@ -594,6 +615,7 @@ rpmtsClean(ts);
     
 rpmtsClean(ts);
     pkt = _free(pkt);
+    t = _free(t);
     return res;
 }
 
@@ -807,10 +829,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 		xx = pgpPrtPkts(sig, siglen, dig,
 			(_print_pkts & rpmIsDebug()));
 
-		/* XXX only V3 signatures for now. */
-		if (sigp->version != 3) {
+		if (sigp->version != 3 && sigp->version != 4) {
 		    rpmError(RPMERR_SIGVFY,
-		_("only V3 signatures can be verified, skipping V%u signature\n"),
+		_("only V3 or V4 signatures can be verified, skipping V%u signature\n"),
 			sigp->version);
 		    continue;
 		}
@@ -1024,6 +1045,7 @@ int rpmcliSign(rpmts ts, QVA_t qva, const char ** argv)
 	/*@notreached@*/ break;
     case RPMSIGN_NEW_SIGNATURE:
     case RPMSIGN_ADD_SIGNATURE:
+    case RPMSIGN_DEL_SIGNATURE:
 	return rpmReSign(ts, qva, argv);
 	/*@notreached@*/ break;
     case RPMSIGN_NONE:
