@@ -115,7 +115,7 @@ fprintf(stderr, "*** ds %p\t%s[%d]\n", ds, ds->Type, ds->Count);
 }
 
 /*@unchecked@*/ /*@observer@*/
-static const char * beehiveToken = "archfilter";
+static const char * beehiveToken = "redhatbuilddependency";
 
 /**
  * Return archScore filter boolean.
@@ -123,7 +123,8 @@ static const char * beehiveToken = "archfilter";
  * @returns		0 == false, 1 == true
  */
 static int archFilter(const char * arch)
-	/*@*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     static int oneshot = 0;
     int negate = 0;	/* assume no negation. */
@@ -173,9 +174,11 @@ fprintf(stderr, "=== archScore(\"%s\") %d negate %d rc %d\n", arch, archScore, n
  * @param token		namespace string
  * @returns		filtered dependency set
  */
+/*@null@*/
 static rpmds rpmdsFilter(/*@null@*/ /*@returned@*/ rpmds ds,
 		/*@null@*/ const char * token)
-	/*@modifies ds @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ds, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     size_t toklen;
     rpmds fds;
@@ -248,11 +251,11 @@ if (_rpmds_debug < 0)
 fprintf(stderr, "***   deleting N[%d:%d] = \"%s\"\n", i, Count, N);
 /*@=modfilesys@*/
 	    if (i < (Count - 1)) {
-		memcpy((fds->N + i), (fds->N + i + 1), (Count - (i+1)) * sizeof(*fds->N));
+		memmove((fds->N + i), (fds->N + i + 1), (Count - (i+1)) * sizeof(*fds->N));
 		if (fds->EVR != NULL)
-		    memcpy((fds->EVR + i), (fds->EVR + i + 1), (Count - (i+1)) * sizeof(*fds->EVR));
+		    memmove((fds->EVR + i), (fds->EVR + i + 1), (Count - (i+1)) * sizeof(*fds->EVR));
 		if (fds->Flags != NULL)
-		    memcpy((fds->Flags + i), (fds->Flags + i + 1), (Count - (i+1)) * sizeof(*fds->Flags));
+		    memmove((fds->Flags + i), (fds->Flags + i + 1), (Count - (i+1)) * sizeof(*fds->Flags));
 	 	fds->i--;
 	    }
 	    fds->Count--;
@@ -851,7 +854,7 @@ assert(ods->Flags != NULL);
 
 }
 
-int rpmdsFind(rpmds ds, rpmds ods)
+int rpmdsFind(rpmds ds, const rpmds ods)
 {
     int comparison;
 
@@ -1072,9 +1075,8 @@ int rpmdsCompare(const rpmds A, const rpmds B)
 
     if (sense == 0) {
 	sense = rpmvercmp(aV, bV);
-	if (sense == 0 && aR && *aR && bR && *bR) {
+	if (sense == 0 && aR && *aR && bR && *bR)
 	    sense = rpmvercmp(aR, bR);
-	}
     }
 /*@=boundsread@*/
     aEVR = _free(aEVR);
