@@ -5,9 +5,6 @@
  * \file lib/rpmlead.h
  * Routines to read and write an rpm lead structure for a a package.
  */
-
-#include <rpmlib.h>
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -15,20 +12,20 @@ extern "C" {
 /** \ingroup lead
  * Write lead to file handle.
  * @param fd		file handle
- * @param lead		data address
- * @return		0 on success, 1 on error
+ * @param lead		package lead
+ * @return		RPMRC_OK on success, RPMRC_FAIL on error
  */
-int writeLead(FD_t fd, const struct rpmlead *lead)
+rpmRC writeLead(FD_t fd, const struct rpmlead *lead)
 	/*@globals fileSystem @*/
 	/*@modifies fd, fileSystem @*/;
 
 /** \ingroup lead
  * Read lead from file handle.
  * @param fd		file handle
- * @retval lead		data address
- * @return		0 on success, 1 on error
+ * @retval lead		package lead
+ * @return		RPMRC_OK on success, RPMRC_FAIL/RPMRC_NOTFOUND on error
  */
-int readLead(FD_t fd, /*@out@*/ struct rpmlead *lead)
+rpmRC readLead(FD_t fd, /*@out@*/ struct rpmlead *lead)
 	/*@modifies fd, *lead @*/;
 
 #ifdef __cplusplus
