@@ -3,6 +3,9 @@
 
 /** \ingroup rpmcli rpmrc rpmdep rpmtrans rpmdb lead signature header payload dbi
  * \file lib/rpmlib.h
+ *
+ * In Memoriam: Steve Taylor <staylor@redhat.com> was here, now he's not.
+ *
  */
 
 #include "rpmio.h"
@@ -109,6 +112,11 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmdb_s * rpmdb;
  */
 typedef /*@abstract@*/ struct _rpmdbMatchIterator * rpmdbMatchIterator;
 
+/** \ingroup rpmcli
+ * Generalized iterator.
+ */
+typedef /*@abstract@*/ /*@refcounted@*/ struct rpmgi_s * rpmgi;
+
 /** \ingroup header
  * Return name, version, release strings from header.
  * @param h		header
@@ -204,19 +212,23 @@ extern const struct headerSprintfExtension_s rpmHeaderFormats[];
 /*@=redecl@*/
 
 /**
- * Pseudo-tags used by the rpmdb iterator API.
+ * Pseudo-tags used by the rpmdb and rpmgi iterator API's.
  */
-#define	RPMDBI_PACKAGES		0	/*!< Installed package headers. */
-#define	RPMDBI_DEPENDS		1	/*!< Dependency resolution cache. */
-#define	RPMDBI_LABEL		2	/*!< Fingerprint search marker. */
-#define	RPMDBI_ADDED		3	/*!< Added package headers. */
-#define	RPMDBI_REMOVED		4	/*!< Removed package headers. */
-#define	RPMDBI_AVAILABLE	5	/*!< Available package headers. */
+#define	RPMDBI_PACKAGES		0	/* Installed package headers. */
+#define	RPMDBI_DEPENDS		1	/* Dependency resolution cache. */
+#define	RPMDBI_LABEL		2	/* Fingerprint search marker. */
+#define	RPMDBI_ADDED		3	/* Added package headers. */
+#define	RPMDBI_REMOVED		4	/* Removed package headers. */
+#define	RPMDBI_AVAILABLE	5	/* Available package headers. */
+#define	RPMDBI_HDLIST		6	/* (rpmgi) Header list. */
+#define	RPMDBI_ARGLIST		7	/* (rpmgi) Argument list. */
+#define	RPMDBI_FTSWALK		8	/* (rpmgi) File tree  walk. */
 
 /**
  * Tags identify data in package headers.
  * @note tags should not have value 0!
  */
+/** @todo: Somehow supply type **/
 typedef enum rpmTag_e {
 
     RPMTAG_HEADERIMAGE		= HEADER_IMAGE,		/*!< Current image. */
@@ -232,86 +244,97 @@ typedef enum rpmTag_e {
 /* the md5 sum was broken *twice* on big endian machines */
 /* XXX 2nd underscore prevents tagTable generation */
     RPMTAG_SIG_BASE		= HEADER_SIGBASE,
-    RPMTAG_SIGSIZE		= RPMTAG_SIG_BASE+1,
+    RPMTAG_SIGSIZE		= RPMTAG_SIG_BASE+1,	/* i */
     RPMTAG_SIGLEMD5_1		= RPMTAG_SIG_BASE+2,	/*!< internal - obsolete */
-    RPMTAG_SIGPGP		= RPMTAG_SIG_BASE+3,
+    RPMTAG_SIGPGP		= RPMTAG_SIG_BASE+3,	/* x */
     RPMTAG_SIGLEMD5_2		= RPMTAG_SIG_BASE+4,	/*!< internal - obsolete */
-    RPMTAG_SIGMD5	        = RPMTAG_SIG_BASE+5,
-    RPMTAG_SIGGPG	        = RPMTAG_SIG_BASE+6,
+    RPMTAG_SIGMD5	        = RPMTAG_SIG_BASE+5,	/* x */
+#define	RPMTAG_PKGID	RPMTAG_SIGMD5			/* x */
+    RPMTAG_SIGGPG	        = RPMTAG_SIG_BASE+6,	/* x */
     RPMTAG_SIGPGP5	        = RPMTAG_SIG_BASE+7,	/*!< internal - obsolete */
 
     RPMTAG_BADSHA1_1		= RPMTAG_SIG_BASE+8,	/*!< internal - obsolete */
     RPMTAG_BADSHA1_2		= RPMTAG_SIG_BASE+9,	/*!< internal - obsolete */
-    RPMTAG_PUBKEYS		= RPMTAG_SIG_BASE+10,
-    RPMTAG_DSAHEADER		= RPMTAG_SIG_BASE+11,
-    RPMTAG_RSAHEADER		= RPMTAG_SIG_BASE+12,
-    RPMTAG_SHA1HEADER		= RPMTAG_SIG_BASE+13,
+    RPMTAG_PUBKEYS		= RPMTAG_SIG_BASE+10,	/* s[] */
+    RPMTAG_DSAHEADER		= RPMTAG_SIG_BASE+11,	/* x */
+    RPMTAG_RSAHEADER		= RPMTAG_SIG_BASE+12,	/* x */
+    RPMTAG_SHA1HEADER		= RPMTAG_SIG_BASE+13,	/* s */
+#define	RPMTAG_HDRID	RPMTAG_SHA1HEADER	/* s */
 
-    RPMTAG_NAME  		= 1000,
-    RPMTAG_VERSION		= 1001,
-    RPMTAG_RELEASE		= 1002,
-    RPMTAG_EPOCH   		= 1003,
-#define	RPMTAG_SERIAL	RPMTAG_EPOCH	/* backward comaptibility */
-    RPMTAG_SUMMARY		= 1004,
-    RPMTAG_DESCRIPTION		= 1005,
-    RPMTAG_BUILDTIME		= 1006,
-    RPMTAG_BUILDHOST		= 1007,
-    RPMTAG_INSTALLTIME		= 1008,
-    RPMTAG_SIZE			= 1009,
-    RPMTAG_DISTRIBUTION		= 1010,
-    RPMTAG_VENDOR		= 1011,
-    RPMTAG_GIF			= 1012,
-    RPMTAG_XPM			= 1013,
-    RPMTAG_LICENSE		= 1014,
-#define	RPMTAG_COPYRIGHT RPMTAG_LICENSE	/* backward comaptibility */
-    RPMTAG_PACKAGER		= 1015,
-    RPMTAG_GROUP		= 1016,
+    RPMTAG_NAME  		= 1000,	/* s */
+#define	RPMTAG_N	RPMTAG_NAME	/* s */
+    RPMTAG_VERSION		= 1001,	/* s */
+#define	RPMTAG_V	RPMTAG_VERSION	/* s */
+    RPMTAG_RELEASE		= 1002,	/* s */
+#define	RPMTAG_R	RPMTAG_RELEASE	/* s */
+    RPMTAG_EPOCH   		= 1003,	/* i */
+#define	RPMTAG_E	RPMTAG_EPOCH	/* i */
+#define	RPMTAG_SERIAL	RPMTAG_EPOCH	/* i backward comaptibility */
+    RPMTAG_SUMMARY		= 1004,	/* s{} */
+    RPMTAG_DESCRIPTION		= 1005,	/* s{} */
+    RPMTAG_BUILDTIME		= 1006,	/* i */
+    RPMTAG_BUILDHOST		= 1007,	/* s */
+    RPMTAG_INSTALLTIME		= 1008,	/* i */
+    RPMTAG_SIZE			= 1009,	/* i */
+    RPMTAG_DISTRIBUTION		= 1010,	/* s */
+    RPMTAG_VENDOR		= 1011,	/* s */
+    RPMTAG_GIF			= 1012,	/* x */
+    RPMTAG_XPM			= 1013,	/* x */
+    RPMTAG_LICENSE		= 1014,	/* s */
+#define	RPMTAG_COPYRIGHT RPMTAG_LICENSE	/* s backward comaptibility */
+    RPMTAG_PACKAGER		= 1015,	/* s */
+    RPMTAG_GROUP		= 1016,	/* s{} */
 /*@-enummemuse@*/
-    RPMTAG_CHANGELOG		= 1017, /*!< internal */
+    RPMTAG_CHANGELOG		= 1017, /*!< s[] internal */
 /*@=enummemuse@*/
-    RPMTAG_SOURCE		= 1018,
-    RPMTAG_PATCH		= 1019,
-    RPMTAG_URL			= 1020,
-    RPMTAG_OS			= 1021,
-    RPMTAG_ARCH			= 1022,
-    RPMTAG_PREIN		= 1023,
-    RPMTAG_POSTIN		= 1024,
-    RPMTAG_PREUN		= 1025,
-    RPMTAG_POSTUN		= 1026,
-    RPMTAG_OLDFILENAMES		= 1027, /* obsolete */
-    RPMTAG_FILESIZES		= 1028,
-    RPMTAG_FILESTATES		= 1029,
-    RPMTAG_FILEMODES		= 1030,
+    RPMTAG_SOURCE		= 1018,	/* s[] */
+    RPMTAG_PATCH		= 1019,	/* s[] */
+    RPMTAG_URL			= 1020,	/* s */
+    RPMTAG_OS			= 1021,	/* s legacy used int */
+    RPMTAG_ARCH			= 1022,	/* s legacy used int */
+    RPMTAG_PREIN		= 1023,	/* s */
+    RPMTAG_POSTIN		= 1024,	/* s */
+    RPMTAG_PREUN		= 1025,	/* s */
+    RPMTAG_POSTUN		= 1026,	/* s */
+    RPMTAG_OLDFILENAMES		= 1027, /* s[] obsolete */
+    RPMTAG_FILESIZES		= 1028,	/* i */
+    RPMTAG_FILESTATES		= 1029, /* c */
+    RPMTAG_FILEMODES		= 1030,	/* h */
     RPMTAG_FILEUIDS		= 1031, /*!< internal */
     RPMTAG_FILEGIDS		= 1032, /*!< internal */
-    RPMTAG_FILERDEVS		= 1033,
-    RPMTAG_FILEMTIMES		= 1034,
-    RPMTAG_FILEMD5S		= 1035,
-    RPMTAG_FILELINKTOS		= 1036,
-    RPMTAG_FILEFLAGS		= 1037,
+    RPMTAG_FILERDEVS		= 1033,	/* h */
+    RPMTAG_FILEMTIMES		= 1034, /* i */
+    RPMTAG_FILEMD5S		= 1035,	/* s[] */
+    RPMTAG_FILELINKTOS		= 1036,	/* s[] */
+    RPMTAG_FILEFLAGS		= 1037,	/* i */
 /*@-enummemuse@*/
     RPMTAG_ROOT			= 1038, /*!< internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_FILEUSERNAME		= 1039,
-    RPMTAG_FILEGROUPNAME	= 1040,
+    RPMTAG_FILEUSERNAME		= 1039,	/* s[] */
+    RPMTAG_FILEGROUPNAME	= 1040,	/* s[] */
 /*@-enummemuse@*/
     RPMTAG_EXCLUDE		= 1041, /*!< internal - obsolete */
     RPMTAG_EXCLUSIVE		= 1042, /*!< internal - obsolete */
 /*@=enummemuse@*/
     RPMTAG_ICON			= 1043,
-    RPMTAG_SOURCERPM		= 1044,
-    RPMTAG_FILEVERIFYFLAGS	= 1045,
-    RPMTAG_ARCHIVESIZE		= 1046,
-    RPMTAG_PROVIDENAME		= 1047,
-#define	RPMTAG_PROVIDES RPMTAG_PROVIDENAME	/* backward comaptibility */
-    RPMTAG_REQUIREFLAGS		= 1048,
-    RPMTAG_REQUIRENAME		= 1049,
-    RPMTAG_REQUIREVERSION	= 1050,
+    RPMTAG_SOURCERPM		= 1044,	/* s */
+    RPMTAG_FILEVERIFYFLAGS	= 1045,	/* i */
+    RPMTAG_ARCHIVESIZE		= 1046,	/* i */
+    RPMTAG_PROVIDENAME		= 1047,	/* s[] */
+#define	RPMTAG_PROVIDES RPMTAG_PROVIDENAME	/* s[] */
+#define	RPMTAG_P	RPMTAG_PROVIDENAME	/* s[] */
+    RPMTAG_REQUIREFLAGS		= 1048,	/* i */
+    RPMTAG_REQUIRENAME		= 1049,	/* s[] */
+#define	RPMTAG_REQUIRES RPMTAG_REQUIRENAME	/* s[] */
+#define	RPMTAG_D	RPMTAG_REQUIRENAME	/* s[] */
+    RPMTAG_REQUIREVERSION	= 1050,	/* s[] */
     RPMTAG_NOSOURCE		= 1051, /*!< internal */
     RPMTAG_NOPATCH		= 1052, /*!< internal */
-    RPMTAG_CONFLICTFLAGS	= 1053,
-    RPMTAG_CONFLICTNAME		= 1054,
-    RPMTAG_CONFLICTVERSION	= 1055,
+    RPMTAG_CONFLICTFLAGS	= 1053, /* i */
+    RPMTAG_CONFLICTNAME		= 1054,	/* s[] */
+#define	RPMTAG_CONFLICTS RPMTAG_CONFLICTNAME	/* s[] */
+#define	RPMTAG_C	RPMTAG_CONFLICTNAME	/* s[] */
+    RPMTAG_CONFLICTVERSION	= 1055,	/* s[] */
     RPMTAG_DEFAULTPREFIX	= 1056, /*!< internal - deprecated */
     RPMTAG_BUILDROOT		= 1057, /*!< internal */
     RPMTAG_INSTALLPREFIX	= 1058, /*!< internal - deprecated */
@@ -320,36 +343,37 @@ typedef enum rpmTag_e {
     RPMTAG_EXCLUSIVEARCH	= 1061,
     RPMTAG_EXCLUSIVEOS		= 1062,
     RPMTAG_AUTOREQPROV		= 1063, /*!< internal */
-    RPMTAG_RPMVERSION		= 1064,
-    RPMTAG_TRIGGERSCRIPTS	= 1065,
-    RPMTAG_TRIGGERNAME		= 1066,
-    RPMTAG_TRIGGERVERSION	= 1067,
-    RPMTAG_TRIGGERFLAGS		= 1068,
-    RPMTAG_TRIGGERINDEX		= 1069,
-    RPMTAG_VERIFYSCRIPT		= 1079,
-    RPMTAG_CHANGELOGTIME	= 1080,
-    RPMTAG_CHANGELOGNAME	= 1081,
-    RPMTAG_CHANGELOGTEXT	= 1082,
+    RPMTAG_RPMVERSION		= 1064,	/* s */
+    RPMTAG_TRIGGERSCRIPTS	= 1065,	/* s[] */
+    RPMTAG_TRIGGERNAME		= 1066,	/* s[] */
+    RPMTAG_TRIGGERVERSION	= 1067,	/* s[] */
+    RPMTAG_TRIGGERFLAGS		= 1068,	/* i */
+    RPMTAG_TRIGGERINDEX		= 1069,	/* i */
+    RPMTAG_VERIFYSCRIPT		= 1079,	/* s */
+    RPMTAG_CHANGELOGTIME	= 1080,	/* i */
+    RPMTAG_CHANGELOGNAME	= 1081,	/* s[] */
+    RPMTAG_CHANGELOGTEXT	= 1082,	/* s[] */
 /*@-enummemuse@*/
     RPMTAG_BROKENMD5		= 1083, /*!< internal - obsolete */
 /*@=enummemuse@*/
     RPMTAG_PREREQ		= 1084, /*!< internal */
-    RPMTAG_PREINPROG		= 1085,
-    RPMTAG_POSTINPROG		= 1086,
-    RPMTAG_PREUNPROG		= 1087,
-    RPMTAG_POSTUNPROG		= 1088,
+    RPMTAG_PREINPROG		= 1085,	/* s */
+    RPMTAG_POSTINPROG		= 1086,	/* s */
+    RPMTAG_PREUNPROG		= 1087,	/* s */
+    RPMTAG_POSTUNPROG		= 1088,	/* s */
     RPMTAG_BUILDARCHS		= 1089,
-    RPMTAG_OBSOLETENAME		= 1090,
-#define	RPMTAG_OBSOLETES RPMTAG_OBSOLETENAME	/* backward comaptibility */
-    RPMTAG_VERIFYSCRIPTPROG	= 1091,
-    RPMTAG_TRIGGERSCRIPTPROG	= 1092,
+    RPMTAG_OBSOLETENAME		= 1090,	/* s[] */
+#define	RPMTAG_OBSOLETES RPMTAG_OBSOLETENAME	/* s[] */
+#define	RPMTAG_O	RPMTAG_OBSOLETENAME	/* s[] */
+    RPMTAG_VERIFYSCRIPTPROG	= 1091,	/* s */
+    RPMTAG_TRIGGERSCRIPTPROG	= 1092,	/* s */
     RPMTAG_DOCDIR		= 1093, /*!< internal */
-    RPMTAG_COOKIE		= 1094,
-    RPMTAG_FILEDEVICES		= 1095,
-    RPMTAG_FILEINODES		= 1096,
-    RPMTAG_FILELANGS		= 1097,
-    RPMTAG_PREFIXES		= 1098,
-    RPMTAG_INSTPREFIXES		= 1099,
+    RPMTAG_COOKIE		= 1094,	/* s */
+    RPMTAG_FILEDEVICES		= 1095,	/* i */
+    RPMTAG_FILEINODES		= 1096,	/* i */
+    RPMTAG_FILELANGS		= 1097,	/* s[] */
+    RPMTAG_PREFIXES		= 1098,	/* s[] */
+    RPMTAG_INSTPREFIXES		= 1099,	/* s[] */
     RPMTAG_TRIGGERIN		= 1100, /*!< internal */
     RPMTAG_TRIGGERUN		= 1101, /*!< internal */
     RPMTAG_TRIGGERPOSTUN	= 1102, /*!< internal */
@@ -358,7 +382,7 @@ typedef enum rpmTag_e {
 /*@-enummemuse@*/
     RPMTAG_CAPABILITY		= 1105, /*!< internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_SOURCEPACKAGE	= 1106, /*!< src.rpm header marker */
+    RPMTAG_SOURCEPACKAGE	= 1106, /*!< i src.rpm header marker */
 /*@-enummemuse@*/
     RPMTAG_OLDORIGFILENAMES	= 1107, /*!< internal - obsolete */
 /*@=enummemuse@*/
@@ -368,43 +392,59 @@ typedef enum rpmTag_e {
 /*@-enummemuse@*/
     RPMTAG_BUILDMACROS		= 1111, /*!< internal - unused */
 /*@=enummemuse@*/
-    RPMTAG_PROVIDEFLAGS		= 1112,
-    RPMTAG_PROVIDEVERSION	= 1113,
-    RPMTAG_OBSOLETEFLAGS	= 1114,
-    RPMTAG_OBSOLETEVERSION	= 1115,
-    RPMTAG_DIRINDEXES		= 1116,
-    RPMTAG_BASENAMES		= 1117,
-    RPMTAG_DIRNAMES		= 1118,
+    RPMTAG_PROVIDEFLAGS		= 1112,	/* i */
+    RPMTAG_PROVIDEVERSION	= 1113,	/* s[] */
+    RPMTAG_OBSOLETEFLAGS	= 1114,	/* i */
+    RPMTAG_OBSOLETEVERSION	= 1115,	/* s[] */
+    RPMTAG_DIRINDEXES		= 1116,	/* i */
+    RPMTAG_BASENAMES		= 1117,	/* s[] */
+    RPMTAG_DIRNAMES		= 1118,	/* s[] */
     RPMTAG_ORIGDIRINDEXES	= 1119, /*!< internal */
     RPMTAG_ORIGBASENAMES	= 1120, /*!< internal */
     RPMTAG_ORIGDIRNAMES		= 1121, /*!< internal */
-    RPMTAG_OPTFLAGS		= 1122,
-    RPMTAG_DISTURL		= 1123,
-    RPMTAG_PAYLOADFORMAT	= 1124,
-    RPMTAG_PAYLOADCOMPRESSOR	= 1125,
-    RPMTAG_PAYLOADFLAGS		= 1126,
-    RPMTAG_INSTALLCOLOR		= 1127, /*!< transaction color when installed */
-    RPMTAG_INSTALLTID		= 1128,
-    RPMTAG_REMOVETID		= 1129,
+    RPMTAG_OPTFLAGS		= 1122,	/* s */
+    RPMTAG_DISTURL		= 1123,	/* s */
+    RPMTAG_PAYLOADFORMAT	= 1124,	/* s */
+    RPMTAG_PAYLOADCOMPRESSOR	= 1125,	/* s */
+    RPMTAG_PAYLOADFLAGS		= 1126,	/* s */
+    RPMTAG_INSTALLCOLOR		= 1127, /*!< i transaction color when installed */
+    RPMTAG_INSTALLTID		= 1128,	/* i */
+    RPMTAG_REMOVETID		= 1129,	/* i */
 /*@-enummemuse@*/
     RPMTAG_SHA1RHN		= 1130, /*!< internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_RHNPLATFORM		= 1131,
-    RPMTAG_PLATFORM		= 1132,
+    RPMTAG_RHNPLATFORM		= 1131,	/* s */
+    RPMTAG_PLATFORM		= 1132,	/* s */
     RPMTAG_PATCHESNAME		= 1133, /*!< placeholder (SuSE) */
     RPMTAG_PATCHESFLAGS		= 1134, /*!< placeholder (SuSE) */
     RPMTAG_PATCHESVERSION	= 1135, /*!< placeholder (SuSE) */
-    RPMTAG_CACHECTIME		= 1136,
-    RPMTAG_CACHEPKGPATH		= 1137,
-    RPMTAG_CACHEPKGSIZE		= 1138,
-    RPMTAG_CACHEPKGMTIME	= 1139,
-    RPMTAG_FILECOLORS		= 1140,
-    RPMTAG_FILECLASS		= 1141,
-    RPMTAG_CLASSDICT		= 1142,
-    RPMTAG_FILEDEPENDSX		= 1143,
-    RPMTAG_FILEDEPENDSN		= 1144,
-    RPMTAG_DEPENDSDICT		= 1145,
-    RPMTAG_SOURCEPKGID		= 1146,
+    RPMTAG_CACHECTIME		= 1136,	/* i */
+    RPMTAG_CACHEPKGPATH		= 1137,	/* s */
+    RPMTAG_CACHEPKGSIZE		= 1138,	/* i */
+    RPMTAG_CACHEPKGMTIME	= 1139,	/* i */
+    RPMTAG_FILECOLORS		= 1140,	/* i */
+    RPMTAG_FILECLASS		= 1141,	/* i */
+    RPMTAG_CLASSDICT		= 1142,	/* s[] */
+    RPMTAG_FILEDEPENDSX		= 1143,	/* i */
+    RPMTAG_FILEDEPENDSN		= 1144,	/* i */
+    RPMTAG_DEPENDSDICT		= 1145,	/* i */
+    RPMTAG_SOURCEPKGID		= 1146,	/* x */
+    RPMTAG_FILECONTEXTS		= 1147,	/* s[] */
+    RPMTAG_FSCONTEXTS		= 1148,	/*!< s[] extension */
+    RPMTAG_RECONTEXTS		= 1149,	/*!< s[] extension */
+    RPMTAG_POLICIES		= 1150,	/*!< s[] selinux *.te policy file. */
+    RPMTAG_PRETRANS		= 1151,	/* s */
+    RPMTAG_POSTTRANS		= 1152,	/* s */
+    RPMTAG_PRETRANSPROG		= 1153,	/* s */
+    RPMTAG_POSTTRANSPROG	= 1154,	/* s */
+    RPMTAG_DISTTAG		= 1155,	/* s */
+    RPMTAG_SUGGESTSNAME		= 1156,	/* s[] extension placeholder */
+    RPMTAG_SUGGESTSVERSION	= 1157,	/* s[] extension placeholder */
+    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i   extension placeholder */
+    RPMTAG_ENHANCESNAME		= 1159,	/* s[] extension placeholder */
+    RPMTAG_ENHANCESVERSION	= 1160,	/* s[] extension placeholder */
+    RPMTAG_ENHANCESFLAGS	= 1161,	/* i   extension placeholder */
+    RPMTAG_PRIORITY		= 1162, /* i   extension placeholder */
 /*@-enummemuse@*/
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
 /*@=enummemuse@*/
@@ -440,9 +480,10 @@ typedef	enum rpmfileAttrs_e {
     RPMFILE_GHOST	= (1 <<  6),	/*!< from %%ghost */
     RPMFILE_LICENSE	= (1 <<  7),	/*!< from %%license */
     RPMFILE_README	= (1 <<  8),	/*!< from %%readme */
-    RPMFILE_EXCLUDE	= (1 <<  9),	/*!< from %%exclude */
+    RPMFILE_EXCLUDE	= (1 <<  9),	/*!< from %%exclude, internal */
     RPMFILE_UNPATCHED	= (1 << 10),	/*!< placeholder (SuSE) */
-    RPMFILE_PUBKEY	= (1 << 11)	/*!< from %%pubkey */
+    RPMFILE_PUBKEY	= (1 << 11),	/*!< from %%pubkey */
+    RPMFILE_POLICY	= (1 << 12)	/*!< from %%policy */
 } rpmfileAttrs;
 
 #define	RPMFILE_ALL	~(RPMFILE_NONE)
@@ -460,7 +501,8 @@ typedef	enum rpmsenseFlags_e {
     RPMSENSE_EQUAL	= (1 << 3),
     RPMSENSE_PROVIDES	= (1 << 4), /* only used internally by builds */
     RPMSENSE_CONFLICTS	= (1 << 5), /* only used internally by builds */
-    RPMSENSE_PREREQ	= (1 << 6),	/*!< @todo Legacy. */
+	/* bit 6 used to be RPMSENSE_PREREQ */
+#define	RPMSENSE_PREREQ	RPMSENSE_ANY
     RPMSENSE_OBSOLETES	= (1 << 7), /* only used internally by builds */
     RPMSENSE_INTERP	= (1 << 8),	/*!< Interpreter used by scriptlet. */
     RPMSENSE_SCRIPT_PRE	= ((1 << 9)|RPMSENSE_PREREQ), /*!< %pre dependency. */
@@ -474,12 +516,12 @@ typedef	enum rpmsenseFlags_e {
     RPMSENSE_TRIGGERIN	= (1 << 16),	/*!< %triggerin dependency. */
     RPMSENSE_TRIGGERUN	= (1 << 17),	/*!< %triggerun dependency. */
     RPMSENSE_TRIGGERPOSTUN = (1 << 18),	/*!< %triggerpostun dependency. */
-	/* (1 << 19) unused. */
+    RPMSENSE_MISSINGOK	= (1 << 19),	/*!< suggests/enhances hint. */
     RPMSENSE_SCRIPT_PREP = (1 << 20),	/*!< %prep build dependency. */
     RPMSENSE_SCRIPT_BUILD = (1 << 21),	/*!< %build build dependency. */
     RPMSENSE_SCRIPT_INSTALL = (1 << 22),/*!< %install build dependency. */
     RPMSENSE_SCRIPT_CLEAN = (1 << 23),	/*!< %clean build dependency. */
-    RPMSENSE_RPMLIB	= ((1 << 24) | RPMSENSE_PREREQ), /*!< rpmlib(feature) dependency. */
+    RPMSENSE_RPMLIB = ((1 << 24) | RPMSENSE_PREREQ), /*!< rpmlib(feature) dependency. */
 /*@-enummemuse@*/
     RPMSENSE_TRIGGERPREIN = (1 << 25),	/*!< @todo Implement %triggerprein. */
 /*@=enummemuse@*/
@@ -570,7 +612,7 @@ enum rpm_machtable_e {
  */
 int rpmReadConfigFiles(/*@null@*/ const char * file,
 		/*@null@*/ const char * target)
-	/*@globals rpmGlobalMacroContext, rpmCLIMacroContext,
+	/*@globals rpmGlobalMacroContext, rpmCLIMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies rpmGlobalMacroContext, rpmCLIMacroContext,
 		fileSystem, internalState @*/;
@@ -617,7 +659,7 @@ int rpmMachineScore(int type, const char * name)
  * @return		0 always
  */
 int rpmShowRC(FILE * fp)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies *fp, rpmGlobalMacroContext, fileSystem, internalState  @*/;
 
 /** \ingroup rpmrc
@@ -628,7 +670,7 @@ int rpmShowRC(FILE * fp)
  * @param osTable
  */
 void rpmSetTables(int archTable, int osTable)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmrc
@@ -642,7 +684,7 @@ void rpmSetTables(int archTable, int osTable)
  * @param os		os name (or NULL)
  */
 void rpmSetMachine(/*@null@*/ const char * arch, /*@null@*/ const char * os)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmrc
@@ -863,7 +905,7 @@ int headerVerifyInfo(int il, int dl, const void * pev, void * iv, int negate)
  */
 rpmRC headerCheck(rpmts ts, const void * uh, size_t uc,
 		/*@out@*/ /*@null@*/ const char ** msg)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, *msg, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
@@ -877,7 +919,7 @@ rpmRC headerCheck(rpmts ts, const void * uh, size_t uc,
  */
 rpmRC rpmReadHeader(rpmts ts, FD_t fd, /*@out@*/ Header *hdrp,
 		/*@out@*/ /*@null@*/ const char ** msg)
-        /*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+        /*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
         /*@modifies ts, *hdrp, *msg, rpmGlobalMacroContext,
                 fileSystem, internalState @*/;
 
@@ -887,11 +929,11 @@ rpmRC rpmReadHeader(rpmts ts, FD_t fd, /*@out@*/ Header *hdrp,
  * @param fd		file handle
  * @param fn		file name
  * @retval hdrp		address of header (or NULL)
- * @return		0 on success
+ * @return		RPMRC_OK on success
  */
-int rpmReadPackageFile(rpmts ts, FD_t fd,
+rpmRC rpmReadPackageFile(rpmts ts, FD_t fd,
 		const char * fn, /*@null@*/ /*@out@*/ Header * hdrp)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fd, *hdrp, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
@@ -906,7 +948,7 @@ int rpmReadPackageFile(rpmts ts, FD_t fd,
 rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 			/*@null@*/ /*@out@*/ const char ** specFilePtr,
 			/*@null@*/ /*@out@*/ const char ** cookie)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fd, *specFilePtr, *cookie, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
@@ -925,7 +967,7 @@ typedef enum rpmtransFlags_e {
 /*@-enummemuse@*/
     RPMTRANS_FLAG_KEEPOBSOLETE	= (1 <<  7),	/*!< @todo Document. */
 /*@=enummemuse@*/
-	/* (1 << 8) unused. */
+    RPMTRANS_FLAG_NOCONTEXTS	= (1 <<  8),	/*!< from --nocontexts */
     RPMTRANS_FLAG_DIRSTASH	= (1 <<  9),	/*!< from --dirstash */
     RPMTRANS_FLAG_REPACKAGE	= (1 << 10),	/*!< from --repackage */
 
@@ -952,11 +994,12 @@ typedef enum rpmtransFlags_e {
 /*@=enummemuse@*/
     RPMTRANS_FLAG_APPLYONLY	= (1 << 25),
 
-    RPMTRANS_FLAG_ANACONDA	= (1 << 26),
+    RPMTRANS_FLAG_ANACONDA	= (1 << 26),	/*!< from --anaconda */
     RPMTRANS_FLAG_NOMD5		= (1 << 27),	/*!< from --nomd5 */
     RPMTRANS_FLAG_NOSUGGEST	= (1 << 28),	/*!< from --nosuggest */
     RPMTRANS_FLAG_ADDINDEPS	= (1 << 29),	/*!< from --aid */
-    RPMTRANS_FLAG_NOCONFIGS	= (1 << 30)	/*!< from --noconfigs */
+    RPMTRANS_FLAG_NOCONFIGS	= (1 << 30),	/*!< from --noconfigs */
+    RPMTRANS_FLAG_DEPLOOPS	= (1 << 31)	/*!< from --deploops */
 } rpmtransFlags;
 
 #define	_noTransScripts		\
@@ -1095,7 +1138,7 @@ int rpmGetFilesystemList( /*@null@*/ /*@out@*/ const char *** listptr,
 int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes,
 		int numFiles, /*@null@*/ /*@out@*/ uint_32 ** usagesPtr,
 		int flags)
-	/*@globals rpmGlobalMacroContext,
+	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies *usagesPtr, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
@@ -1174,7 +1217,7 @@ enum rpmtagSignature {
  */
 rpmRC rpmVerifySignature(const rpmts ts,
 		/*@out@*/ char * result)
-	/*@globals rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, *result, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
