@@ -51,6 +51,9 @@ static struct rpmlibProvides_s rpmlibProvides[] = {
     { "rpmlib(ConcurrentAccess)",    "4.1-1",
 	(                RPMSENSE_EQUAL),
     N_("package scriptlets may access the rpm database while installing.") },
+    { "rpmlib(BuiltinLuaScripts)",    "4.2.2-1",
+	(                RPMSENSE_EQUAL),
+    N_("internal support for lua scripts.") },
     { NULL,				NULL, 0,	NULL }
 };
 
