@@ -16,6 +16,7 @@
 
 #include <rpmlib.h>
 #include <rpmmacro.h>
+#include <rpmlua.h>
 
 #include "misc.h"
 #include "debug.h"
@@ -892,16 +893,13 @@ static inline void cpuid(unsigned int op, int *eax, int *ebx, int *ecx, int *edx
 #ifdef	__LCLINT__
     *eax = *ebx = *ecx = *edx = 0;
 #endif
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
-		: "=a"(*eax), "=g"(*ebx), "=&c"(*ecx), "=&d"(*edx)
-		: "a" (op));
-#else
-	__asm__("cpuid"
-		: "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx)
-		: "a" (op));
-#endif
-
+    asm volatile (
+	"pushl	%%ebx		\n"
+	"cpuid			\n"
+	"movl	%%ebx,	%%esi	\n"
+	"popl	%%ebx		\n"
+    : "=a" (*eax), "=S" (*ebx), "=c" (*ecx), "=d" (*edx)
+    : "a" (op));
 }
 
 /*
@@ -910,15 +908,8 @@ static inline void cpuid(unsigned int op, int *eax, int *ebx, int *ecx, int *edx
 static inline unsigned int cpuid_eax(unsigned int op)
 	/*@*/
 {
-	unsigned int val;
-
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (val) : "a" (op) : "ecx", "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (val) : "a" (op) : "ebx", "ecx", "edx");
-#endif
+	unsigned int tmp, val;
+	cpuid(op, &val, &tmp, &tmp, &tmp);
 	return val;
 }
 
@@ -926,14 +917,7 @@ static inline unsigned int cpuid_ebx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
-		: "=a" (tmp), "=g" (val) : "a" (op) : "ecx", "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=b" (val) : "a" (op) : "ecx", "edx");
-#endif
+	cpuid(op, &tmp, &val, &tmp, &tmp);
 	return val;
 }
 
@@ -941,30 +925,16 @@ static inline unsigned int cpuid_ecx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (tmp), "=c" (val) : "a" (op) : "edx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=c" (val) : "a" (op) : "ebx", "edx");
-#endif
+	cpuid(op, &tmp, &tmp, &val, &tmp);
 	return val;
-
 }
 
 static inline unsigned int cpuid_edx(unsigned int op)
 	/*@*/
 {
 	unsigned int tmp, val;
-#ifdef PIC
-	__asm__("pushl %%ebx; cpuid; popl %%ebx"
-		: "=a" (tmp), "=d" (val) : "a" (op) : "ecx");
-#else
-	__asm__("cpuid"
-		: "=a" (tmp), "=d" (val) : "a" (op) : "ebx", "ecx");
-#endif
+	cpuid(op, &tmp, &tmp, &tmp, &val);
 	return val;
-
 }
 
 /*@unchecked@*/
@@ -1037,6 +1007,73 @@ static int is_athlon(void)
 	return 1;
 }
 
+static int is_pentium3()
+{
+    unsigned int eax, ebx, ecx, edx, family, model;
+    char vendor[16];
+    cpuid(0, &eax, &ebx, &ecx, &edx);
+    memset(vendor, 0, sizeof(vendor));
+    *((unsigned int *)&vendor[0]) = ebx;
+    *((unsigned int *)&vendor[4]) = edx;
+    *((unsigned int *)&vendor[8]) = ecx;
+    if (strncmp(vendor, "GenuineIntel", 12) != 0)
+	return 0;
+    cpuid(1, &eax, &ebx, &ecx, &edx);
+    family = (eax >> 8) & 0x0f;
+    model = (eax >> 4) & 0x0f;
+    if (family == 6)
+	switch (model)
+	{
+	    case 7:	// Pentium III, Pentium III Xeon (model 7)
+	    case 8:	// Pentium III, Pentium III Xeon, Celeron (model 8)
+	    case 9:	// Pentium M
+			/*
+			    Intel recently announced its new technology for mobile platforms,
+			    named Centrino, and presents it as a big advance in mobile PCs.
+			    One of the main part of Centrino consists in a brand new CPU,
+			    the Pentium M, codenamed Banias, that we'll study in this review.
+			    A particularity of this CPU is that it was designed for mobile platform
+			    exclusively, unlike previous mobile CPU (Pentium III-M, Pentium 4-M)
+			    that share the same micro-architecture as their desktop counterparts.
+			    The Pentium M introduces a new micro-architecture, adapted for mobility
+			    constraints, and that is halfway between the Pentium III and the Pentium 4.
+						    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			*/
+	    case 10:	// Pentium III Xeon (model A)
+	    case 11:	// Pentium III (model B)
+		return 1;
+	}
+    return 0;
+}
+
+static int is_pentium4()
+{
+    unsigned int eax, ebx, ecx, edx, family, model;
+    char vendor[16];
+    cpuid(0, &eax, &ebx, &ecx, &edx);
+    memset(vendor, 0, sizeof(vendor));
+    *((unsigned int *)&vendor[0]) = ebx;
+    *((unsigned int *)&vendor[4]) = edx;
+    *((unsigned int *)&vendor[8]) = ecx;
+    if (strncmp(vendor, "GenuineIntel", 12) != 0)
+	return 0;
+    cpuid(1, &eax, &ebx, &ecx, &edx);
+    family = (eax >> 8) & 0x0f;
+    model = (eax >> 4) & 0x0f;
+    if (family == 15)
+	switch (model)
+	{
+	    case 0:	// Pentium 4, Pentium 4 Xeon                 (0.18um)
+	    case 1:	// Pentium 4, Pentium 4 Xeon MP, Celeron     (0.18um)
+	    case 2:	// Pentium 4, Mobile Pentium 4-M,
+			// Pentium 4 Xeon, Pentium 4 Xeon MP,
+			// Celeron, Mobile Celron                    (0.13um)
+	    case 3:	// Pentium 4, Celeron                        (0.09um)
+		return 1;
+	}
+    return 0;
+}
+
 #endif
 
 #if defined(__linux__) && defined(__powerpc__)
@@ -1094,6 +1131,13 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 	    strcpy(un.machine, __power_pc() ? "ppc" : "rs6000");
 	    sprintf(un.sysname,"aix%s.%s", un.version, un.release);
 	}
+	else if(!strcmp(un.sysname, "Darwin")) { 
+#ifdef __ppc__
+	    strcpy(un.machine, "ppc");
+#else ifdef __i386__
+	    strcpy(un.machine, "i386");
+#endif 
+	}
 	else if (!strcmp(un.sysname, "SunOS")) {
 	    if (!strncmp(un.release,"4", 1)) /* SunOS 4.x */ {
 		int fd;
@@ -1273,6 +1317,10 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 
 	    if ((class == '6' && is_athlon()) || class == '7')
 	    	strcpy(un.machine, "athlon");
+	    else if (is_pentium4())
+		strcpy(un.machine, "pentium4");
+	    else if (is_pentium3())
+		strcpy(un.machine, "pentium3");
 	    else if (strchr("3456", un.machine[1]) && un.machine[1] != class)
 		un.machine[1] = class;
 	}
@@ -1834,6 +1882,9 @@ int rpmReadConfigFiles(const char * file, const char * target)
 	os = _free(os);
     }
 
+    /* Force Lua state initialization */
+    (void)rpmluaGetPrintBuffer(NULL);
+
     return 0;
 }
 
