@@ -85,8 +85,9 @@ static void delTE(rpmte p)
 static void addTE(rpmts ts, rpmte p, Header h,
 		/*@dependent@*/ /*@null@*/ fnpyKey key,
 		/*@null@*/ rpmRelocation * relocs)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
-	/*@modifies ts, p, h, rpmGlobalMacroContext, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, p, h,
+		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     int scareMem = 0;
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
@@ -104,6 +105,11 @@ static void addTE(rpmts ts, rpmte p, Header h,
     if ((p->version = strrchr(p->name, '-')) != NULL)
 	*p->version++ = '\0';
 
+    /* Set db_instance to 0 as it has not been installed
+     * necessarily yet.
+     */
+    p->db_instance = 0;
+
     arch = NULL;
     xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
     if (arch != NULL) {
@@ -122,15 +128,20 @@ static void addTE(rpmts ts, rpmte p, Header h,
 	p->os = NULL;
 	p->osScore = 0;
     }
+    p->isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
 
     nb = strlen(p->NEVR) + 1;
-    if (p->arch)
+    if (p->isSource)
+	nb += sizeof("src");
+    else if (p->arch)
 	nb += strlen(p->arch) + 1;
     t = xmalloc(nb);
     p->NEVRA = t;
     *t = '\0';
     t = stpcpy(t, p->NEVR);
-    if (p->arch)
+    if (p->isSource)
+	t = stpcpy( t, ".src");
+    else if (p->arch)
 	t = stpcpy( stpcpy( t, "."), p->arch);
 
     ep = NULL;
@@ -225,6 +236,34 @@ rpmte rpmteNew(const rpmts ts, Header h,
     return p;
 }
 
+/* Get the DB Instance value */
+unsigned int rpmteDBInstance(rpmte te) 
+{
+    return (te != NULL ? te->db_instance : 0);
+}
+
+/* Set the DB Instance value */
+void rpmteSetDBInstance(rpmte te, unsigned int instance) 
+{
+    if (te != NULL) 
+	te->db_instance = instance;
+}
+
+Header rpmteHeader(rpmte te)
+{
+    return (te != NULL && te->h != NULL ? headerLink(te->h) : NULL);
+}
+
+Header rpmteSetHeader(rpmte te, Header h)
+{
+    if (te != NULL)  {
+	te->h = headerFree(te->h);
+	if (h != NULL)
+	    te->h = headerLink(h);
+    }
+    return NULL;
+}
+
 rpmElementType rpmteType(rpmte te)
 {
     return (te != NULL ? te->type : -1);
@@ -260,6 +299,11 @@ const char * rpmteO(rpmte te)
     return (te != NULL ? te->os : NULL);
 }
 
+int rpmteIsSource(rpmte te)
+{
+    return (te != NULL ? te->isSource : NULL);
+}
+
 uint_32 rpmteColor(rpmte te)
 {
     return (te != NULL ? te->color : 0);
@@ -295,6 +339,21 @@ int rpmteSetDepth(rpmte te, int ndepth)
     return odepth;
 }
 
+int rpmteBreadth(rpmte te)
+{
+    return (te != NULL ? te->depth : 0);
+}
+
+int rpmteSetBreadth(rpmte te, int nbreadth)
+{
+    int obreadth = 0;
+    if (te != NULL) {
+	obreadth = te->breadth;
+	te->breadth = nbreadth;
+    }
+    return obreadth;
+}
+
 int rpmteNpreds(rpmte te)
 {
     return (te != NULL ? te->npreds : 0);
