@@ -226,7 +226,7 @@ static int isArch(const char * arch)
 /*@-nullassign@*/
     /*@observer@*/
     static const char *arches[] = {
-	"i386", "i486", "i586", "i686", "athlon", "x86_64", "amd64", "ia32e",
+	"i386", "i486", "i586", "i686", "athlon", "pentium3", "pentium4", "x86_64", "amd64", "ia32e",
 	"alpha", "alphaev5", "alphaev56", "alphapca56", "alphaev6", "alphaev67",
 	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9",
 	"sparc64", "sun4u",
@@ -340,25 +340,36 @@ rpmRC rpmtsFindPubkey(rpmts ts)
     const void * sig = rpmtsSig(ts);
     pgpDig dig = rpmtsDig(ts);
     pgpDigParams sigp = rpmtsSignature(ts);
-    pgpDigParams pubp = rpmtsSignature(ts);
-    rpmRC res;
+    pgpDigParams pubp = rpmtsPubkey(ts);
+    rpmRC res = RPMRC_NOKEY;
+    const char * pubkeysource = NULL;
     int xx;
 
-    if (sig == NULL || dig == NULL || sigp == NULL || pubp == NULL) {
-	res = RPMRC_NOKEY;
+    if (sig == NULL || dig == NULL || sigp == NULL || pubp == NULL)
 	goto exit;
-    }
 
-    if (ts->pkpkt == NULL
-     || memcmp(sigp->signid, ts->pksignid, sizeof(ts->pksignid)))
-    {
-	int ix = -1;
-	rpmdbMatchIterator mi;
-	Header h;
+#if 0
+fprintf(stderr, "==> find sig id %08x %08x ts pubkey id %08x %08x\n",
+pgpGrab(sigp->signid, 4), pgpGrab(sigp->signid+4, 4),
+pgpGrab(ts->pksignid, 4), pgpGrab(ts->pksignid+4, 4));
+#endif
 
+    /* Lazy free of previous pubkey if pubkey does not match this signature. */
+    if (memcmp(sigp->signid, ts->pksignid, sizeof(ts->pksignid))) {
+#if 0
+fprintf(stderr, "*** free pkt %p[%d] id %08x %08x\n", ts->pkpkt, ts->pkpktlen, pgpGrab(ts->pksignid, 4), pgpGrab(ts->pksignid+4, 4));
+#endif
 	ts->pkpkt = _free(ts->pkpkt);
 	ts->pkpktlen = 0;
 	memset(ts->pksignid, 0, sizeof(ts->pksignid));
+    }
+
+    /* Try rpmdb keyring lookup. */
+    if (ts->pkpkt == NULL) {
+	int hx = -1;
+	int ix = -1;
+	rpmdbMatchIterator mi;
+	Header h;
 
 	/* Retrieve the pubkey that matches the signature. */
 	mi = rpmtsInitIterator(ts, RPMTAG_PUBKEYS, sigp->signid, sizeof(sigp->signid));
@@ -368,6 +379,7 @@ rpmRC rpmtsFindPubkey(rpmts ts)
 
 	    if (!headerGetEntry(h, RPMTAG_PUBKEYS, &pt, (void **)&pubkeys, &pc))
 		continue;
+	    hx = rpmdbGetIteratorOffset(mi);
 	    ix = rpmdbGetIteratorFileNum(mi);
 /*@-boundsread@*/
 	    if (ix >= pc
@@ -379,57 +391,61 @@ rpmRC rpmtsFindPubkey(rpmts ts)
 	}
 	mi = rpmdbFreeIterator(mi);
 
-	/* Was a matching pubkey found? */
-	if (ix < 0 || ts->pkpkt == NULL) {
-	    res = RPMRC_NOKEY;
-	    goto exit;
-	}
-
-	/*
-	 * Can the pubkey packets be parsed?
-	 * Do the parameters match the signature?
-	 */
-	if (pgpPrtPkts(ts->pkpkt, ts->pkpktlen, NULL, 0)
-	 && sigp->pubkey_algo == pubp->pubkey_algo
-#ifdef	NOTYET
-	 && sigp->hash_algo == pubp->hash_algo
-#endif
-	 && !memcmp(sigp->signid, pubp->signid, sizeof(sigp->signid)))
-	{
+/*@-branchstate@*/
+	if (ix >= 0) {
+	    char hnum[32];
+	    sprintf(hnum, "h#%d", hx);
+	    pubkeysource = xstrdup(hnum);
+	} else {
 	    ts->pkpkt = _free(ts->pkpkt);
 	    ts->pkpktlen = 0;
-	    res = RPMRC_NOKEY;
-	    goto exit;
 	}
+/*@=branchstate@*/
+    }
 
-	/* XXX Verify the pubkey signature. */
-
-	/* Packet looks good, save the signer id. */
-/*@-boundsread@*/
-	memcpy(ts->pksignid, sigp->signid, sizeof(ts->pksignid));
-/*@=boundsread@*/
-
-	rpmMessage(RPMMESS_DEBUG, "========== %s pubkey id %s\n",
-		(sigp->pubkey_algo == PGPPUBKEYALGO_DSA ? "DSA" :
-		(sigp->pubkey_algo == PGPPUBKEYALGO_RSA ? "RSA" : "???")),
-		pgpHexStr(sigp->signid, sizeof(sigp->signid)));
+    /* Try keyserver lookup. */
+    if (ts->pkpkt == NULL) {
+	const char * fn = rpmExpand("%{_hkp_keyserver_query}",
+			pgpHexStr(sigp->signid, sizeof(sigp->signid)), NULL);
 
+	xx = 0;
+	if (fn && *fn != '%') {
+	    xx = (pgpReadPkts(fn,&ts->pkpkt,&ts->pkpktlen) != PGPARMOR_PUBKEY);
+	}
+	fn = _free(fn);
+/*@-branchstate@*/
+	if (xx) {
+	    ts->pkpkt = _free(ts->pkpkt);
+	    ts->pkpktlen = 0;
+	} else {
+	    /* Save new pubkey in local ts keyring for delayed import. */
+	    pubkeysource = xstrdup("keyserver");
+	}
+/*@=branchstate@*/
     }
 
 #ifdef	NOTNOW
-    {
-	if (ts->pkpkt == NULL) {
-	    const char * pkfn = rpmExpand("%{_gpg_pubkey}", NULL);
-	    if (pgpReadPkts(pkfn, &ts->pkpkt, &ts->pkpktlen) != PGPARMOR_PUBKEY) {
-		pkfn = _free(pkfn);
-		res = RPMRC_NOKEY;
-		goto exit;
-	    }
-	    pkfn = _free(pkfn);
+    /* Try filename from macro lookup. */
+    if (ts->pkpkt == NULL) {
+	const char * fn = rpmExpand("%{_gpg_pubkey}", NULL);
+
+	xx = 0;
+	if (fn && *fn != '%')
+	    xx = (pgpReadPkts(fn,&ts->pkpkt,&ts->pkpktlen) != PGPARMOR_PUBKEY);
+	fn = _free(fn);
+	if (xx) {
+	    ts->pkpkt = _free(ts->pkpkt);
+	    ts->pkpktlen = 0;
+	} else {
+	    pubkeysource = xstrdup("macro");
 	}
     }
 #endif
 
+    /* Was a matching pubkey found? */
+    if (ts->pkpkt == NULL || ts->pkpktlen == 0)
+	goto exit;
+
     /* Retrieve parameters from pubkey packet(s). */
     xx = pgpPrtPkts(ts->pkpkt, ts->pkpktlen, dig, 0);
 
@@ -439,13 +455,31 @@ rpmRC rpmtsFindPubkey(rpmts ts)
      && sigp->hash_algo == pubp->hash_algo
 #endif
      &&	!memcmp(sigp->signid, pubp->signid, sizeof(sigp->signid)) )
-	res = RPMRC_OK;
-    else
-	res = RPMRC_NOKEY;
+    {
 
-    /* XXX Verify the signature signature. */
+	/* XXX Verify any pubkey signatures. */
+
+	/* Pubkey packet looks good, save the signer id. */
+/*@-boundsread@*/
+	memcpy(ts->pksignid, pubp->signid, sizeof(ts->pksignid));
+/*@=boundsread@*/
+
+	if (pubkeysource)
+	    rpmMessage(RPMMESS_DEBUG, "========== %s pubkey id %08x %08x (%s)\n",
+		(sigp->pubkey_algo == PGPPUBKEYALGO_DSA ? "DSA" :
+		(sigp->pubkey_algo == PGPPUBKEYALGO_RSA ? "RSA" : "???")),
+		pgpGrab(sigp->signid, 4), pgpGrab(sigp->signid+4, 4),
+		pubkeysource);
+
+	res = RPMRC_OK;
+    }
 
 exit:
+    pubkeysource = _free(pubkeysource);
+    if (res != RPMRC_OK) {
+	ts->pkpkt = _free(ts->pkpkt);
+	ts->pkpktlen = 0;
+    }
     return res;
 }
 
@@ -767,6 +801,8 @@ void rpmtsEmpty(rpmts ts)
     pi = rpmtsiFree(pi);
 
     ts->orderCount = 0;
+    ts->ntrees = 0;
+    ts->maxDepth = 0;
 
     ts->numRemovedPackages = 0;
 /*@-nullstate@*/	/* FIX: partial annotations */
@@ -856,6 +892,11 @@ rpmts rpmtsFree(rpmts ts)
     if (_rpmts_stats)
 	rpmtsPrintStats(ts);
 
+    /* Free up the memory used by the rpmtsScore */
+/*@-kepttrans -onlytrans @*/
+    ts->score = rpmtsScoreFree(ts->score);
+/*@=kepttrans =onlytrans @*/
+
     (void) rpmtsUnlink(ts, "tsCreate");
 
     /*@-refcounttrans -usereleased @*/
@@ -883,6 +924,36 @@ rpmVSFlags rpmtsSetVSFlags(rpmts ts, rpmVSFlags vsflags)
     return ovsflags;
 }
 
+/* 
+ * This allows us to mark transactions as being of a certain type.
+ * The three types are:
+ * 
+ *     RPM_TRANS_NORMAL 	
+ *     RPM_TRANS_ROLLBACK
+ *     RPM_TRANS_AUTOROLLBACK
+ * 
+ * ROLLBACK and AUTOROLLBACK transactions should always be ran as
+ * a best effort.  In particular this is important to the autorollback 
+ * feature to avoid rolling back a rollback (otherwise known as 
+ * dueling rollbacks (-;).  AUTOROLLBACK's additionally need instance 
+ * counts passed to scriptlets to be altered.
+ */
+void rpmtsSetType(rpmts ts, rpmtsType type)
+{
+    if (ts != NULL) {
+	ts->type = type;
+    }	 
+}
+
+/* Let them know what type of transaction we are */
+rpmtsType rpmtsGetType(rpmts ts) 
+{
+    if (ts != NULL) 
+	return ts->type;
+    else
+	return 0;
+}
+
 int rpmtsUnorderedSuccessors(rpmts ts, int first)
 {
     int unorderedSuccessors = 0;
@@ -896,7 +967,27 @@ int rpmtsUnorderedSuccessors(rpmts ts, int first)
 
 const char * rpmtsRootDir(rpmts ts)
 {
-    return (ts != NULL ? ts->rootDir : NULL);
+    const char * rootDir = NULL;
+
+/*@-branchstate@*/
+    if (ts != NULL && ts->rootDir != NULL) {
+	urltype ut = urlPath(ts->rootDir, &rootDir);
+	switch (ut) {
+	case URL_IS_UNKNOWN:
+	case URL_IS_PATH:
+	    break;
+	case URL_IS_HTTPS:
+	case URL_IS_HTTP:
+	case URL_IS_HKP:
+	case URL_IS_FTP:
+	case URL_IS_DASH:
+	default:
+	    rootDir = "/";
+	    break;
+	}
+    }
+/*@=branchstate@*/
+    return rootDir;
 }
 
 void rpmtsSetRootDir(rpmts ts, const char * rootDir)
@@ -1129,7 +1220,7 @@ int rpmtsInitDSI(const rpmts ts)
 
     rpmMessage(RPMMESS_DEBUG, _("mounted filesystems:\n"));
     rpmMessage(RPMMESS_DEBUG,
-	_("    i    dev bsize       bavail       iavail mount point\n"));
+	_("    i        dev    bsize       bavail       iavail mount point\n"));
 
     rc = rpmGetFilesystemList(&ts->filesystems, &ts->filesystemCount);
     if (rc || ts->filesystems == NULL || ts->filesystemCount <= 0)
@@ -1185,7 +1276,7 @@ int rpmtsInitDSI(const rpmts ts)
 	/* XXX Avoid FAT and other file systems that have not inodes. */
 	dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
 				? sfb.f_ffree : -1;
-	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%04x %5u %12ld %12ld %s\n"),
+	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%08x %8u %12ld %12ld %s\n"),
 		i, (unsigned) dsi->dev, (unsigned) dsi->bsize,
 		(signed long) dsi->bavail, (signed long) dsi->iavail,
 		ts->filesystems[i]);
@@ -1442,6 +1533,7 @@ rpmts rpmtsCreate(void)
     ts = xcalloc(1, sizeof(*ts));
     memset(&ts->ops, 0, sizeof(ts->ops));
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_TOTAL), -1);
+    ts->type = RPMTRANS_TYPE_NORMAL;
     ts->goal = TSM_UNKNOWN;
     ts->filesystemCount = 0;
     ts->filesystems = NULL;
@@ -1483,6 +1575,8 @@ rpmts rpmtsCreate(void)
     ts->orderAlloced = 0;
     ts->orderCount = 0;
     ts->order = NULL;
+    ts->ntrees = 0;
+    ts->maxDepth = 0;
 
     ts->probs = NULL;
 
@@ -1492,7 +1586,193 @@ rpmts rpmtsCreate(void)
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
     ts->dig = NULL;
 
+    /* 
+     * We only use the score in an autorollback.  So set this to
+     * NULL by default.
+     */
+    ts->score = NULL;
+
     ts->nrefs = 0;
 
     return rpmtsLink(ts, "tsCreate");
 }
+
+/**********************
+ * Transaction Scores *
+ **********************/
+
+
+rpmRC rpmtsScoreInit(rpmts runningTS, rpmts rollbackTS) 
+{
+    rpmtsScore score;
+    rpmtsi     pi;
+    rpmte      p;
+    int        i;
+    int        tranElements;  /* Number of transaction elements in runningTS */
+    int        found = 0;
+    rpmRC      rc = RPMRC_OK; /* Assume success */
+    rpmtsScoreEntry se;
+
+    rpmMessage(RPMMESS_DEBUG, _("Creating transaction score board(%p, %p)\n"),
+	runningTS, rollbackTS); 
+
+    /* Allocate space for score board */
+    score = xcalloc(1, sizeof(*score));
+    rpmMessage(RPMMESS_DEBUG, _("\tScore board address:  %p\n"), score);
+
+    /* 
+     * Determine the maximum size needed for the entry list.
+     * XXX: Today, I just get the count of rpmts elements, and allocate
+     *      an array that big.  Yes this is guaranteed to waste memory.
+     *      Future updates will hopefully make this more efficient,
+     *      but for now it will work.
+     */
+    tranElements  = rpmtsNElements(runningTS);
+    rpmMessage(RPMMESS_DEBUG, _("\tAllocating space for %d entries\n"), tranElements);
+    score->scores = xcalloc(tranElements, sizeof(score->scores));
+
+    /* Initialize score entry count */
+    score->entries = 0;
+    score->nrefs   = 0;
+
+    /*
+     * Increment through transaction elements and make sure for every 
+     * N there is an rpmtsScoreEntry.
+     */
+    pi = rpmtsiInit(runningTS); 
+    while ((p = rpmtsiNext(pi, TR_ADDED|TR_REMOVED)) != NULL) {
+	found  = 0;
+
+	/* Try to find the entry in the score list */
+	for(i = 0; i < score->entries; i++) {
+	    se = score->scores[i]; 
+  	    if (strcmp(rpmteN(p), se->N) == 0) {
+		found = 1;
+	 	/*@innerbreak@*/ break;
+	    }
+	}
+
+	/* If we did not find the entry then allocate space for it */
+	if (!found) {
+/*@-compdef -usereleased@*/ /* XXX p->fi->te undefined. */
+    	    rpmMessage(RPMMESS_DEBUG, _("\tAdding entry for %s to score board.\n"),
+		rpmteN(p));
+/*@=compdef =usereleased@*/
+    	    se = xcalloc(1, sizeof(*(*(score->scores))));
+    	    rpmMessage(RPMMESS_DEBUG, _("\t\tEntry address:  %p\n"), se);
+	    se->N         = xstrdup(rpmteN(p));
+	    se->te_types  = rpmteType(p); 
+	    se->installed = 0;
+	    se->erased    = 0; 
+	    score->scores[score->entries] = se;
+	    score->entries++;
+	} else {
+	    /* We found this one, so just add the element type to the one 
+	     * already there.
+	     */
+    	    rpmMessage(RPMMESS_DEBUG, _("\tUpdating entry for %s in score board.\n"),
+		rpmteN(p));
+	    score->scores[i]->te_types |= rpmteType(p);
+	}
+	 
+    }
+    pi = rpmtsiFree(pi);
+ 
+    /* 
+     * Attach the score to the running transaction and the autorollback
+     * transaction.
+     */
+    runningTS->score  = score;
+    score->nrefs++;
+    rollbackTS->score = score;
+    score->nrefs++;
+
+    return rc;
+}
+
+rpmtsScore rpmtsScoreFree(rpmtsScore score) 
+{
+    rpmtsScoreEntry se = NULL;
+    int i;
+
+    rpmMessage(RPMMESS_DEBUG, _("May free Score board(%p)\n"), score);
+
+    /* If score is not initialized, then just return.
+     * This is likely the case if autorollbacks are not enabled.
+     */
+    if (score == NULL) return NULL;
+
+    /* Decrement the reference count */
+    score->nrefs--;
+
+    /* Do we have any more references?  If so
+     * just return.
+     */
+    if (score->nrefs > 0) return NULL;
+
+    rpmMessage(RPMMESS_DEBUG, _("\tRefcount is zero...will free\n"));
+    /* No more references, lets clean up  */
+    /* First deallocate the score entries */
+/*@-branchstate@*/
+    for(i = 0; i < score->entries; i++) {
+	/* Get the score for the ith entry */
+	se = score->scores[i]; 
+	
+	/* Deallocate the score entries name */
+	se->N = _free(se->N);
+
+	/* Deallocate the score entry itself */
+	se = _free(se);
+    }
+/*@=branchstate@*/
+
+    /* Next deallocate the score entry table */
+    score->scores = _free(score->scores);
+
+    /* Finally deallocate the score itself */
+    score = _free(score);
+
+    return NULL;
+}
+
+/* 
+ * XXX: Do not get the score and then store it aside for later use.
+ *      we will delete it out from under you.  There is no rpmtsScoreLink()
+ *      as this may be a very temporary fix for autorollbacks.
+ */
+rpmtsScore rpmtsGetScore(rpmts ts) 
+{
+    if (ts == NULL) return NULL;
+    return ts->score;
+}
+
+/* 
+ * XXX: Do not get the score entry and then store it aside for later use.
+ *      we will delete it out from under you.  There is no 
+ *      rpmtsScoreEntryLink() as this may be a very temporary fix 
+ *      for autorollbacks.
+ * XXX: The scores are not sorted.  This should be fixed at earliest
+ *      opportunity (i.e. when we have the whole autorollback working).
+ */
+rpmtsScoreEntry rpmtsScoreGetEntry(rpmtsScore score, const char *N) 
+{
+    int i;
+    rpmtsScoreEntry se;
+    rpmtsScoreEntry ret = NULL; /* Assume we don't find it */
+
+    rpmMessage(RPMMESS_DEBUG, _("Looking in score board(%p) for %s\n"), score, N);
+
+    /* Try to find the entry in the score list */
+    for(i = 0; i < score->entries; i++) {
+	se = score->scores[i]; 
+ 	if (strcmp(N, se->N) == 0) {
+    	    rpmMessage(RPMMESS_DEBUG, _("\tFound entry at address:  %p\n"), se);
+	    ret = se;
+	    break;
+	}
+    }
+	
+/*@-compdef@*/ /* XXX score->scores undefined. */
+    return ret;	
+/*@=compdef@*/
+}
