@@ -26,7 +26,7 @@
 /*@access pgpDig@*/
 /*@access pgpDigParams@*/
 
-#if !defined(__GLIBC__)
+#if !defined(__GLIBC__) && !defined(__APPLE__)
 char ** environ = NULL;
 #endif
 
@@ -124,8 +124,15 @@ static inline rpmRC printSize(FD_t fd, int siglen, int pad, int datalen)
 	/*@modifies fileSystem @*/
 {
     struct stat st;
+    int fdno = Fileno(fd);
 
-    if (fstat(Fileno(fd), &st) < 0)
+    /* HACK: workaround for davRead wiring. */
+    if (fdno == 123456789) {
+	st.st_size = 0;
+/*@-sizeoftype@*/
+	st.st_size -= sizeof(struct rpmlead)+siglen+pad+datalen;
+/*@=sizeoftype@*/
+    } else if (fstat(fdno, &st) < 0)
 	return RPMRC_FAIL;
 
     /*@-sizeoftype@*/
@@ -314,8 +321,12 @@ rpmRC rpmReadSignature(FD_t fd, Header * sighp, sigType sig_type,
 	}
 
 	/* Print package component sizes. */
-	if (headerGetEntry(sigh, RPMSIGTAG_SIZE, NULL,(void **)&archSize, NULL))
+	if (headerGetEntry(sigh, RPMSIGTAG_SIZE, NULL,(void **)&archSize, NULL)) {
 	    rc = printSize(fd, sigSize, pad, *archSize);
+	    if (rc != RPMRC_OK)
+		(void) snprintf(buf, sizeof(buf),
+			_("sigh sigSize(%d): BAD, fstat(2) failed\n"), sigSize);
+	}
     }
 
 exit:
@@ -1025,6 +1036,7 @@ verifyMD5Signature(const rpmts ts, /*@out@*/ char * t,
     (void) rpmDigestFinal(rpmDigestDup(md5ctx),
 		(void **)&md5sum, &md5len, 0);
     (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), 0);
+    rpmtsOp(ts, RPMTS_OP_DIGEST)->count--;	/* XXX one too many */
 
     if (md5len != siglen || memcmp(md5sum, sig, md5len)) {
 	res = RPMRC_FAIL;
@@ -1187,6 +1199,7 @@ verifyPGPSignature(rpmts ts, /*@out@*/ char * t,
 
 	xx = rpmDigestFinal(ctx, (void **)&dig->md5, &dig->md5len, 1);
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), sigp->hashlen);
+	rpmtsOp(ts, RPMTS_OP_DIGEST)->count--;	/* XXX one too many */
 
 	/* Compare leading 16 bits of digest for quick check. */
 	s = dig->md5;
@@ -1214,7 +1227,7 @@ verifyPGPSignature(rpmts ts, /*@out@*/ char * t,
 	tt = stpcpy(tt, prefix);
 	tt = stpcpy(tt, dig->md5);
 
-	mpnzero(&dig->rsahm);	mpnsethex(&dig->rsahm, hexstr);
+	mpnzero(&dig->rsahm);	(void) mpnsethex(&dig->rsahm, hexstr);
 
 	hexstr = _free(hexstr);
 
@@ -1276,7 +1289,10 @@ verifyGPGSignature(rpmts ts, /*@out@*/ char * t,
     *t = '\0';
     if (dig != NULL && dig->hdrsha1ctx == sha1ctx)
 	t = stpcpy(t, _("Header "));
-    t = stpcpy(t, _("V3 DSA signature: "));
+    if (sigp->version == 4) 
+	t = stpcpy(t, _("V4 DSA signature: "));
+    else
+	t = stpcpy(t, _("V3 DSA signature: "));
 
     if (sha1ctx == NULL || sig == NULL || dig == NULL || sigp == NULL) {
 	res = RPMRC_NOKEY;
@@ -1299,21 +1315,20 @@ verifyGPGSignature(rpmts ts, /*@out@*/ char * t,
 	if (sigp->hash != NULL)
 	    xx = rpmDigestUpdate(ctx, sigp->hash, sigp->hashlen);
 
-#ifdef	NOTYET	/* XXX not for binary/text document signatures. */
-	if (sigp->sigtype == 4) {
-	    int nb = dig->nbytes + sigp->hashlen;
+	if (sigp->version == 4) {
+	    int nb = sigp->hashlen;
 	    byte trailer[6];
 	    nb = htonl(nb);
-	    trailer[0] = 0x4;
+	    trailer[0] = sigp->version;
 	    trailer[1] = 0xff;
 	    memcpy(trailer+2, &nb, sizeof(nb));
 	    xx = rpmDigestUpdate(ctx, trailer, sizeof(trailer));
 	}
-#endif
 	xx = rpmDigestFinal(ctx, (void **)&dig->sha1, &dig->sha1len, 1);
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), sigp->hashlen);
+	rpmtsOp(ts, RPMTS_OP_DIGEST)->count--;	/* XXX one too many */
 
-	mpnzero(&dig->hm);	mpnsethex(&dig->hm, dig->sha1);
+	mpnzero(&dig->hm);	(void) mpnsethex(&dig->hm, dig->sha1);
 
 	/* Compare leading 16 bits of digest for quick check. */
 	signhash16[0] = (*dig->hm.data >> 24) & 0xff;
