@@ -32,6 +32,13 @@
 
 #include "debug.h"
 
+/*
+ * This is needed for the IDTX definitions.  I think probably those need
+ * to be moved into a different source file (idtx.{c,h}), but that is up
+ * to Jeff Johnson.
+ */
+#include "rpmcli.h"
+
 /*@access Header @*/		/* XXX ts->notify arg1 is void ptr */
 /*@access rpmps @*/	/* XXX need rpmProblemSetOK() */
 /*@access dbiIndexSet @*/
@@ -47,6 +54,22 @@
 /*@access rpmtsi @*/
 /*@access rpmts @*/
 
+/*@access IDT @*/
+/*@access IDTX @*/
+/*@access FD_t @*/
+
+/* XXX: This is a hack.  I needed a to setup a notify callback
+ * for the rollback transaction, but I did not want to create
+ * a header for rpminstall.c.
+ */
+extern void * rpmShowProgress(/*@null@*/ const void * arg,
+                        const rpmCallbackType what,
+                        const unsigned long amount,
+                        const unsigned long total,
+                        /*@null@*/ fnpyKey key,
+                        /*@null@*/ void * data)
+	/*@*/;
+
 /**
  */
 static int archOkay(/*@null@*/ const char * pkgArch)
@@ -231,7 +254,7 @@ static int handleRmvdInstalledFiles(const rpmts ts, rpmfi fi,
     Header h;
     const char * otherStates;
     int i, xx;
-   
+
     rpmdbMatchIterator mi;
 
     mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES,
@@ -424,7 +447,7 @@ static void handleOverlappedFiles(const rpmts ts,
     rpmps ps;
     const char * fn;
     int i, j;
-  
+
     ps = rpmtsProblems(ts);
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)
@@ -613,6 +636,8 @@ assert(otherFi != NULL);
 	    }
 		
 	    /* Here is a pre-existing modified config file that needs saving. */
+	    /* XXX avoid md5 on sparse /var/log/lastlog file. */
+	    if (strcmp(fn, "/var/log/lastlog"))
 	    {	char md5sum[50];
 		const unsigned char * MD5 = rpmfiMD5(fi);
 		if (!domd5(fn, md5sum, 0, NULL) && memcmp(MD5, md5sum, 16)) {
@@ -664,7 +689,7 @@ static int ensureOlder(rpmts ts,
     *t++ = '-';
     if (rpmteR(p) != NULL)	t = stpcpy(t, rpmteR(p));
 /*@=boundswrite@*/
-    
+
     req = rpmdsSingle(RPMTAG_REQUIRENAME, rpmteN(p), reqEVR, reqFlags);
     rc = rpmdsNVRMatchesDep(h, req, _rpmds_nopromote);
     req = rpmdsFree(req);
@@ -940,6 +965,427 @@ rpmfi rpmtsiFi(const rpmtsi tsi)
     /*@=compdef =refcounttrans =usereleased @*/
 }
 
+/**
+ * This is not a generalized function to be called from outside
+ * librpm.  It is called internally by rpmtsRun() to rollback
+ * a failed transaction.
+ * @param rollbackTransaction		rollback transaction
+ * @return 				RPMRC_OK, or RPMRC_FAIL
+ */
+static rpmRC _rpmtsRollback(rpmts rollbackTransaction)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rollbackTransaction,
+		rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    int    rc         = 0;
+    int    numAdded   = 0;
+    int    numRemoved = 0;
+    int_32 tid;
+    rpmtsi tsi;
+    rpmte  te;
+    rpmps  ps;
+
+    /*
+     * Gather information about this rollback transaction for reporting.
+     *    1) Get tid
+     */
+    tid = rpmtsGetTid(rollbackTransaction);
+
+    /*
+     *    2) Get number of install elments and erase elements
+     */
+    tsi = rpmtsiInit(rollbackTransaction);
+    while((te = rpmtsiNext(tsi, 0)) != NULL) {
+	switch (rpmteType(te)) {
+	case TR_ADDED:
+	   numAdded++;
+	   /*@switchbreak@*/ break;
+	case TR_REMOVED:
+	   numRemoved++;
+	   /*@switchbreak@*/ break;
+	default:
+	   /*@switchbreak@*/ break;
+	}	
+    }
+    tsi = rpmtsiFree(tsi);
+
+    rpmMessage(RPMMESS_NORMAL, _("Transaction failed...rolling back\n"));
+    rpmMessage(RPMMESS_NORMAL,
+	_("Rollback packages (+%d/-%d) to %-24.24s (0x%08x):\n"),
+                        numAdded, numRemoved, ctime(&tid), tid);
+
+    /* Check the transaction to see if it is doable */
+    rc = rpmtsCheck(rollbackTransaction);
+    ps = rpmtsProblems(rollbackTransaction);
+    if (rc != 0 && rpmpsNumProblems(ps) > 0) {
+	rpmMessage(RPMMESS_ERROR, _("Failed dependencies:\n"));
+	rpmpsPrint(NULL, ps);
+	ps = rpmpsFree(ps);
+	return -1;
+    }
+    ps = rpmpsFree(ps);
+
+    /* Order the transaction */
+    rc = rpmtsOrder(rollbackTransaction);
+    if (rc != 0) {
+	rpmMessage(RPMMESS_ERROR,
+	    _("Could not order auto-rollback transaction!\n"));
+       return -1;
+    }
+
+
+
+    /* Run the transaction and print any problems
+     * We want to stay with the original transactions flags except
+     * that we want to add what is essentially a force.
+     * This handles two things in particular:
+     *	
+     *	1.  We we want to upgrade over a newer package.
+     * 	2.  If a header for the old package is there we
+     *      we want to replace it.  No questions asked.
+     */
+    rc = rpmtsRun(rollbackTransaction, NULL,
+	  RPMPROB_FILTER_REPLACEPKG
+	| RPMPROB_FILTER_REPLACEOLDFILES
+	| RPMPROB_FILTER_REPLACENEWFILES
+	| RPMPROB_FILTER_OLDPACKAGE
+    );
+    ps = rpmtsProblems(rollbackTransaction);
+    if (rc > 0 && rpmpsNumProblems(ps) > 0)
+	rpmpsPrint(stderr, ps);
+    ps = rpmpsFree(ps);
+
+    /*
+     * After we have ran through the transaction we need to
+     * remove any repackaged packages we just installed/upgraded
+     * from the rp repository.
+     */
+    tsi = rpmtsiInit(rollbackTransaction);
+    while((te = rpmtsiNext(tsi, 0)) != NULL) {
+	rpmMessage(RPMMESS_NORMAL, _("Cleaning up repackaged packages:\n"));
+	switch (rpmteType(te)) {
+	/* The install elements are repackaged packages */
+	case TR_ADDED:
+	    /* Make sure the filename is still there.  XXX: Can't happen */
+	    if(te->key) {
+		rpmMessage(RPMMESS_NORMAL, _("\tRemoving %s:\n"), te->key);
+		(void) unlink(te->key); /* XXX: Should check for an error? */
+	    }
+	    /*@switchbreak@*/ break;
+                                                                                
+	/* Ignore erase elements...nothing to do */
+	default:
+	    /*@switchbreak@*/ break;
+	}
+    }
+    tsi = rpmtsiFree(tsi);
+
+    /* Free the rollback transaction */
+    rollbackTransaction = rpmtsFree(rollbackTransaction);
+
+    return rc;
+}
+
+/**
+ * Get the repackaged header and filename from the repackage directory.
+ * @todo Find a suitable home for this function.
+ * @todo This function creates an IDTX everytime it is called.  Needs to
+ *       be made more efficient (only create on per running transaction).
+ * @param ts		rpm transaction
+ * @param te		transaction element
+ * @retval hdrp		Repackaged header
+ * @retval fnp		Repackaged package's path (transaction key)
+ * @return 		RPMRC_NOTFOUND or RPMRC_OK
+ */
+static rpmRC getRepackageHeaderFromTE(rpmts ts, rpmte te,
+		/*@out@*/ /*@null@*/ Header *hdrp,
+		/*@out@*/ /*@null@*/ const char **fnp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, *hdrp, *fnp,
+		rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    int_32 tid;
+    const char * name;
+    const char * rpname = NULL;
+    const char * _repackage_dir = NULL;
+    const char * globStr = "-*.rpm";
+    char * rp = NULL;		/* Rollback package name */
+    IDTX rtids = NULL;
+    IDT rpIDT;
+    int nrids = 0;
+    int nb;			/* Number of bytes */
+    Header h = NULL;
+    int rc   = RPMRC_NOTFOUND;	/* Assume we do not find it*/
+    int xx;
+
+    rpmMessage(RPMMESS_DEBUG,
+	_("Getting repackaged header from transaction element\n"));
+
+    /* Set header pointer to null if its not already */
+    if (hdrp)
+	*hdrp = NULL;
+    if (fnp)
+	*fnp = NULL;
+
+    /* Get the TID of the current transaction */
+    tid = rpmtsGetTid(ts);
+    /* Need the repackage dir if the user want to
+     * rollback on a failure.
+     */
+    _repackage_dir = rpmExpand("%{?_repackage_dir}", NULL);
+    if (_repackage_dir == NULL) goto exit;
+
+    /* Build the glob string to find the possible repackaged
+     * packages for this package.
+     */
+    name = rpmteN(te);	
+    nb = strlen(_repackage_dir) + strlen(name) + strlen(globStr) + 2;
+    rp = memset((char *) malloc(nb), 0, nb);
+    xx = snprintf(rp, nb, "%s/%s%s.rpm", _repackage_dir, name, globStr);
+
+    /* Get the index of possible repackaged packages */
+    rpmMessage(RPMMESS_DEBUG, _("\tLooking for %s...\n"), rp);
+    rtids = IDTXglob(ts, rp, RPMTAG_REMOVETID);
+    rp = _free(rp);
+    if (rtids != NULL) {
+    	rpmMessage(RPMMESS_DEBUG, _("\tMatches found.\n"));
+	rpIDT = rtids->idt;
+	nrids = rtids->nidt;
+    } else {
+    	rpmMessage(RPMMESS_DEBUG, _("\tNo matches found.\n"));
+	goto exit;
+    }
+
+    /* Now walk through index until we find the package (or we have
+     * exhausted the index.
+     */
+/*@-branchstate@*/
+    do {
+	/* If index is null we have exhausted the list and need to
+	 * get out of here...the repackaged package was not found.
+	 */
+	if (rpIDT == NULL) {
+    	    rpmMessage(RPMMESS_DEBUG, _("\tRepackaged package not found!.\n"));
+	    break;
+	}
+
+	/* Is this the same tid.  If not decrement the list and continue */
+	if (rpIDT->val.u32 != tid) {
+	    nrids--;
+	    if (nrids > 0)
+		rpIDT++;
+	    else
+		rpIDT = NULL;
+	    continue;
+	}
+
+	/* OK, the tid matches.  Now lets see if the name is the same.
+	 * If I could not get the name from the package, I will go onto
+	 * the next one.  Perhaps I should return an error at this
+	 * point, but if this was not the correct one, at least the correct one
+	 * would be found.
+	 * XXX:  Should Match NAC!
+	 */
+    	rpmMessage(RPMMESS_DEBUG, _("\tREMOVETID matched INSTALLTID.\n"));
+	if (headerGetEntry(rpIDT->h, RPMTAG_NAME, NULL, (void **) &rpname, NULL)) {
+    	    rpmMessage(RPMMESS_DEBUG, _("\t\tName:  %s.\n"), rpname);
+	    if (!strcmp(name,rpname)) {
+		/* It matched we have a canidate */
+		h  = headerLink(rpIDT->h);
+		nb = strlen(rpIDT->key) + 1;
+		rp = memset((char *) malloc(nb), 0, nb);
+		rp = strncat(rp, rpIDT->key, nb);
+		rc = RPMRC_OK;
+		break;
+	    }
+	}
+
+	/* Decrement list */	
+	nrids--;
+	if (nrids > 0)
+	    rpIDT++;
+	else
+	    rpIDT = NULL;
+    } while (1);
+/*@=branchstate@*/
+
+exit:
+    if (rc != RPMRC_NOTFOUND && h != NULL && hdrp != NULL) {
+    	rpmMessage(RPMMESS_DEBUG, _("\tRepackaged Package was %s...\n"), rp);
+	if (hdrp != NULL)
+	    *hdrp = headerLink(h);
+/*@-branchstate@*/
+	if (fnp != NULL)
+	    *fnp = rp;
+	else
+	    rp = _free(rp);
+/*@=branchstate@*/
+    }
+    if (h != NULL)
+	h = headerFree(h);
+    rtids = IDTXfree(rtids);
+    return rc;	
+}
+
+/**
+ * This is not a generalized function to be called from outside
+ * librpm.  It is called internally by rpmtsRun() to add elements
+ * to its autorollback transaction.
+ * @param rollbackTransaction		rollback transaction
+ * @param runningTransaction		running transaction (the one you want to rollback)
+ * @param te 				Transaction element.
+ * @return 				RPMRC_OK, or RPMRC_FAIL
+ */
+static rpmRC _rpmtsAddRollbackElement(rpmts rollbackTransaction,
+		rpmts runningTransaction, rpmte te)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rollbackTransaction, runningTransaction,
+		rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    Header h   = NULL;
+    Header rph = NULL;
+    char * rpn;	
+    unsigned int db_instance = 0;
+    rpmtsi pi;  	
+    rpmte p;
+    int rc  = RPMRC_FAIL;	/* Assume Failure */
+
+    switch(rpmteType(te)) {
+    case TR_ADDED:
+    {   rpmdbMatchIterator mi;
+
+	rpmMessage(RPMMESS_DEBUG,
+	    _("Adding install element to auto-rollback transaction.\n"));
+
+	/* Get the header for this package from the database
+	 * First get the database instance (the key).
+	 */
+	db_instance = rpmteDBInstance(te);
+	if (db_instance == 0) {
+	    /* Could not get the db instance: WTD! */
+    	    rpmMessage(RPMMESS_FATALERROR,
+		_("Could not get install element database instance!\n"));
+	    break;
+	}
+
+	/* Now suck the header out of the database */
+	mi = rpmtsInitIterator(rollbackTransaction,
+	    RPMDBI_PACKAGES, &db_instance, sizeof(db_instance));
+	h = rpmdbNextIterator(mi);
+	if (h != NULL) h = headerLink(h);
+	mi = rpmdbFreeIterator(mi);
+	if (h == NULL) {
+	    /* Header was not there??? */
+    	    rpmMessage(RPMMESS_FATALERROR,
+		_("Could not get header for auto-rollback transaction!\n"));
+	    break;
+	}
+
+	/* Now see if there is a repackaged package for this */
+	rc = getRepackageHeaderFromTE(runningTransaction, te, &rph, &rpn);
+	switch(rc) {
+	case RPMRC_OK:
+	    /* Add the install element, as we had a repackaged package */
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tAdded repackaged package header: %s.\n"), rpn);
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tAdded from install element %s.\n"), rpmteNEVRA(te));
+	    rc = rpmtsAddInstallElement(rollbackTransaction, headerLink(rph),
+		(fnpyKey) rpn, 1, te->relocs);
+	    /*@innerbreak@*/ break;
+
+	case RPMRC_NOTFOUND:
+	    /* Add the header as an erase element, we did not
+	     * have a repackaged package
+	     */
+	    rpmMessage(RPMMESS_DEBUG, _("\tAdded erase element.\n"));
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tAdded from install element %s.\n"), rpmteNEVRA(te));
+	    rc = rpmtsAddEraseElement(rollbackTransaction, h, db_instance);
+	    /*@innerbreak@*/ break;
+			
+	default:
+	    /* Not sure what to do on failure...just give up */
+   	    rpmMessage(RPMMESS_FATALERROR,
+		_("Could not get repackaged header for auto-rollback transaction!\n"));
+	    /*@innerbreak@*/ break;
+	}
+    }	break;
+
+   case TR_REMOVED:
+	rpmMessage(RPMMESS_DEBUG,
+	    _("Add erase element to auto-rollback transaction.\n"));
+	/* See if this element has already been added.
+ 	 * If so we want to do nothing.  Compare N's for match.
+	 * XXX:  Really should compare NAC's.
+	 */
+	pi = rpmtsiInit(rollbackTransaction);
+	while ((p = rpmtsiNext(pi, 0)) != NULL) {
+	    if (!strcmp(rpmteN(p), rpmteN(te))) {
+	    	rpmMessage(RPMMESS_DEBUG, _("\tFound existing upgrade element.\n"));
+	    	rpmMessage(RPMMESS_DEBUG, _("\tNot adding erase element for %s.\n"),
+			rpmteN(te));
+		rc = RPMRC_OK;	
+		pi = rpmtsiFree(pi);
+		goto cleanup;
+	    }
+	}
+	pi = rpmtsiFree(pi);
+
+	/* Get the repackage header from the current transaction
+	* element.
+	*/
+	rc = getRepackageHeaderFromTE(runningTransaction, te, &rph, &rpn);
+	switch(rc) {
+	case RPMRC_OK:
+	    /* Add the install element */
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tAdded repackaged package %s.\n"), rpn);
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tAdded from erase element %s.\n"), rpmteNEVRA(te));
+	    rc = rpmtsAddInstallElement(rollbackTransaction, rph,
+		(fnpyKey) rpn, 1, te->relocs);
+	    if (rc != RPMRC_OK)
+	        rpmMessage(RPMMESS_FATALERROR,
+		    _("Could not add erase element to auto-rollback transaction.\n"));
+	    /*@innerbreak@*/ break;
+
+	case RPMRC_NOTFOUND:
+	    /* Just did not have a repackaged package */
+	    rpmMessage(RPMMESS_DEBUG,
+		_("\tNo repackaged package...nothing to do.\n"));
+	    rc = RPMRC_OK;
+	    /*@innerbreak@*/ break;
+
+	default:
+	    rpmMessage(RPMMESS_FATALERROR,
+		_("Failure reading repackaged package!\n"));
+	    /*@innerbreak@*/ break;
+	}
+	break;
+
+    default:
+	break;
+    }
+
+/* XXX:  I want to free this, but if I do then the consumers of
+ *       are hosed.  Just leaving you a little note Jeff, so you
+ *       know that this does introduce a memory leak.  I wanted
+ *       keep the patch as simple as possible so I am not fixxing
+ *       the leak.
+ *   if (rpn != NULL)
+ *	free(rpn);
+ */
+
+cleanup:
+    /* Clean up */
+    if (h != NULL)
+	h = headerFree(h);
+    if (rph != NULL)
+	rph = headerFree(rph);
+    return rc;
+}
+
 #define	NOTIFY(_ts, _al) /*@i@*/ if ((_ts)->notify) (void) (_ts)->notify _al
 
 int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
@@ -960,6 +1406,8 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     rpmtsi qi;	rpmte q;
     int numAdded;
     int numRemoved;
+    rpmts rollbackTransaction = NULL;
+    int rollbackOnFailure = 0;
     void * lock = NULL;
     int xx;
 
@@ -967,12 +1415,26 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     if (rpmtsNElements(ts) <= 0)
 	return -1;
 
+    /* See if we need to rollback on failure */
+    rollbackOnFailure = rpmExpandNumeric(
+	"%{?_rollback_transaction_on_failure}");
+    if (rpmtsGetType(ts) & (RPMTRANS_TYPE_ROLLBACK
+	| RPMTRANS_TYPE_AUTOROLLBACK)) {
+	rollbackOnFailure = 0;
+    }
+    /* If we are in test mode, there is no need to rollback on
+     * failure, nor acquire the transaction lock.
+     */
+/*@-branchstate@*/
     /* If we are in test mode, then there's no need for transaction lock. */
-    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)) {
+    if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST) {
+	rollbackOnFailure = 0;
+    } else {
 	lock = rpmtsAcquireLock(ts);
 	if (lock == NULL)
-	    return -1;
+	    return -1;	/* XXX W2DO? */
     }
+/*@=branchstate@*/
 
     if (rpmtsFlags(ts) & RPMTRANS_FLAG_NOSCRIPTS)
 	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | _noTransScripts | _noTransTriggers));
@@ -993,7 +1455,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 	if (rpmtsOpenDB(ts, dbmode)) {
 	    rpmtsFreeLock(lock);
 	    return -1;	/* XXX W2DO? */
-    }
+	}
     }
 
     ts->ignoreSet = ignoreSet;
@@ -1024,6 +1486,7 @@ rpmMessage(RPMMESS_DEBUG, _("sanity checking %d elements\n"), rpmtsNElements(ts)
     ps = rpmtsProblems(ts);
     /* The ordering doesn't matter here */
     pi = rpmtsiInit(ts);
+    /* XXX Only added packages need be checked. */
     while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
 	rpmdbMatchIterator mi;
 	int fc;
@@ -1099,6 +1562,77 @@ rpmMessage(RPMMESS_DEBUG, _("sanity checking %d elements\n"), rpmtsNElements(ts)
     }
     pi = rpmtsiFree(pi);
 
+
+    /* Run pre-transaction scripts, but only if there are no known
+     * problems up to this point. */
+    if (!((rpmtsFlags(ts) & RPMTRANS_FLAG_BUILD_PROBS)
+     	  || (ts->probs->numProblems &&
+		(okProbs == NULL || rpmpsTrim(ts->probs, okProbs))))) {
+	rpmMessage(RPMMESS_DEBUG, _("running pre-transaction scripts\n"));
+	pi = rpmtsiInit(ts);
+	while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
+	    if ((fi = rpmtsiFi(pi)) == NULL)
+		continue;	/* XXX can't happen */
+
+	    /* If no pre-transaction script, then don't bother. */
+	    if (fi->pretrans == NULL)
+		continue;
+
+	    p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
+			    rpmteKey(p), ts->notifyData);
+	    p->h = NULL;
+	    if (rpmteFd(p) != NULL) {
+		rpmVSFlags ovsflags = rpmtsVSFlags(ts);
+		rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
+		rpmRC rpmrc;
+		ovsflags = rpmtsSetVSFlags(ts, vsflags);
+		rpmrc = rpmReadPackageFile(ts, rpmteFd(p),
+			    rpmteNEVR(p), &p->h);
+		vsflags = rpmtsSetVSFlags(ts, ovsflags);
+		switch (rpmrc) {
+		default:
+		    /*@-noeffectuncon@*/ /* FIX: notify annotations */
+		    p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
+				    0, 0,
+				    rpmteKey(p), ts->notifyData);
+		    /*@=noeffectuncon@*/
+		    p->fd = NULL;
+		    /*@switchbreak@*/ break;
+		case RPMRC_NOTTRUSTED:
+		case RPMRC_NOKEY:
+		case RPMRC_OK:
+		    /*@switchbreak@*/ break;
+		}
+	    }
+
+/*@-branchstate@*/
+	    if (rpmteFd(p) != NULL) {
+		fi = rpmfiNew(ts, p->h, RPMTAG_BASENAMES, 1);
+		if (fi != NULL) {	/* XXX can't happen */
+		    fi->te = p;
+		    p->fi = fi;
+		}
+/*@-compdef -usereleased@*/	/* p->fi->te undefined */
+		psm = rpmpsmNew(ts, p, p->fi);
+/*@=compdef =usereleased@*/
+assert(psm != NULL);
+		psm->scriptTag = RPMTAG_PRETRANS;
+		psm->progTag = RPMTAG_PRETRANSPROG;
+		xx = rpmpsmStage(psm, PSM_SCRIPT);
+		psm = rpmpsmFree(psm);
+
+/*@-noeffectuncon -compdef -usereleased @*/
+		(void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
+				  rpmteKey(p), ts->notifyData);
+/*@=noeffectuncon =compdef =usereleased @*/
+		p->fd = NULL;
+		p->h = headerFree(p->h);
+	    }
+/*@=branchstate@*/
+	}
+	pi = rpmtsiFree(pi);
+    }
+
     /* ===============================================
      * Initialize transaction element file info for package:
      */
@@ -1143,7 +1677,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing %d file fingerprints\n"), totalFileCount)
 	const char * rootDir = rpmtsRootDir(ts);
 	xx = chdir("/");
 	/*@-superuser -noeffect @*/
-	if (rootDir != NULL)
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
 	    xx = chroot(rootDir);
 	/*@=superuser =noeffect @*/
 	(void) rpmtsSetChrootDone(ts, 1);
@@ -1315,9 +1849,11 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
     ps = rpmpsFree(ps);
 
     if (rpmtsChrootDone(ts)) {
+	const char * rootDir = rpmtsRootDir(ts);
 	const char * currDir = rpmtsCurrDir(ts);
 	/*@-superuser -noeffect @*/
-	xx = chroot(".");
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
+	    xx = chroot(".");
 	/*@=superuser =noeffect @*/
 	(void) rpmtsSetChrootDone(ts, 0);
 	if (currDir != NULL)
@@ -1348,13 +1884,63 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
      */
     if ((rpmtsFlags(ts) & RPMTRANS_FLAG_BUILD_PROBS)
      || (ts->probs->numProblems &&
-		(okProbs != NULL || rpmpsTrim(ts->probs, okProbs)))
+		(okProbs == NULL || rpmpsTrim(ts->probs, okProbs)))
        )
     {
 	rpmtsFreeLock(lock);
 	return ts->orderCount;
     }
 
+    /* ===============================================
+     * If we were requested to rollback this transaction
+     * if an error occurs, then we need to create a
+     * a rollback transaction.
+     */
+     if (rollbackOnFailure) {
+	rpmtransFlags tsFlags;
+	rpmVSFlags ovsflags;
+	rpmVSFlags vsflags;
+
+	rpmMessage(RPMMESS_DEBUG,
+	    _("Creating auto-rollback transaction\n"));
+
+	rollbackTransaction = rpmtsCreate();
+
+	/* Set the verify signature flags:
+	 * 	- can't verify digests on repackaged packages.  Other than
+	 *	  they are wrong, this will cause segfaults down stream.
+	 *	- signatures are out too.
+	 * 	- header check are out.
+	 */ 	
+	vsflags = rpmExpandNumeric("%{?_vsflags_erase}");
+	vsflags |= _RPMVSF_NODIGESTS;
+	vsflags |= _RPMVSF_NOSIGNATURES;
+	vsflags |= RPMVSF_NOHDRCHK;
+	vsflags |= RPMVSF_NEEDPAYLOAD;      /* XXX no legacy signatures */
+	ovsflags = rpmtsSetVSFlags(ts, vsflags);
+
+	/*
+	 *  If we run this thing its imperitive that it be known that it
+	 *  is an autorollback transaction.  This will affect the instance
+	 *  counts passed to the scriptlets in the psm.
+	 */
+	rpmtsSetType(rollbackTransaction, RPMTRANS_TYPE_AUTOROLLBACK);
+
+	/* Set transaction flags to be the same as the running transaction */
+	tsFlags = rpmtsSetFlags(rollbackTransaction, rpmtsFlags(ts));
+
+	/* Set root dir to be the same as the running transaction */
+	rpmtsSetRootDir(rollbackTransaction, rpmtsRootDir(ts));
+
+	/* Setup the notify of the call back to be the same as the running
+	 * transaction
+	 */
+	xx = rpmtsSetNotifyCallback(rollbackTransaction, ts->notify, ts->notifyData);
+
+	/* Create rpmtsScore for running transaction and rollback transaction */
+	xx = rpmtsScoreInit(ts, rollbackTransaction);
+     }
+
     /* ===============================================
      * Save removed files before erasing.
      */
@@ -1390,6 +1976,7 @@ rpmMessage(RPMMESS_DEBUG, _("computing file dispositions\n"));
 		fi->mapflags |= CPIO_MAP_ADDDOT;
 		fi->mapflags |= CPIO_ALL_HARDLINKS;
 		psm = rpmpsmNew(ts, p, fi);
+assert(psm != NULL);
 		xx = rpmpsmStage(psm, PSM_PKGSAVE);
 		psm = rpmpsmFree(psm);
 		fi->mapflags &= ~CPIO_MAP_ABSOLUTE;
@@ -1464,6 +2051,23 @@ assert(psm != NULL);
 			/*@=noeffectuncon@*/
 			p->fd = NULL;
 			ourrc++;
+
+			/* If we should rollback this transaction
+			   on failure, lets do it.                 */
+			if (rollbackOnFailure) {
+			    rpmMessage(RPMMESS_ERROR,
+				_("Add failed.  Could not read package header.\n"));
+			    /* Clean up the current transaction */
+			    p->h = headerFree(p->h);
+			    xx = rpmdbSync(rpmtsGetRdb(ts));
+			    psm = rpmpsmFree(psm);
+			    p->fi = rpmfiFree(p->fi);
+			    pi = rpmtsiFree(pi);
+
+			    /* Run the rollback transaction */
+			    xx = _rpmtsRollback(rollbackTransaction);
+			    return -1;
+			}
 			/*@innerbreak@*/ break;
 		    case RPMRC_NOTTRUSTED:
 		    case RPMRC_NOKEY:
@@ -1511,11 +2115,67 @@ assert(psm != NULL);
 		if (rpmpsmStage(psm, PSM_PKGINSTALL)) {
 		    ourrc++;
 		    lastFailKey = pkgKey;
+
+		    /* If we should rollback this transaction
+		       on failure, lets do it.                 */
+		    if (rollbackOnFailure) {
+			rpmMessage(RPMMESS_ERROR,
+			    _("Add failed in rpmpsmStage().\n"));
+			/* Clean up the current transaction */
+			p->h = headerFree(p->h);
+			xx = rpmdbSync(rpmtsGetRdb(ts));
+			psm = rpmpsmFree(psm);
+			p->fi = rpmfiFree(p->fi);
+			pi = rpmtsiFree(pi);
+
+			/* Run the rollback transaction */
+			xx = _rpmtsRollback(rollbackTransaction);
+			return -1;
+		    }
+		}
+		
+		/* If we should rollback on failure lets add
+		 * this element to the rollback transaction
+		 * as an erase element as it has installed succesfully.
+		 */
+		if (rollbackOnFailure) {
+		    int rc;
+
+		    rc = _rpmtsAddRollbackElement(rollbackTransaction, ts, p);
+		    if (rc != RPMRC_OK) {
+			/* Clean up the current transaction */
+			p->h = headerFree(p->h);
+			xx = rpmdbSync(rpmtsGetRdb(ts));
+			psm = rpmpsmFree(psm);
+			p->fi = rpmfiFree(p->fi);
+			pi = rpmtsiFree(pi);
+			
+			/* Clean up rollback transaction */
+			rollbackTransaction = rpmtsFree(rollbackTransaction);
+			return -1;
+		    }
 		}
 /*@=nullstate@*/
 	    } else {
 		ourrc++;
 		lastFailKey = pkgKey;
+		
+		/* If we should rollback this transaction
+		 * on failure, lets do it.
+		 */
+		if (rollbackOnFailure) {
+		    rpmMessage(RPMMESS_ERROR, _("Add failed.  Could not get file list.\n"));
+		    /* Clean up the current transaction */
+	    	    p->h = headerFree(p->h);
+		    xx = rpmdbSync(rpmtsGetRdb(ts));
+		    psm = rpmpsmFree(psm);
+		    p->fi = rpmfiFree(p->fi);
+		    pi = rpmtsiFree(pi);
+
+		    /* Run the rollback transaction */
+		    xx = _rpmtsRollback(rollbackTransaction);
+		    return -1;
+		}
 	    }
 
 	    if (gotfd) {
@@ -1545,8 +2205,48 @@ assert(psm != NULL);
 	     * If install failed, then we shouldn't erase.
 	     */
 	    if (rpmteDependsOnKey(p) != lastFailKey) {
-		if (rpmpsmStage(psm, PSM_PKGERASE))
+		if (rpmpsmStage(psm, PSM_PKGERASE)) {
 		    ourrc++;
+
+		    /* If we should rollback this transaction
+		     * on failure, lets do it.
+		     */
+		    if (rollbackOnFailure) {
+			rpmMessage(RPMMESS_ERROR,
+			    _("Erase failed failed in rpmpsmStage().\n"));
+			/* Clean up the current transaction */
+			xx = rpmdbSync(rpmtsGetRdb(ts));
+			psm = rpmpsmFree(psm);
+			p->fi = rpmfiFree(p->fi);
+			pi = rpmtsiFree(pi);
+
+			/* Run the rollback transaction */
+			xx = _rpmtsRollback(rollbackTransaction);
+			return -1;
+		    }
+		}
+
+		/* If we should rollback on failure lets add
+		 * this element to the rollback transaction
+		 * as an install element as it has erased succesfully.
+		 */
+		if (rollbackOnFailure) {
+		    int rc;
+
+		    rc = _rpmtsAddRollbackElement(rollbackTransaction, ts, p);
+
+		    if (rc != RPMRC_OK) {
+			/* Clean up the current transaction */
+			xx = rpmdbSync(rpmtsGetRdb(ts));
+			psm = rpmpsmFree(psm);
+			p->fi = rpmfiFree(p->fi);
+			pi = rpmtsiFree(pi);
+		
+			/* Clean up rollback transaction */
+			rollbackTransaction = rpmtsFree(rollbackTransaction);
+			return -1;
+		    }
+		}
 	    }
 
 	    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_ERASE), 0);
@@ -1559,14 +2259,83 @@ assert(psm != NULL);
 	psm = rpmpsmFree(psm);
 /*@=nullstate@*/
 
+#ifdef	DYING
 /*@-type@*/ /* FIX: p is almost opaque */
 	p->fi = rpmfiFree(p->fi);
 /*@=type@*/
+#endif
 
     }
     /*@=branchstate@*/
     pi = rpmtsiFree(pi);
 
+    /* If we created a rollback transaction lets get rid of it */
+    if (rollbackOnFailure && rollbackTransaction != NULL)
+	rollbackTransaction = rpmtsFree(rollbackTransaction);
+
+    rpmMessage(RPMMESS_DEBUG, _("running post-transaction scripts\n"));
+    pi = rpmtsiInit(ts);
+    while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
+	int haspostscript;
+
+	if ((fi = rpmtsiFi(pi)) == NULL)
+	    continue;	/* XXX can't happen */
+
+	haspostscript = (fi->posttrans != NULL ? 1 : 0);
+	p->fi = rpmfiFree(p->fi);
+
+	/* If no post-transaction script, then don't bother. */
+	if (haspostscript)
+	    continue;
+
+	p->fd = ts->notify(p->h, RPMCALLBACK_INST_OPEN_FILE, 0, 0,
+			rpmteKey(p), ts->notifyData);
+	p->h = NULL;
+	if (rpmteFd(p) != NULL) {
+	    rpmVSFlags ovsflags = rpmtsVSFlags(ts);
+	    rpmVSFlags vsflags = ovsflags | RPMVSF_NEEDPAYLOAD;
+	    rpmRC rpmrc;
+	    ovsflags = rpmtsSetVSFlags(ts, vsflags);
+	    rpmrc = rpmReadPackageFile(ts, rpmteFd(p),
+			rpmteNEVR(p), &p->h);
+	    vsflags = rpmtsSetVSFlags(ts, ovsflags);
+	    switch (rpmrc) {
+	    default:
+		p->fd = ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE,
+				0, 0, rpmteKey(p), ts->notifyData);
+		p->fd = NULL;
+		/*@switchbreak@*/ break;
+	    case RPMRC_NOTTRUSTED:
+	    case RPMRC_NOKEY:
+	    case RPMRC_OK:
+		/*@switchbreak@*/ break;
+	    }
+	}
+
+	if (rpmteFd(p) != NULL) {
+	    p->fi = rpmfiNew(ts, p->h, RPMTAG_BASENAMES, 1);
+	    if (p->fi != NULL)	/* XXX can't happen */
+		p->fi->te = p;
+/*@-compdef -usereleased@*/	/* p->fi->te undefined */
+	    psm = rpmpsmNew(ts, p, p->fi);
+/*@=compdef =usereleased@*/
+assert(psm != NULL);
+	    psm->scriptTag = RPMTAG_POSTTRANS;
+	    psm->progTag = RPMTAG_POSTTRANSPROG;
+	    xx = rpmpsmStage(psm, PSM_SCRIPT);
+	    psm = rpmpsmFree(psm);
+
+/*@-noeffectuncon -compdef -usereleased @*/
+	    (void) ts->notify(p->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
+			      rpmteKey(p), ts->notifyData);
+/*@=noeffectuncon =compdef =usereleased @*/
+	    p->fd = NULL;
+	    p->fi = rpmfiFree(p->fi);
+	    p->h = headerFree(p->h);
+	}
+    }
+    pi = rpmtsiFree(pi);
+
     rpmtsFreeLock(lock);
 
     /*@-nullstate@*/ /* FIX: ts->flList may be NULL */
