@@ -7,12 +7,15 @@
 #include <rpmcli.h>
 #include <rpmpgp.h>
 #include <rpmdb.h>
+#include <rpmbuild.h>
 
 #include "header-py.h"
 #include "rpmds-py.h"	/* XXX for rpmdsNew */
 #include "rpmfi-py.h"	/* XXX for rpmfiNew */
 #include "rpmmi-py.h"
+#include "rpmps-py.h"
 #include "rpmte-py.h"
+#include "spec-py.h"
 
 #define	_RPMTS_INTERNAL	/* XXX for ts->rdb, ts->availablePackage */
 #include "rpmts-py.h"
@@ -21,7 +24,7 @@
 
 /*@unchecked@*/
 /*@-shadow@*/
-static int _rpmts_debug = 0;
+extern int _rpmts_debug;
 /*@=shadow@*/
 
 /*@access alKey @*/
@@ -165,11 +168,15 @@ struct rpmtsCallbackType_s {
  */
 /*@null@*/
 static PyObject *
-rpmts_Debug(/*@unused@*/ rpmtsObject * s, PyObject * args)
+rpmts_Debug(/*@unused@*/ rpmtsObject * s, PyObject * args, PyObject * kwds)
         /*@globals _Py_NoneStruct @*/
         /*@modifies _Py_NoneStruct @*/
 {
-    if (!PyArg_ParseTuple(args, "i:Debug", &_rpmts_debug)) return NULL;
+    char * kwlist[] = {"debugLevel", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:Debug", kwlist,
+    	    &_rpmts_debug))
+	return NULL;
 
 if (_rpmts_debug < 0)
 fprintf(stderr, "*** rpmts_Debug(%p) ts %p\n", s, s->ts);
@@ -208,7 +215,7 @@ fprintf(stderr, "\tAddAvailable(%p) list %p\n", ts, ts->availablePackages);
  */
 /*@null@*/
 static PyObject *
-rpmts_AddInstall(rpmtsObject * s, PyObject * args)
+rpmts_AddInstall(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -216,8 +223,10 @@ rpmts_AddInstall(rpmtsObject * s, PyObject * args)
     PyObject * key;
     char * how = "u";	/* XXX default to upgrade element if missing */
     int isUpgrade = 0;
+    char * kwlist[] = {"header", "key", "how", NULL};
 
-    if (!PyArg_ParseTuple(args, "O!O|s:AddInstall", &hdr_Type, &h, &key, &how))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O|s:AddInstall", kwlist,
+    	    &hdr_Type, &h, &key, &how))
 	return NULL;
 
     {	PyObject * hObj = (PyObject *) h;
@@ -254,18 +263,19 @@ fprintf(stderr, "*** rpmts_AddInstall(%p,%p,%p,%s) ts %p\n", s, h, key, how, s->
  */
 /*@null@*/
 static PyObject *
-rpmts_AddErase(rpmtsObject * s, PyObject * args)
+rpmts_AddErase(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
     PyObject * o;
     int count;
     rpmdbMatchIterator mi;
+    char * kwlist[] = {"name", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_AddErase(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "O:AddErase", &o))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:AddErase", kwlist, &o))
         return NULL;
 
     if (PyString_Check(o)) {
@@ -352,7 +362,7 @@ fprintf(stderr, "*** rpmts_SolveCallback(%p,%p,%p) \"%s\"\n", ts, ds, data, rpmd
  */
 /*@null@*/
 static PyObject *
-rpmts_Check(rpmtsObject * s, PyObject * args)
+rpmts_Check(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -362,9 +372,11 @@ rpmts_Check(rpmtsObject * s, PyObject * args)
     struct rpmtsCallbackType_s cbInfo;
     int i;
     int xx;
+    char * kwlist[] = {"callback", NULL};
 
     memset(&cbInfo, 0, sizeof(cbInfo));
-    if (!PyArg_ParseTuple(args, "|O:Check", &cbInfo.cb))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O:Check", kwlist,
+	    &cbInfo.cb))
 	return NULL;
 
     if (cbInfo.cb != NULL) {
@@ -410,16 +422,17 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 				   conflicts[i].suggestedPkgs[0] : Py_None,
 			       conflicts[i].sense);
 #else
-	    char * byName, * byVersion, * byRelease, * byArch;
+	    char * byName, * byVersion, * byRelease, *byArch;
 	    char * needsName, * needsOP, * needsVersion;
 	    int needsFlags, sense;
 	    fnpyKey key;
 
 	    p = ps->probs + i;
 
-	    /* XXX autorelocated i386 on ia64, fix system-config-packages! */
+            /* XXX autorelocated i386 on ia64, fix system-config-packages! */
 	    if (p->type == RPMPROB_BADRELOCATE)
 		continue;
+
 	    byName = p->pkgNEVR;
 	    if ((byArch= strrchr(byName, '.')) != NULL)
 		*byArch++ = '\0';
@@ -471,7 +484,7 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
  */
 /*@null@*/
 static PyObject *
-rpmts_Order(rpmtsObject * s, PyObject * args)
+rpmts_Order(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -480,8 +493,6 @@ rpmts_Order(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Order(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":Order")) return NULL;
-
     Py_BEGIN_ALLOW_THREADS
     rc = rpmtsOrder(s->ts);
     Py_END_ALLOW_THREADS
@@ -493,15 +504,13 @@ fprintf(stderr, "*** rpmts_Order(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_Clean(rpmtsObject * s, PyObject * args)
+rpmts_Clean(rpmtsObject * s)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies s, _Py_NoneStruct @*/
 {
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Clean(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":Clean")) return NULL;
-
     rpmtsClean(s->ts);
 
     Py_INCREF(Py_None);
@@ -512,7 +521,7 @@ fprintf(stderr, "*** rpmts_Clean(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_IDTXload(rpmtsObject * s, PyObject * args)
+rpmts_IDTXload(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -523,8 +532,6 @@ rpmts_IDTXload(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_IDTXload(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":IDTXload")) return NULL;
-
     Py_BEGIN_ALLOW_THREADS
     idtx = IDTXload(s->ts, tag);
     Py_END_ALLOW_THREADS
@@ -559,7 +566,7 @@ fprintf(stderr, "*** rpmts_IDTXload(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_IDTXglob(rpmtsObject * s, PyObject * args)
+rpmts_IDTXglob(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -571,8 +578,6 @@ rpmts_IDTXglob(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_IDTXglob(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":IDTXglob")) return NULL;
-
     Py_BEGIN_ALLOW_THREADS
     globstr = rpmExpand("%{_repackage_dir}/*.rpm", NULL);
     idtx = IDTXglob(s->ts, globstr, tag);
@@ -609,7 +614,7 @@ fprintf(stderr, "*** rpmts_IDTXglob(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_Rollback(rpmtsObject * s, PyObject * args)
+rpmts_Rollback(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -618,11 +623,13 @@ rpmts_Rollback(rpmtsObject * s, PyObject * args)
     const char ** av = NULL;
     uint_32 rbtid;
     int rc;
+    char * kwlist[] = {"transactionId", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Rollback(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "i:Rollback", &rbtid)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:Rollback", kwlist, &rbtid))
+    	return NULL;
 
     Py_BEGIN_ALLOW_THREADS
     memset(ia, 0, sizeof(*ia));
@@ -646,7 +653,7 @@ fprintf(stderr, "*** rpmts_Rollback(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_OpenDB(rpmtsObject * s, PyObject * args)
+rpmts_OpenDB(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -654,8 +661,6 @@ rpmts_OpenDB(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_OpenDB(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":OpenDB")) return NULL;
-
     if (s->ts->dbmode == -1)
 	s->ts->dbmode = O_RDONLY;
 
@@ -666,7 +671,7 @@ fprintf(stderr, "*** rpmts_OpenDB(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_CloseDB(rpmtsObject * s, PyObject * args)
+rpmts_CloseDB(rpmtsObject * s)
 	/*@modifies s @*/
 {
     int rc;
@@ -674,8 +679,6 @@ rpmts_CloseDB(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_CloseDB(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":CloseDB")) return NULL;
-
     rc = rpmtsCloseDB(s->ts);
     s->ts->dbmode = -1;		/* XXX disable lazy opens */
 
@@ -686,7 +689,7 @@ fprintf(stderr, "*** rpmts_CloseDB(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_InitDB(rpmtsObject * s, PyObject * args)
+rpmts_InitDB(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -695,8 +698,6 @@ rpmts_InitDB(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_InitDB(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":InitDB")) return NULL;
-
     rc = rpmtsInitDB(s->ts, O_RDONLY);
     if (rc == 0)
 	rc = rpmtsCloseDB(s->ts);
@@ -708,7 +709,7 @@ fprintf(stderr, "*** rpmts_InitDB(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_RebuildDB(rpmtsObject * s, PyObject * args)
+rpmts_RebuildDB(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -717,8 +718,6 @@ rpmts_RebuildDB(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_RebuildDB(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":RebuildDB")) return NULL;
-
     Py_BEGIN_ALLOW_THREADS
     rc = rpmtsRebuildDB(s->ts);
     Py_END_ALLOW_THREADS
@@ -730,7 +729,7 @@ fprintf(stderr, "*** rpmts_RebuildDB(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_VerifyDB(rpmtsObject * s, PyObject * args)
+rpmts_VerifyDB(rpmtsObject * s)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
@@ -739,8 +738,6 @@ rpmts_VerifyDB(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_VerifyDB(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":VerifyDB")) return NULL;
-
     Py_BEGIN_ALLOW_THREADS
     rc = rpmtsVerifyDB(s->ts);
     Py_END_ALLOW_THREADS
@@ -752,7 +749,7 @@ fprintf(stderr, "*** rpmts_VerifyDB(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_HdrFromFdno(rpmtsObject * s, PyObject * args)
+rpmts_HdrFromFdno(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, fileSystem @*/
 	/*@modifies s, rpmGlobalMacroContext, fileSystem @*/
 {
@@ -761,8 +758,11 @@ rpmts_HdrFromFdno(rpmtsObject * s, PyObject * args)
     FD_t fd;
     int fdno;
     rpmRC rpmrc;
+    char * kwlist[] = {"fd", NULL};
 
-    if (!PyArg_ParseTuple(args, "i:HdrFromFdno", &fdno)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:HdrFromFdno", kwlist,
+	    &fdno))
+    	return NULL;
 
     fd = fdDup(fdno);
     rpmrc = rpmReadPackageFile(s->ts, fd, "rpmts_HdrFromFdno", &h);
@@ -802,7 +802,7 @@ fprintf(stderr, "*** rpmts_HdrFromFdno(%p) ts %p rc %d\n", s, s->ts, rpmrc);
  */
 /*@null@*/
 static PyObject *
-rpmts_HdrCheck(rpmtsObject * s, PyObject * args)
+rpmts_HdrCheck(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -812,11 +812,14 @@ rpmts_HdrCheck(rpmtsObject * s, PyObject * args)
     const void * uh;
     int uc;
     rpmRC rpmrc;
+    char * kwlist[] = {"headers", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_HdrCheck(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "O:HdrCheck", &blob)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:HdrCheck", kwlist, &blob))
+    	return NULL;
+
     if (blob == Py_None) {
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -858,17 +861,21 @@ fprintf(stderr, "*** rpmts_HdrCheck(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_SetVSFlags(rpmtsObject * s, PyObject * args)
+rpmts_SetVSFlags(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
     rpmVSFlags vsflags;
+    char * kwlist[] = {"flags", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_SetVSFlags(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "i:SetVSFlags", &vsflags)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:SetVSFlags", kwlist,
+	    &vsflags))
+    	return NULL;
 
-    /* XXX FIXME: value check on vsflags. */
+    /* XXX FIXME: value check on vsflags, or build pure python object 
+     * for it, and require an object of that type */
 
     return Py_BuildValue("i", rpmtsSetVSFlags(s->ts, vsflags));
 }
@@ -877,17 +884,20 @@ fprintf(stderr, "*** rpmts_SetVSFlags(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_SetColor(rpmtsObject * s, PyObject * args)
+rpmts_SetColor(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
     uint_32 tscolor;
+    char * kwlist[] = {"color", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_SetColor(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "i:Color", &tscolor)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:Color", kwlist, &tscolor))
+    	return NULL;
 
-    /* XXX FIXME: value check on tscolor. */
+    /* XXX FIXME: value check on tscolor, or build pure python object
+     * for it, and require an object of that type */
 
     return Py_BuildValue("i", rpmtsSetColor(s->ts, tscolor));
 }
@@ -896,7 +906,7 @@ fprintf(stderr, "*** rpmts_SetColor(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_PgpPrtPkts(rpmtsObject * s, PyObject * args)
+rpmts_PgpPrtPkts(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies _Py_NoneStruct @*/
 {
@@ -904,11 +914,14 @@ rpmts_PgpPrtPkts(rpmtsObject * s, PyObject * args)
     unsigned char * pkt;
     unsigned int pktlen;
     int rc;
+    char * kwlist[] = {"octets", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_PgpPrtPkts(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "O:PgpPrtPkts", &blob)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:PgpPrtPkts", kwlist, &blob))
+    	return NULL;
+
     if (blob == Py_None) {
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -929,7 +942,7 @@ fprintf(stderr, "*** rpmts_PgpPrtPkts(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_PgpImportPubkey(rpmtsObject * s, PyObject * args)
+rpmts_PgpImportPubkey(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -937,11 +950,15 @@ rpmts_PgpImportPubkey(rpmtsObject * s, PyObject * args)
     unsigned char * pkt;
     unsigned int pktlen;
     int rc;
+    char * kwlist[] = {"pubkey", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_PgpImportPubkey(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "O:PgpImportPubkey", &blob)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:PgpImportPubkey",
+    	    kwlist, &blob))
+	return NULL;
+
     if (blob == Py_None) {
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -962,7 +979,7 @@ fprintf(stderr, "*** rpmts_PgpImportPubkey(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
-rpmts_GetKeys(rpmtsObject * s, PyObject * args)
+rpmts_GetKeys(rpmtsObject * s)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies s, _Py_NoneStruct @*/
 {
@@ -973,8 +990,6 @@ rpmts_GetKeys(rpmtsObject * s, PyObject * args)
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_GetKeys(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, ":GetKeys")) return NULL;
-
     rpmtsGetKeys(s->ts, &data, &num);
     if (data == NULL || num <= 0) {
 	data = _free(data);
@@ -1058,6 +1073,8 @@ rpmtsCallback(/*@unused@*/ const void * hd, const rpmCallbackType what,
 if (_rpmts_debug)
 fprintf(stderr, "\t%p = fdDup(%d)\n", fd, fdno);
 
+	fcntl(Fileno(fd), F_SETFD, FD_CLOEXEC);
+
 	return fd;
     } else
     if (what == RPMCALLBACK_INST_CLOSE_FILE) {
@@ -1077,29 +1094,38 @@ fprintf(stderr, "\t%ld:%ld key %p\n", amount, total, pkgKey);
 
 /** \ingroup py_c
  */
-static PyObject * rpmts_SetFlags(rpmtsObject * s, PyObject * args)
+static PyObject *
+rpmts_SetFlags(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
     rpmtransFlags transFlags = 0;
+    char * kwlist[] = {"flags", NULL};
 
-    if (!PyArg_ParseTuple(args, "i:SetFlags", &transFlags))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:SetFlags", kwlist,
+	    &transFlags))
 	return NULL;
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_SetFlags(%p) ts %p transFlags %x\n", s, s->ts, transFlags);
 
+    /* XXX FIXME: value check on flags, or build pure python object 
+     * for it, and require an object of that type */
+
     return Py_BuildValue("i", rpmtsSetFlags(s->ts, transFlags));
 }
 
 /** \ingroup py_c
  */
-static PyObject * rpmts_SetProbFilter(rpmtsObject * s, PyObject * args)
+static PyObject *
+rpmts_SetProbFilter(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
     rpmprobFilterFlags ignoreSet = 0;
     rpmprobFilterFlags oignoreSet;
+    char * kwlist[] = {"ignoreSet", NULL};
 
-    if (!PyArg_ParseTuple(args, "i:ProbFilter", &ignoreSet))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:ProbFilter", kwlist,
+	    &ignoreSet))
 	return NULL;
 
 if (_rpmts_debug)
@@ -1113,7 +1139,22 @@ fprintf(stderr, "*** rpmts_SetProbFilter(%p) ts %p ignoreSet %x\n", s, s->ts, ig
 
 /** \ingroup py_c
  */
-static PyObject * rpmts_Run(rpmtsObject * s, PyObject * args)
+/*@null@*/
+static rpmpsObject *
+rpmts_Problems(rpmtsObject * s)
+	/*@modifies s @*/
+{
+
+if (_rpmts_debug)
+fprintf(stderr, "*** rpmts_Problems(%p) ts %p\n", s, s->ts);
+
+    return rpmps_Wrap( rpmtsProblems(s->ts) );
+}
+
+/** \ingroup py_c
+ */
+static PyObject *
+rpmts_Run(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies s, rpmGlobalMacroContext, _Py_NoneStruct @*/
 {
@@ -1121,8 +1162,10 @@ static PyObject * rpmts_Run(rpmtsObject * s, PyObject * args)
     PyObject * list;
     rpmps ps;
     struct rpmtsCallbackType_s cbInfo;
+    char * kwlist[] = {"callback", "data", NULL};
 
-    if (!PyArg_ParseTuple(args, "OO:Run", &cbInfo.cb, &cbInfo.data))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO:Run", kwlist,
+	    &cbInfo.cb, &cbInfo.data))
 	return NULL;
 
     cbInfo.tso = s;
@@ -1151,7 +1194,6 @@ static PyObject * rpmts_Run(rpmtsObject * s, PyObject * args)
 	sx = rpmsxFree(sx);
     } 
 
-
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Run(%p) ts %p ignore %x\n", s, s->ts, s->ignoreSet);
 
@@ -1263,34 +1305,68 @@ fprintf(stderr, "*** rpmts_Next(%p) ts %p\n", s, s->ts);
     return result;
 }
 
+/**
+ */
+/*@null@*/
+static specObject *
+spec_Parse(rpmtsObject * s, PyObject * args, PyObject * kwds)
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies s, rpmGlobalMacroContext @*/
+{
+    const char * specfile;
+    Spec spec;
+    char * buildRoot = NULL;
+    int recursing = 0;
+    char * passPhrase = "";
+    char *cookie = NULL;
+    int anyarch = 1;
+    int force = 1;
+    char * kwlist[] = {"specfile", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s:Parse", kwlist, &specfile))
+	return NULL;
+
+    if (parseSpec(s->ts, specfile,"/", buildRoot,recursing, passPhrase,
+             cookie, anyarch, force)!=0) {
+             PyErr_SetString(pyrpmError, "can't parse specfile\n");
+                     return NULL;
+   }
+
+    spec = rpmtsSpec(s->ts);
+    return spec_Wrap(spec);
+}
+
 /**
  */
 /*@null@*/
 static rpmmiObject *
-rpmts_Match(rpmtsObject * s, PyObject * args)
+rpmts_Match(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
     PyObject *TagN = NULL;
     PyObject *Key = NULL;
     char *key = NULL;
-    long lkey = 0;
+/* XXX lkey *must* be a 32 bit integer, int "works" on all known platforms. */
+    int lkey = 0;
     int len = 0;
     int tag = RPMDBI_PACKAGES;
+    char * kwlist[] = {"tagNumber", "key", NULL};
 
 if (_rpmts_debug)
 fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
 
-    if (!PyArg_ParseTuple(args, "|OO", &TagN, &Key))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OO:Match", kwlist,
+	    &TagN, &Key))
 	return NULL;
 
     if (TagN && (tag = tagNumFromPyObject (TagN)) == -1) {
 	PyErr_SetString(PyExc_TypeError, "unknown tag type");
 	return NULL;
-
     }
 
     if (Key) {
+/*@-branchstate@*/
 	if (PyString_Check(Key)) {
 	    key = PyString_AsString(Key);
 	    len = PyString_Size(Key);
@@ -1302,6 +1378,7 @@ fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
 	    PyErr_SetString(PyExc_TypeError, "unknown key type");
 	    return NULL;
 	}
+/*@=branchstate@*/
     }
 
     /* XXX If not already opened, open the database O_RDONLY now. */
@@ -1322,63 +1399,66 @@ fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
 /*@-fullinitblock@*/
 /*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef rpmts_methods[] = {
- {"Debug",	(PyCFunction)rpmts_Debug,	METH_VARARGS,
+ {"Debug",	(PyCFunction)rpmts_Debug,	METH_VARARGS|METH_KEYWORDS,
         NULL},
 
- {"addInstall",	(PyCFunction) rpmts_AddInstall,	METH_VARARGS,
+ {"addInstall",	(PyCFunction) rpmts_AddInstall,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"addErase",	(PyCFunction) rpmts_AddErase,	METH_VARARGS,
+ {"addErase",	(PyCFunction) rpmts_AddErase,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"check",	(PyCFunction) rpmts_Check,	METH_VARARGS,
+ {"check",	(PyCFunction) rpmts_Check,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"order",	(PyCFunction) rpmts_Order,	METH_VARARGS,
+ {"order",	(PyCFunction) rpmts_Order,	METH_NOARGS,
 	NULL },
- {"setFlags",	(PyCFunction) rpmts_SetFlags,	METH_VARARGS,
+ {"setFlags",	(PyCFunction) rpmts_SetFlags,	METH_VARARGS|METH_KEYWORDS,
 "ts.setFlags(transFlags) -> previous transFlags\n\
 - Set control bit(s) for executing ts.run().\n\
   Note: This method replaces the 1st argument to the old ts.run()\n" },
- {"setProbFilter",	(PyCFunction) rpmts_SetProbFilter,	METH_VARARGS,
+ {"setProbFilter",	(PyCFunction) rpmts_SetProbFilter,	METH_VARARGS|METH_KEYWORDS,
 "ts.setProbFilter(ignoreSet) -> previous ignoreSet\n\
 - Set control bit(s) for ignoring problems found by ts.run().\n\
   Note: This method replaces the 2nd argument to the old ts.run()\n" },
- {"run",	(PyCFunction) rpmts_Run,	METH_VARARGS,
+ {"problems",	(PyCFunction) rpmts_Problems,	METH_NOARGS,
+"ts.problems() -> ps\n\
+- Return current problem set.\n" },
+ {"run",	(PyCFunction) rpmts_Run,	METH_VARARGS|METH_KEYWORDS,
 "ts.run(callback, data) -> (problems)\n\
 - Run a transaction set, returning list of problems found.\n\
   Note: The callback may not be None.\n" },
- {"clean",	(PyCFunction) rpmts_Clean,	METH_VARARGS,
+ {"clean",	(PyCFunction) rpmts_Clean,	METH_NOARGS,
 	NULL },
- {"IDTXload",	(PyCFunction) rpmts_IDTXload,	METH_VARARGS,
+ {"IDTXload",	(PyCFunction) rpmts_IDTXload,	METH_NOARGS,
 "ts.IDTXload() -> ((tid,hdr,instance)+)\n\
 - Return list of installed packages reverse sorted by transaction id.\n" },
- {"IDTXglob",	(PyCFunction) rpmts_IDTXglob,	METH_VARARGS,
+ {"IDTXglob",	(PyCFunction) rpmts_IDTXglob,	METH_NOARGS,
 "ts.IDTXglob() -> ((tid,hdr,instance)+)\n\
 - Return list of removed packages reverse sorted by transaction id.\n" },
- {"rollback",	(PyCFunction) rpmts_Rollback,	METH_VARARGS,
+ {"rollback",	(PyCFunction) rpmts_Rollback,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"openDB",	(PyCFunction) rpmts_OpenDB,	METH_VARARGS,
+ {"openDB",	(PyCFunction) rpmts_OpenDB,	METH_NOARGS,
 "ts.openDB() -> None\n\
 - Open the default transaction rpmdb.\n\
   Note: The transaction rpmdb is lazily opened, so ts.openDB() is seldom needed.\n" },
- {"closeDB",	(PyCFunction) rpmts_CloseDB,	METH_VARARGS,
+ {"closeDB",	(PyCFunction) rpmts_CloseDB,	METH_NOARGS,
 "ts.closeDB() -> None\n\
 - Close the default transaction rpmdb.\n\
   Note: ts.closeDB() disables lazy opens, and should hardly ever be used.\n" },
- {"initDB",	(PyCFunction) rpmts_InitDB,	METH_VARARGS,
+ {"initDB",	(PyCFunction) rpmts_InitDB,	METH_NOARGS,
 "ts.initDB() -> None\n\
 - Initialize the default transaction rpmdb.\n\
  Note: ts.initDB() is seldom needed anymore.\n" },
- {"rebuildDB",	(PyCFunction) rpmts_RebuildDB,	METH_VARARGS,
+ {"rebuildDB",	(PyCFunction) rpmts_RebuildDB,	METH_NOARGS,
 "ts.rebuildDB() -> None\n\
 - Rebuild the default transaction rpmdb.\n" },
- {"verifyDB",	(PyCFunction) rpmts_VerifyDB,	METH_VARARGS,
+ {"verifyDB",	(PyCFunction) rpmts_VerifyDB,	METH_NOARGS,
 "ts.verifyDB() -> None\n\
 - Verify the default transaction rpmdb.\n" },
- {"hdrFromFdno",(PyCFunction) rpmts_HdrFromFdno,METH_VARARGS,
+ {"hdrFromFdno",(PyCFunction) rpmts_HdrFromFdno,METH_VARARGS|METH_KEYWORDS,
 "ts.hdrFromFdno(fdno) -> hdr\n\
 - Read a package header from a file descriptor.\n" },
- {"hdrCheck",	(PyCFunction) rpmts_HdrCheck,	METH_VARARGS,
+ {"hdrCheck",	(PyCFunction) rpmts_HdrCheck,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"setVSFlags",(PyCFunction) rpmts_SetVSFlags,	METH_VARARGS,
+ {"setVSFlags",(PyCFunction) rpmts_SetVSFlags,	METH_VARARGS|METH_KEYWORDS,
 "ts.setVSFlags(vsflags) -> ovsflags\n\
 - Set signature verification flags. Values for vsflags are:\n\
     rpm.RPMVSF_NOHDRCHK      if set, don't check rpmdb headers\n\
@@ -1390,18 +1470,21 @@ static struct PyMethodDef rpmts_methods[] = {
     rpm.RPMVSF_NORSA         if set, don't check header+payload RSA signature\n\
     rpm._RPMVSF_NODIGESTS    if set, don't check digest(s)\n\
     rpm._RPMVSF_NOSIGNATURES if set, don't check signature(s)\n" },
- {"setColor",(PyCFunction) rpmts_SetColor,	METH_VARARGS,
+ {"setColor",(PyCFunction) rpmts_SetColor,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"pgpPrtPkts",	(PyCFunction) rpmts_PgpPrtPkts,	METH_VARARGS,
+ {"pgpPrtPkts",	(PyCFunction) rpmts_PgpPrtPkts,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"pgpImportPubkey",	(PyCFunction) rpmts_PgpImportPubkey,	METH_VARARGS,
+ {"pgpImportPubkey",	(PyCFunction) rpmts_PgpImportPubkey,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
- {"getKeys",	(PyCFunction) rpmts_GetKeys,	METH_VARARGS,
+ {"getKeys",	(PyCFunction) rpmts_GetKeys,	METH_NOARGS,
 	NULL },
- {"dbMatch",	(PyCFunction) rpmts_Match,	METH_VARARGS,
+ {"parseSpec",	(PyCFunction) spec_Parse,	METH_VARARGS|METH_KEYWORDS,
+"ts.parseSpec(\"/path/to/foo.spec\") -> spec\n\
+- Parse a spec file.\n" },
+ {"dbMatch",	(PyCFunction) rpmts_Match,	METH_VARARGS|METH_KEYWORDS,
 "ts.dbMatch([TagN, [key, [len]]]) -> mi\n\
 - Create a match iterator for the default transaction rpmdb.\n" },
- {"next",		(PyCFunction)rpmts_Next,	METH_VARARGS,
+ {"next",		(PyCFunction)rpmts_Next,	METH_NOARGS,
 "ts.next() -> te\n\
 - Retrieve next transaction set element.\n" },
     {NULL,		NULL}		/* sentinel */
@@ -1465,15 +1548,20 @@ static int rpmts_init(rpmtsObject * s, PyObject *args, PyObject *kwds)
 {
     char * rootDir = "/";
     int vsflags = rpmExpandNumeric("%{?_vsflags_up2date}");
+    char * kwlist[] = {"rootdir", "vsflags", 0};
 
 if (_rpmts_debug < 0)
 fprintf(stderr, "*** rpmts_init(%p,%p,%p)\n", s, args, kwds);
 
-    if (!PyArg_ParseTuple(args, "|si:rpmts_init", &rootDir, &vsflags))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|si:rpmts_init", kwlist,
+	    &rootDir, &vsflags))
 	return -1;
 
     s->ts = rpmtsCreate();
+    /* XXX: Why is there no rpmts_SetRootDir() ? */
     (void) rpmtsSetRootDir(s->ts, rootDir);
+    /* XXX: make this use common code with rpmts_SetVSFlags() to check the
+     *      python objects */
     (void) rpmtsSetVSFlags(s->ts, vsflags);
     s->keyList = PyList_New(0);
     s->scriptFd = NULL;
@@ -1592,20 +1680,26 @@ PyTypeObject rpmts_Type = {
 
 /**
  */
+/* XXX: This should use the same code as rpmts_init */
 rpmtsObject *
-rpmts_Create(/*@unused@*/ PyObject * self, PyObject * args)
+rpmts_Create(/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds)
 {
     rpmtsObject * o;
     char * rootDir = "/";
     int vsflags = rpmExpandNumeric("%{?_vsflags_up2date}");
+    char * kwlist[] = {"rootdir", "vsflags", NULL};
 
-    if (!PyArg_ParseTuple(args, "|si:Create", &rootDir, &vsflags))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|si:Create", kwlist,
+	    &rootDir, &vsflags))
 	return NULL;
 
     o = (void *) PyObject_New(rpmtsObject, &rpmts_Type);
 
     o->ts = rpmtsCreate();
+    /* XXX: Why is there no rpmts_SetRootDir() ? */
     (void) rpmtsSetRootDir(o->ts, rootDir);
+    /* XXX: make this use common code with rpmts_SetVSFlags() to check the
+     *      python objects */
     (void) rpmtsSetVSFlags(o->ts, vsflags);
 
     o->keyList = PyList_New(0);
