@@ -217,6 +217,7 @@ static int db3_fsync_disable(/*@unused@*/ int fd)
     return 0;
 }
 
+#if 0
 #if HAVE_LIBPTHREAD
 #if HAVE_PTHREAD_H
 #include <pthread.h>
@@ -265,6 +266,7 @@ static int db3_pthread_nptl(void)
     return ret;
 }
 #endif
+#endif
 
 /*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db_init(dbiIndex dbi, const char * dbhome,
@@ -355,8 +357,10 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	    sleep(15);
 	}
     } else {
+#if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
+#endif
 	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
 	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
@@ -506,6 +510,7 @@ static int db3copen(dbiIndex dbi, DB_TXN * txnid,
     int flags;
     int rc;
 
+   /* XXX DB_WRITECURSOR cannot be used with sunrpc dbenv. */
     assert(db != NULL);
     if ((dbiflags & DB_WRITECURSOR) &&
 	(dbi->dbi_eflags & DB_INIT_CDB) && !(dbi->dbi_oflags & DB_RDONLY))
@@ -666,6 +671,9 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
 	/*@modifies dbi, fileSystem @*/
 {
     DB * db = dbi->dbi_db;
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+    DB_TXN * txnid = NULL;
+#endif
     int rc = 0;
 
     assert(db != NULL);
@@ -678,7 +686,11 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
     dbi->dbi_stats = _free(dbi->dbi_stats);
 /* XXX 3.3.4 change. */
 #if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3) || (DB_VERSION_MAJOR == 4)
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+    rc = db->stat(db, txnid, &dbi->dbi_stats, flags);
+#else
     rc = db->stat(db, &dbi->dbi_stats, flags);
+#endif
 #else
     rc = db->stat(db, &dbi->dbi_stats, NULL, flags);
 #endif
@@ -809,8 +821,10 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
  /*	dbenv->set_paniccall(???) */
 	/*@=noeffectuncon@*/
+#if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
+#endif
 	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
 	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
@@ -850,7 +864,7 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 			(dbhome ? dbhome : ""),
 			(dbfile ? dbfile : tagName(dbi->dbi_rpmtag)));
 
-		/*
+	        /*
 		 * The DB handle may not be accessed again after
 		 * DB->verify is called, regardless of its return.
 		 */
@@ -978,6 +992,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if (dbi->dbi_use_dbenv) {
 
+#if 0
 #if HAVE_LIBPTHREAD
 	if (rpmdb->db_dbenv == NULL) {
 	    /* Set DB_PRIVATE if posix mutexes are not shared. */
@@ -987,6 +1002,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		rpmMessage(RPMMESS_DEBUG, _("unshared posix mutexes found(%d), adding DB_PRIVATE, using fcntl lock\n"), xx);
 	    }
 	}
+#endif
 #endif
 
 	if (access(dbhome, W_OK) == -1) {
