@@ -4,10 +4,6 @@
 
 #include "system.h"
 #include "rpmio_internal.h"
-#include "beecrypt.h"
-#include "md5.h"
-#include "endianness.h"
-#include "fips180.h"
 #include "debug.h"
 
 #ifdef	SHA_DEBUG
@@ -23,25 +19,28 @@
  */
 struct DIGEST_CTX_s {
     rpmDigestFlags flags;	/*!< Bit(s) to control digest operation. */
-    uint32 datalen;		/*!< No. bytes in block of plaintext data. */
-    uint32 paramlen;		/*!< No. bytes of digest parameters. */
-    uint32 digestlen;		/*!< No. bytes of digest. */
+    uint32_t datalen;		/*!< No. bytes in block of plaintext data. */
+    uint32_t paramlen;		/*!< No. bytes of digest parameters. */
+    uint32_t digestlen;		/*!< No. bytes of digest. */
     void * param;		/*!< Digest parameters. */
     int (*Reset) (void * param)
 	/*@modifies param @*/;	/*!< Digest initialize. */
-    int (*Update) (void * param, const byte * data, int len)
+    int (*Update) (void * param, const byte * data, size_t size)
 	/*@modifies param @*/;	/*!< Digest transform. */
-    int (*Digest) (void * param, /*@out@*/ uint32 * digest)
+    int (*Digest) (void * param, /*@out@*/ byte * digest)
 	/*@modifies param, digest @*/;	/*!< Digest finish. */
 };
 
+/*@-boundsread@*/
 DIGEST_CTX
 rpmDigestDup(DIGEST_CTX octx)
 {
-    DIGEST_CTX nctx = memcpy(xcalloc(1, sizeof(*nctx)), octx, sizeof(*nctx));
+    DIGEST_CTX nctx;
+    nctx = memcpy(xcalloc(1, sizeof(*nctx)), octx, sizeof(*nctx));
     nctx->param = memcpy(xcalloc(1, nctx->paramlen), octx->param, nctx->paramlen);
     return nctx;
 }
+/*@=boundsread@*/
 
 DIGEST_CTX
 rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
@@ -55,28 +54,28 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
     case PGPHASHALGO_MD5:
 	ctx->digestlen = 16;
 	ctx->datalen = 64;
-	/*@-sizeoftype@*/ /* FIX: union, not void pointer */
+/*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(md5Param);
-	/*@=sizeoftype@*/
+/*@=sizeoftype@*/
 	ctx->param = xcalloc(1, ctx->paramlen);
-	/*@-type@*/ /* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
 	ctx->Reset = (void *) md5Reset;
 	ctx->Update = (void *) md5Update;
 	ctx->Digest = (void *) md5Digest;
-	/*@=type@*/
+/*@=type@*/
 	break;
     case PGPHASHALGO_SHA1:
 	ctx->digestlen = 20;
 	ctx->datalen = 64;
-	/*@-sizeoftype@*/ /* FIX: union, not void pointer */
+/*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(sha1Param);
-	/*@=sizeoftype@*/
+/*@=sizeoftype@*/
 	ctx->param = xcalloc(1, ctx->paramlen);
-	/*@-type@*/ /* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
 	ctx->Reset = (void *) sha1Reset;
 	ctx->Update = (void *) sha1Update;
 	ctx->Digest = (void *) sha1Digest;
-	/*@=type@*/
+/*@=type@*/
 	break;
     case PGPHASHALGO_RIPEMD160:
     case PGPHASHALGO_MD2:
@@ -88,7 +87,9 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	/*@notreached@*/ break;
     }
 
+/*@-boundsread@*/
     xx = (*ctx->Reset) (ctx->param);
+/*@=boundsread@*/
 
 DPRINTF((stderr, "*** Init(%x) ctx %p param %p\n", flags, ctx, ctx->param));
     return ctx;
@@ -98,44 +99,35 @@ DPRINTF((stderr, "*** Init(%x) ctx %p param %p\n", flags, ctx, ctx->param));
 int
 rpmDigestUpdate(DIGEST_CTX ctx, const void * data, size_t len)
 {
+    if (ctx == NULL)
+	return -1;
+
 DPRINTF((stderr, "*** Update(%p,%p,%d) param %p \"%s\"\n", ctx, data, len, ctx->param, ((char *)data)));
+/*@-boundsread@*/
     return (*ctx->Update) (ctx->param, data, len);
+/*@=boundsread@*/
 }
 /*@=mustmod@*/
 
-/*@unchecked@*/
-static int _ie = 0x44332211;
-/*@-redef@*/
-/*@unchecked@*/
-static union _dendian {
-/*@unused@*/ int i;
-    char b[4];
-} *_endian = (union _dendian *)&_ie;
-/*@=redef@*/
-#define        IS_BIG_ENDIAN()         (_endian->b[0] == '\x44')
-#define        IS_LITTLE_ENDIAN()      (_endian->b[0] == '\x11')
-
+/*@-boundswrite@*/
 int
-rpmDigestFinal(/*@only@*/ DIGEST_CTX ctx, /*@out@*/ void ** datap,
-	/*@out@*/ size_t *lenp, int asAscii)
+rpmDigestFinal(DIGEST_CTX ctx, void ** datap, size_t *lenp, int asAscii)
 {
-    uint32 * digest = xmalloc(ctx->digestlen);
+    byte * digest;
     char * t;
     int i;
 
-DPRINTF((stderr, "*** Final(%p,%p,%p,%d) param %p digest %p\n", ctx, datap, lenp, asAscii, ctx->param, digest));
-    /*@-noeffectuncon@*/ /* FIX: check rc */
-    (void) (*ctx->Digest) (ctx->param, digest);
-    /*@=noeffectuncon@*/
+    if (ctx == NULL)
+	return -1;
+    digest = xmalloc(ctx->digestlen);
 
-    /*@-sizeoftype@*/
-    if (IS_LITTLE_ENDIAN())
-    for (i = 0; i < (ctx->digestlen/sizeof(uint32)); i++)
-	digest[i] = swapu32(digest[i]);
-    /*@=sizeoftype@*/
+DPRINTF((stderr, "*** Final(%p,%p,%p,%d) param %p digest %p\n", ctx, datap, lenp, asAscii, ctx->param, digest));
+/*@-noeffectuncon@*/ /* FIX: check rc */
+    (void) (*ctx->Digest) (ctx->param, digest);
+/*@=noeffectuncon@*/
 
     /* Return final digest. */
-    /*@-branchstate@*/
+/*@-branchstate@*/
     if (!asAscii) {
 	if (lenp) *lenp = ctx->digestlen;
 	if (datap) {
@@ -156,7 +148,7 @@ DPRINTF((stderr, "*** Final(%p,%p,%p,%d) param %p digest %p\n", ctx, datap, lenp
 	    *t = '\0';
 	}
     }
-    /*@=branchstate@*/
+/*@=branchstate@*/
     if (digest) {
 	memset(digest, 0, ctx->digestlen);	/* In case it's sensitive */
 	free(digest);
@@ -167,3 +159,4 @@ DPRINTF((stderr, "*** Final(%p,%p,%p,%d) param %p digest %p\n", ctx, datap, lenp
     free(ctx);
     return 0;
 }
+/*@=boundswrite@*/
