@@ -59,6 +59,12 @@ static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #   define _STAT_VER		0
 #   define __fxstat64(_stat_ver, _fd, _sbp)	fstat((_fd), (_sbp))
 #endif
+#if defined(__APPLE__)
+#   define __errno_location()	(__error())
+#   define stat64		stat
+#   define _STAT_VER		0
+#   define __fxstat64(_stat_ver, _fd, _sbp)	fstat((_fd), (_sbp))
+#endif
 #include "system.h"
 #include "fts.h"
 #include "rpmio.h"
@@ -188,11 +194,13 @@ Fts_open(char * const * argv, int options,
 		/* Use fchdir(2) speedup only if local DASDI. */
 		switch (urlIsURL(*argv)) {
 		case URL_IS_DASH:
+		case URL_IS_HKP:
 /*@-boundswrite@*/
 			__set_errno (ENOENT);
 /*@=boundswrite@*/
 			goto mem3;
 			/*@notreached@*/ /*@switchbreak@*/ break;
+		case URL_IS_HTTPS:
 		case URL_IS_HTTP:
 		case URL_IS_FTP:
 			SET(FTS_NOCHDIR);
