@@ -288,7 +288,6 @@ findEntry(MacroContext mc, const char * name, size_t namelen)
  * @param buf		input buffer
  * @param size		inbut buffer size (bytes)
  * @param fd		file handle
- * @param escapes	permit escaped newlines?
  * @return		buffer, or NULL on end-of-file
  */
 /*@-boundswrite@*/
@@ -1682,7 +1681,7 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     int argc = 0;
     const char ** argv = NULL;
     char * globRoot = NULL;
-#ifdef ENABLE_NLS	
+#ifdef ENABLE_NLS
     const char * old_collate = NULL;
     const char * old_ctype = NULL;
     const char * t;
@@ -1736,19 +1735,22 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, argv[argc]);
 		maxb = nb;
 	}
 	
-	nb = ((ut > URL_IS_DASH && ut != URL_IS_FTP) ? (path - av[j]) : 0);
+	nb = ((ut == URL_IS_PATH) ? (path - av[j]) : 0);
 	maxb += nb;
 	maxb += 1;
 	globURL = globRoot = xmalloc(maxb);
 
 	switch (ut) {
-	case URL_IS_HTTP:
 	case URL_IS_PATH:
 	case URL_IS_DASH:
 	    strncpy(globRoot, av[j], nb);
 	    /*@switchbreak@*/ break;
+	case URL_IS_HTTPS:
+	case URL_IS_HTTP:
 	case URL_IS_FTP:
+	case URL_IS_HKP:
 	case URL_IS_UNKNOWN:
+	default:
 	    /*@switchbreak@*/ break;
 	}
 	globRoot += nb;
@@ -1786,7 +1788,7 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, globURL);
 
 
 exit:
-#ifdef ENABLE_NLS
+#ifdef ENABLE_NLS	
 /*@-branchstate@*/
     if (old_collate) {
 	(void) setlocale(LC_COLLATE, old_collate);
