@@ -151,6 +151,9 @@ struct _FD_s {
 
 /*@dependent@*/
     void *	url;		/* ufdio: URL info */
+/*@relnull@*/
+    void *	req;		/* ufdio: HTTP request */
+
     int		rd_timeoutsecs;	/* ufdRead: per FD_t timer */
     ssize_t	bytesRemain;	/* ufdio: */
     ssize_t	contentLength;	/* ufdio: */
@@ -181,11 +184,21 @@ struct _FD_s {
 extern int _rpmio_debug;
 /*@=redecl@*/
 
+/*@-redecl@*/
+/*@unchecked@*/
+extern int _av_debug;
+/*@=redecl@*/
+
 /*@-redecl@*/
 /*@unchecked@*/
 extern int _ftp_debug;
 /*@=redecl@*/
 
+/*@-redecl@*/
+/*@unchecked@*/
+extern int _dav_debug;
+/*@=redecl@*/
+
 #define DBG(_f, _m, _x) \
     /*@-modfilesys@*/ \
     if ((_rpmio_debug | ((_f) ? ((FD_t)(_f))->flags : 0)) & (_m)) fprintf _x \
@@ -405,7 +418,14 @@ void fdstat_exit(/*@null@*/ FD_t fd, int opx, ssize_t rc)
     if (rc == -1)
 	fd->syserrno = errno;
     else if (rc > 0 && fd->bytesRemain > 0)
+	switch (opx) {
+	case FDSTAT_READ:
+	case FDSTAT_WRITE:
 	fd->bytesRemain -= rc;
+	    break;
+	default:
+	    break;
+	}
     if (fd->stats != NULL)
 	(void) rpmswExit(fdstat_op(fd, opx), rc);
 }
