@@ -36,7 +36,7 @@ extern MacroContext rpmCLIMacroContext;
  * identified by the token '://', so file paths must not begin with '//'.
  */
 /*@-redecl@*/
-/*@observer@*/ /*@unchecked@*/
+/*@observer@*/ /*@checked@*/
 extern const char * macrofiles;
 /*@=redecl@*/
 
@@ -64,10 +64,21 @@ extern "C" {
  */
 void	rpmDumpMacroTable	(/*@null@*/ MacroContext mc,
 					/*@null@*/ FILE * fp)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem@*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
 	/*@modifies *fp, fileSystem @*/;
 
+/**
+ * Return URL path(s) from a (URL prefixed) pattern glob.
+ * @param patterns	glob pattern
+ * @retval *argcPtr	no. of paths
+ * @retval *argvPtr	array of paths (malloc'd contiguous blob)
+ * @return		0 on success
+ */
+int rpmGlob(const char * patterns, /*@out@*/ int * argcPtr,
+		/*@out@*/ const char *** argvPtr)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies *argcPtr, *argvPtr, fileSystem, internalState @*/;
+
 /**
  * Expand macro into buffer.
  * @deprecated Use rpmExpand().
@@ -81,8 +92,7 @@ void	rpmDumpMacroTable	(/*@null@*/ MacroContext mc,
 int	expandMacros	(/*@null@*/ void * spec, /*@null@*/ MacroContext mc,
 				/*@in@*/ /*@out@*/ char * sbuf,
 				size_t slen)
-	/*@globals rpmGlobalMacroContext,
-		fileSystem @*/
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
 	/*@modifies *sbuf, rpmGlobalMacroContext, fileSystem @*/;
 
 /**
@@ -97,7 +107,7 @@ int	expandMacros	(/*@null@*/ void * spec, /*@null@*/ MacroContext mc,
 void	addMacro	(/*@null@*/ MacroContext mc, const char * n,
 				/*@null@*/ const char * o,
 				/*@null@*/ const char * b, int level)
-	/*@globals rpmGlobalMacroContext@*/
+	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies mc, rpmGlobalMacroContext @*/;
 
 /**
@@ -106,19 +116,19 @@ void	addMacro	(/*@null@*/ MacroContext mc, const char * n,
  * @param n		macro name
  */
 void	delMacro	(/*@null@*/ MacroContext mc, const char * n)
-	/*@globals rpmGlobalMacroContext@*/
+	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies mc, rpmGlobalMacroContext @*/;
 
 /**
  * Define macro in context.
  * @param mc		macro context (NULL uses global context).
- * @param n		macro name, options, body
+ * @param macro		macro name, options, body
  * @param level		macro recursion level (0 is entry API)
  * @return		@todo Document.
  */
 int	rpmDefineMacro	(/*@null@*/ MacroContext mc, const char * macro,
 				int level)
-	/*@globals rpmGlobalMacroContext@*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies mc, rpmGlobalMacroContext @*/;
 
 /**
@@ -127,25 +137,35 @@ int	rpmDefineMacro	(/*@null@*/ MacroContext mc, const char * macro,
  * @param level		macro recursion level (0 is entry API)
  */
 void	rpmLoadMacros	(/*@null@*/ MacroContext mc, int level)
-	/*@globals rpmGlobalMacroContext@*/
+	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 
 /**
- * Initialize global macro context from set of macrofile(s).
+ * Load macro context from a macro file.
  * @param mc		(unused)
+ * @param fn		macro file name
+ */
+int	rpmLoadMacroFile(/*@null@*/ MacroContext mc, const char * fn)
+	/*@globals rpmGlobalMacroContext,
+		h_errno, fileSystem, internalState @*/
+	/*@modifies mc, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Initialize macro context from set of macrofile(s).
+ * @param mc		macro context
  * @param macrofiles	colon separated list of macro files (NULL does nothing)
  */
 void	rpmInitMacros	(/*@null@*/ MacroContext mc, const char * macrofiles)
 	/*@globals rpmGlobalMacroContext, rpmCLIMacroContext,
-		fileSystem @*/
-	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
+		h_errno, fileSystem, internalState @*/
+	/*@modifies mc, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /**
  * Destroy macro context.
  * @param mc		macro context (NULL uses global context).
  */
 void	rpmFreeMacros	(/*@null@*/ MacroContext mc)
-	/*@globals rpmGlobalMacroContext@*/
+	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies mc, rpmGlobalMacroContext @*/;
 
 typedef enum rpmCompressedMagic_e {
@@ -163,8 +183,8 @@ typedef enum rpmCompressedMagic_e {
  */
 int	isCompressed	(const char * file,
 				/*@out@*/ rpmCompressedMagic * compressed)
-	/*@globals fileSystem@*/
-	/*@modifies *compressed, fileSystem @*/;
+	/*@globals h_errno, fileSystem, internalState @*/
+	/*@modifies *compressed, fileSystem, internalState @*/;
 
 /**
  * Return (malloc'ed) concatenated macro expansion(s).
@@ -172,7 +192,7 @@ int	isCompressed	(const char * file,
  * @return		macro expansion (malloc'ed)
  */
 char * rpmExpand	(/*@null@*/ const char * arg, ...)
-	/*@globals rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 
 /**
@@ -180,7 +200,8 @@ char * rpmExpand	(/*@null@*/ const char * arg, ...)
  * @param path		path to canonicalize (in-place)
  * @return		canonicalized path (malloc'ed)
  */
-/*@null@*/ char * rpmCleanPath	(/*@null@*/ char * path)
+/*@null@*/
+char * rpmCleanPath	(/*@returned@*/ /*@null@*/ char * path)
 	/*@modifies *path @*/;
 
 /**
@@ -190,7 +211,7 @@ char * rpmExpand	(/*@null@*/ const char * arg, ...)
  */
 /*@-redecl@*/ /* LCL: shrug */
 const char * rpmGetPath	(/*@null@*/ const char * path, ...)
-	/*@globals rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 /*@=redecl@*/
 
@@ -208,7 +229,7 @@ const char * rpmGetPath	(/*@null@*/ const char * path, ...)
 const char * rpmGenPath	(/*@null@*/ const char * urlroot,
 			/*@null@*/ const char * urlmdir,
 			/*@null@*/ const char * urlfile)
-	/*@globals rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 /*@=redecl@*/
 
@@ -220,7 +241,7 @@ const char * rpmGenPath	(/*@null@*/ const char * urlroot,
  * @return		numeric value
  */
 int	rpmExpandNumeric (const char * arg)
-	/*@globals rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 
 #ifdef __cplusplus
