@@ -135,7 +135,8 @@ struct pgpValTbl_s pgpSymkeyTbl[] = {
     { PGPSYMKEYALGO_AES_128,	"AES(128-bit key)" },
     { PGPSYMKEYALGO_AES_192,	"AES(192-bit key)" },
     { PGPSYMKEYALGO_AES_256,	"AES(256-bit key)" },
-    { PGPSYMKEYALGO_TWOFISH,	"TWOFISH" },
+    { PGPSYMKEYALGO_TWOFISH,	"TWOFISH(256-bit key)" },
+    { PGPSYMKEYALGO_NOENCRYPT,	"no encryption" },
     { -1,			"Unknown symmetric key algorithm" },
 };
 
@@ -143,6 +144,7 @@ struct pgpValTbl_s pgpCompressionTbl[] = {
     { PGPCOMPRESSALGO_NONE,	"Uncompressed" },
     { PGPCOMPRESSALGO_ZIP,	"ZIP" },
     { PGPCOMPRESSALGO_ZLIB, 	"ZLIB" },
+    { PGPCOMPRESSALGO_BZIP2, 	"BZIP2" },
     { -1,			"Unknown compression algorithm" },
 };
 
@@ -153,6 +155,9 @@ struct pgpValTbl_s pgpHashTbl[] = {
     { PGPHASHALGO_MD2,		"MD2" },
     { PGPHASHALGO_TIGER192,	"TIGER192" },
     { PGPHASHALGO_HAVAL_5_160,	"HAVAL-5-160" },
+    { PGPHASHALGO_SHA256,	"SHA256" },
+    { PGPHASHALGO_SHA384,	"SHA384" },
+    { PGPHASHALGO_SHA512,	"SHA512" },
     { -1,			"Unknown hash algorithm" },
 };
 
@@ -172,7 +177,7 @@ struct pgpValTbl_s pgpSubTypeTbl[] = {
     { PGPSUBTYPE_REGEX,		"regular expression" },
     { PGPSUBTYPE_REVOCABLE,	"revocable" },
     { PGPSUBTYPE_KEY_EXPIRE_TIME,"key expiration time" },
-    { PGPSUBTYPE_BACKWARD_COMPAT,"placeholder for backward compatibility" },
+    { PGPSUBTYPE_ARR,		"additional recipient request" },
     { PGPSUBTYPE_PREFER_SYMKEY,	"preferred symmetric algorithms" },
     { PGPSUBTYPE_REVOKE_KEY,	"revocation key" },
     { PGPSUBTYPE_ISSUER_KEYID,	"issuer key ID" },
@@ -186,6 +191,9 @@ struct pgpValTbl_s pgpSubTypeTbl[] = {
     { PGPSUBTYPE_KEY_FLAGS,	"key flags" },
     { PGPSUBTYPE_SIGNER_USERID,	"signer's user id" },
     { PGPSUBTYPE_REVOKE_REASON,	"reason for revocation" },
+    { PGPSUBTYPE_FEATURES,	"features" },
+    { PGPSUBTYPE_EMBEDDED_SIG,	"embedded signature" },
+
     { PGPSUBTYPE_INTERNAL_100,	"internal subpkt type 100" },
     { PGPSUBTYPE_INTERNAL_101,	"internal subpkt type 101" },
     { PGPSUBTYPE_INTERNAL_102,	"internal subpkt type 102" },
@@ -326,7 +334,7 @@ const char * pgpMpiHex(const byte *p)
 static int pgpHexSet(const char * pre, int lbits,
 		/*@out@*/ mpnumber * mpn, const byte * p, const byte * pend)
 	/*@globals fileSystem @*/
-	/*@modifies *mpn, fileSystem @*/
+	/*@modifies mpn, fileSystem @*/
 {
     unsigned int mbits = pgpMpiBits(p);
     unsigned int nbits;
@@ -348,7 +356,7 @@ fprintf(stderr, "*** mbits %u nbits %u nbytes %u t %p[%d] ix %u\n", mbits, nbits
     strcpy(t+ix, pgpMpiHex(p));
 if (_debug)
 fprintf(stderr, "*** %s %s\n", pre, t);
-    mpnsethex(mpn, t);
+    (void) mpnsethex(mpn, t);
     t = _free(t);
 if (_debug && _print)
 fprintf(stderr, "\t %s ", pre), mpfprintln(stderr, mpn->size, mpn->data);
@@ -367,7 +375,10 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	p += i;
 	hlen -= i;
 
-	pgpPrtVal("    ", pgpSubTypeTbl, p[0]);
+	pgpPrtVal("    ", pgpSubTypeTbl, (p[0]&(~PGPSUBTYPE_CRITICAL)));
+	if (p[0] & PGPSUBTYPE_CRITICAL)
+	    if (_print)
+		fprintf(stderr, " *CRITICAL*");
 	switch (*p) {
 	case PGPSUBTYPE_PREFER_SYMKEY:	/* preferred symmetric algorithms */
 	    for (i = 1; i < plen; i++)
@@ -388,7 +399,7 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	case PGPSUBTYPE_SIG_CREATE_TIME:
 /*@-mods -mayaliasunique @*/
 	    if (_digp && !(_digp->saved & PGPDIG_SAVED_TIME) &&
-		sigtype == PGPSIGTYPE_POSITIVE_CERT)
+		(sigtype == PGPSIGTYPE_POSITIVE_CERT || sigtype == PGPSIGTYPE_BINARY || sigtype == PGPSIGTYPE_TEXT || sigtype == PGPSIGTYPE_STANDALONE))
 	    {
 		_digp->saved |= PGPDIG_SAVED_TIME;
 		memcpy(_digp->time, p+1, sizeof(_digp->time));
@@ -408,7 +419,7 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	case PGPSUBTYPE_ISSUER_KEYID:	/* issuer key ID */
 /*@-mods -mayaliasunique @*/
 	    if (_digp && !(_digp->saved & PGPDIG_SAVED_ID) &&
-		sigtype == PGPSIGTYPE_POSITIVE_CERT)
+		(sigtype == PGPSIGTYPE_POSITIVE_CERT || sigtype == PGPSIGTYPE_BINARY || sigtype == PGPSIGTYPE_TEXT || sigtype == PGPSIGTYPE_STANDALONE))
 	    {
 		_digp->saved |= PGPDIG_SAVED_ID;
 		memcpy(_digp->signid, p+1, sizeof(_digp->signid));
@@ -419,7 +430,7 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	case PGPSUBTYPE_TRUST_SIG:
 	case PGPSUBTYPE_REGEX:
 	case PGPSUBTYPE_REVOCABLE:
-	case PGPSUBTYPE_BACKWARD_COMPAT:
+	case PGPSUBTYPE_ARR:
 	case PGPSUBTYPE_REVOKE_KEY:
 	case PGPSUBTYPE_NOTATION:
 	case PGPSUBTYPE_PREFER_KEYSERVER:
@@ -428,6 +439,8 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	case PGPSUBTYPE_KEY_FLAGS:
 	case PGPSUBTYPE_SIGNER_USERID:
 	case PGPSUBTYPE_REVOKE_REASON:
+	case PGPSUBTYPE_FEATURES:
+	case PGPSUBTYPE_EMBEDDED_SIG:
 	case PGPSUBTYPE_INTERNAL_100:
 	case PGPSUBTYPE_INTERNAL_101:
 	case PGPSUBTYPE_INTERNAL_102:
@@ -481,7 +494,7 @@ static int pgpPrtSigParams(/*@unused@*/ pgpTag tag, byte pubkey_algo, byte sigty
 	    {
 		switch (i) {
 		case 0:		/* m**d */
-		    mpnsethex(&_dig->c, pgpMpiHex(p));
+		    (void) mpnsethex(&_dig->c, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t  m**d = "),  mpfprintln(stderr, _dig->c.size, _dig->c.data);
 		    /*@switchbreak@*/ break;
@@ -689,26 +702,12 @@ static const byte * pgpPrtPubkeyParams(byte pubkey_algo,
 	    if (_dig) {
 		switch (i) {
 		case 0:		/* n */
-		    mpbsethex(&_dig->rsa_pk.n, pgpMpiHex(p));
-		    /* Get the keyid */
-		    if (_digp) {
-			uint32_t* np = _dig->rsa_pk.n.modl;
-			size_t nsize = _dig->rsa_pk.n.size;
-			uint32_t keyid[2];
-			#if WORDS_BIGENDIAN
-			keyid[0] = np[nsize-2];
-			keyid[1] = np[nsize-1];
-			#else
-			keyid[0] = swapu32(np[nsize-2]);
-			keyid[1] = swapu32(np[nsize-1]);
-			#endif
-			memcpy(_digp->signid, keyid, sizeof(_digp->signid));
-		    }
+		    (void) mpbsethex(&_dig->rsa_pk.n, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     n = "),  mpfprintln(stderr, _dig->rsa_pk.n.size, _dig->rsa_pk.n.modl);
 		    /*@switchbreak@*/ break;
 		case 1:		/* e */
-		    mpnsethex(&_dig->rsa_pk.e, pgpMpiHex(p));
+		    (void) mpnsethex(&_dig->rsa_pk.e, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     e = "),  mpfprintln(stderr, _dig->rsa_pk.e.size, _dig->rsa_pk.e.data);
 		    /*@switchbreak@*/ break;
@@ -722,22 +721,22 @@ fprintf(stderr, "\t     e = "),  mpfprintln(stderr, _dig->rsa_pk.e.size, _dig->r
 	    if (_dig) {
 		switch (i) {
 		case 0:		/* p */
-		    mpbsethex(&_dig->p, pgpMpiHex(p));
+		    (void) mpbsethex(&_dig->p, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     p = "),  mpfprintln(stderr, _dig->p.size, _dig->p.modl);
 		    /*@switchbreak@*/ break;
 		case 1:		/* q */
-		    mpbsethex(&_dig->q, pgpMpiHex(p));
+		    (void) mpbsethex(&_dig->q, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     q = "),  mpfprintln(stderr, _dig->q.size, _dig->q.modl);
 		    /*@switchbreak@*/ break;
 		case 2:		/* g */
-		    mpnsethex(&_dig->g, pgpMpiHex(p));
+		    (void) mpnsethex(&_dig->g, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     g = "),  mpfprintln(stderr, _dig->g.size, _dig->g.data);
 		    /*@switchbreak@*/ break;
 		case 3:		/* y */
-		    mpnsethex(&_dig->y, pgpMpiHex(p));
+		    (void) mpnsethex(&_dig->y, pgpMpiHex(p));
 if (_debug && _print)
 fprintf(stderr, "\t     y = "),  mpfprintln(stderr, _dig->y.size, _dig->y.data);
 		    /*@switchbreak@*/ break;
@@ -939,6 +938,68 @@ int pgpPrtComment(pgpTag tag, const byte *h, unsigned int hlen)
     return 0;
 }
 
+int pgpPubkeyFingerprint(const byte * pkt, /*@unused@*/ unsigned int pktlen,
+		byte * keyid)
+{
+    const byte *s = pkt;
+    DIGEST_CTX ctx;
+    byte version;
+    int rc = -1;	/* assume failure. */
+
+    if (pkt[0] != 0x99)
+	return rc;
+    version = pkt[3];
+
+    switch (version) {
+    case 3:
+      {	pgpPktKeyV3 v = (pgpPktKeyV3) (pkt + 3);
+
+	s += sizeof(pkt[0]) + sizeof(pkt[1]) + sizeof(pkt[2]) + sizeof(*v);
+	switch (v->pubkey_algo) {
+	case PGPPUBKEYALGO_RSA:
+	    s += (pgpMpiLen(s) - 8);
+/*@-boundswrite@*/
+	    memmove(keyid, s, 8);
+/*@=boundswrite@*/
+	    rc = 0;
+	    /*@innerbreak@*/ break;
+	default:	/* TODO: md5 of mpi bodies (i.e. no length) */
+	    /*@innerbreak@*/ break;
+	}
+      }	break;
+    case 4:
+      {	pgpPktKeyV4 v = (pgpPktKeyV4) (pkt + 3);
+	byte * SHA1 = NULL;
+	int i;
+
+	s += sizeof(pkt[0]) + sizeof(pkt[1]) + sizeof(pkt[2]) + sizeof(*v);
+	switch (v->pubkey_algo) {
+	case PGPPUBKEYALGO_RSA:
+	    for (i = 0; i < 2; i++)
+		s += pgpMpiLen(s);
+	    /*@innerbreak@*/ break;
+	case PGPPUBKEYALGO_DSA:
+	    for (i = 0; i < 4; i++)
+		s += pgpMpiLen(s);
+	    /*@innerbreak@*/ break;
+	}
+
+	ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
+	(void) rpmDigestUpdate(ctx, pkt, (s-pkt));
+	(void) rpmDigestFinal(ctx, (void **)&SHA1, NULL, 0);
+
+	s = SHA1 + 12;
+/*@-boundswrite@*/
+	memmove(keyid, s, 8);
+/*@=boundswrite@*/
+	rc = 0;
+
+	if (SHA1) free(SHA1);
+      }	break;
+    }
+    return rc;
+}
+
 int pgpPrtPkt(const byte *pkt, unsigned int pleft)
 {
     unsigned int val = *pkt;
@@ -972,6 +1033,16 @@ int pgpPrtPkt(const byte *pkt, unsigned int pleft)
 	rc = pgpPrtSig(tag, h, hlen);
 	break;
     case PGPTAG_PUBLIC_KEY:
+	/* Get the public key fingerprint. */
+	if (_digp) {
+/*@-mods@*/
+	    if (!pgpPubkeyFingerprint(pkt, pktlen, _digp->signid))
+		_digp->saved |= PGPDIG_SAVED_ID;
+	    else
+		memset(_digp->signid, 0, sizeof(_digp->signid));
+/*@=mods@*/
+	}
+	/*@fallthrough@*/
     case PGPTAG_PUBLIC_SUBKEY:
 	rc = pgpPrtKey(tag, h, hlen);
 	break;
@@ -1149,7 +1220,7 @@ pgpArmor pgpReadPkts(const char * fn, const byte ** pkt, size_t * pktlen)
     const char * armortype = NULL;
     char * t, * te;
     int pstate = 0;
-    pgpArmor ec = PGPARMOR_ERROR;	/* XXX assume failure */
+    pgpArmor ec = PGPARMOR_ERR_NO_BEGIN_PGP;	/* XXX assume failure */
     int rc;
 
     rc = rpmioSlurp(fn, &b, &blen);
@@ -1180,8 +1251,10 @@ pgpArmor pgpReadPkts(const char * fn, const byte ** pkt, size_t * pktlen)
 	    t += sizeof("-----BEGIN PGP ")-1;
 
 	    rc = pgpValTok(pgpArmorTbl, t, te);
-	    if (rc < 0)
+	    if (rc < 0) {
+		ec = PGPARMOR_ERR_UNKNOWN_ARMOR_TYPE;
 		goto exit;
+	    }
 	    if (rc != PGPARMOR_PUBKEY)	/* XXX ASCII Pubkeys only, please. */
 		continue;
 	    armortype = t;
@@ -1214,10 +1287,13 @@ pgpArmor pgpReadPkts(const char * fn, const byte ** pkt, size_t * pktlen)
 	    /*@switchbreak@*/ break;
 	case 3:
 	    pstate = 0;
-	    if (!TOKEQ(t, "-----END PGP "))
+	    if (!TOKEQ(t, "-----END PGP ")) {
+		ec = PGPARMOR_ERR_NO_END_PGP;
 		goto exit;
+	    }
 	    *t = '\0';		/* Terminate encoded crc */
 	    t += sizeof("-----END PGP ")-1;
+	    if (t >= te) continue;
 
 	    if (armortype == NULL) /* XXX can't happen */
 		continue;
@@ -1225,19 +1301,37 @@ pgpArmor pgpReadPkts(const char * fn, const byte ** pkt, size_t * pktlen)
 	    if (rc)
 		continue;
 
-	    t = te - (sizeof("-----\n")-1);
-	    if (!TOKEQ(t, "-----\n"))
-		goto exit;
+	    t += strlen(armortype);
+	    if (t >= te) continue;
 
-	    if (b64decode(crcenc, (void **)&crcdec, &crclen) != 0)
-		continue;
+	    if (!TOKEQ(t, "-----")) {
+		ec = PGPARMOR_ERR_NO_END_PGP;
+		goto exit;
+	    }
+	    t += (sizeof("-----")-1);
+	    if (t >= te) continue;
+	    /* XXX permitting \r here is not RFC-2440 compliant <shrug> */
+	    if (!(*t == '\n' || *t == '\r')) continue;
+
+	    crcdec = NULL;
+	    crclen = 0;
+	    if (b64decode(crcenc, (void **)&crcdec, &crclen) != 0) {
+		ec = PGPARMOR_ERR_CRC_DECODE;
+		goto exit;
+	    }
 	    crcpkt = pgpGrab(crcdec, crclen);
 	    crcdec = _free(crcdec);
-	    if (b64decode(enc, (void **)&dec, &declen) != 0)
+	    dec = NULL;
+	    declen = 0;
+	    if (b64decode(enc, (void **)&dec, &declen) != 0) {
+		ec = PGPARMOR_ERR_BODY_DECODE;
 		goto exit;
+	    }
 	    crc = pgpCRC(dec, declen);
-	    if (crcpkt != crc)
+	    if (crcpkt != crc) {
+		ec = PGPARMOR_ERR_CRC_CHECK;
 		goto exit;
+	    }
 	    b = _free(b);
 	    b = dec;
 	    blen = declen;
