@@ -9,6 +9,10 @@
 #endif
 
 #include <rpmio_internal.h>
+
+#define	_RPMDAV_INTERNAL
+#include <rpmdav.h>
+
 #include "ugid.h"
 #include "debug.h"
 
@@ -16,6 +20,19 @@
 /*@access FD_t @*/
 /*@access urlinfo @*/
 
+/**
+ * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
+ * @param p		memory to free
+ * @retval		NULL always
+ */
+/*@unused@*/ static inline /*@null@*/ void *
+_free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
+	/*@modifies p@*/
+{
+    if (p != NULL)	free((void *)p);
+    return NULL;
+}
+
 /* =============================================================== */
 static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
 	/*@globals h_errno, fileSystem, internalState @*/
@@ -65,7 +82,6 @@ static int ftpUnlink(const char * path)
 }
 
 /* =============================================================== */
-/* XXX rebuilddb.c: analogues to mkdir(2)/rmdir(2). */
 int Mkdir (const char * path, mode_t mode)
 {
     const char * lpath;
@@ -75,13 +91,17 @@ int Mkdir (const char * path, mode_t mode)
     case URL_IS_FTP:
 	return ftpMkdir(path, mode);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davMkdir(path, mode);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -98,13 +118,21 @@ int Chdir (const char * path)
     case URL_IS_FTP:
 	return ftpChdir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef	NOTYET
+	return davChdir(path);
+#else
+	return -2;
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -121,13 +149,17 @@ int Rmdir (const char * path)
     case URL_IS_FTP:
 	return ftpRmdir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davRmdir(path);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -148,12 +180,16 @@ int Rename (const char * oldpath, const char * newpath)
 
     oldut = urlPath(oldpath, &oe);
     switch (oldut) {
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davRename(oldpath, newpath);
+	/*@notreached@*/ break;
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -169,6 +205,7 @@ fprintf(stderr, "*** rename old %*s new %*s\n", (int)(oe - oldpath), oldpath, (i
 	    return -2;
 	return ftpRename(oldpath, newpath);
 	/*@notreached@*/ break;
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
 	oldpath = oe;
@@ -177,6 +214,7 @@ fprintf(stderr, "*** rename old %*s new %*s\n", (int)(oe - oldpath), oldpath, (i
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -192,12 +230,14 @@ int Link (const char * oldpath, const char * newpath)
 
     oldut = urlPath(oldpath, &oe);
     switch (oldut) {
-    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -205,6 +245,7 @@ int Link (const char * oldpath, const char * newpath)
 
     newut = urlPath(newpath, &ne);
     switch (newut) {
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
@@ -219,6 +260,7 @@ fprintf(stderr, "*** link old %*s new %*s\n", (int)(oe - oldpath), oldpath, (int
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -236,13 +278,17 @@ int Unlink(const char * path) {
     case URL_IS_FTP:
 	return ftpUnlink(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davUnlink(path);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1020,20 +1066,21 @@ static const char * statstr(const struct stat * st,
 {
     sprintf(buf,
 	"*** dev %x ino %x mode %0o nlink %d uid %d gid %d rdev %x size %x\n",
-	(unsigned)st->st_dev,
-	(unsigned)st->st_ino,
-	st->st_mode,
-	(unsigned)st->st_nlink,
-	st->st_uid,
-	st->st_gid,
-	(unsigned)st->st_rdev,
-	(unsigned)st->st_size);
+	(unsigned int)st->st_dev,
+	(unsigned int)st->st_ino,
+	(unsigned int)st->st_mode,
+	(unsigned int)st->st_nlink,
+	(unsigned int)st->st_uid,
+	(unsigned int)st->st_gid,
+	(unsigned int)st->st_rdev,
+	(unsigned int)st->st_size);
     return buf;
 }
 
 /*@unchecked@*/
 static int ftp_st_ino = 0xdead0000;
 
+/* FIXME: borked for path with trailing '/' */
 static int ftpStat(const char * path, /*@out@*/ struct stat *st)
 	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
 	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
@@ -1049,6 +1096,7 @@ fprintf(stderr, "*** ftpStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
     return rc;
 }
 
+/* FIXME: borked for path with trailing '/' */
 static int ftpLstat(const char * path, /*@out@*/ struct stat *st)
 	/*@globals ftp_st_ino, h_errno, fileSystem, internalState @*/
 	/*@modifies *st, ftp_st_ino, fileSystem, internalState @*/
@@ -1075,44 +1123,13 @@ fprintf(stderr, "*** ftpReadlink(%s) rc %d\n", path, rc);
     return rc;
 }
 
-struct __dirstream {
-    int fd;			/* File descriptor.  */
-    char * data;		/* Directory block.  */
-    size_t allocation;		/* Space allocated for the block.  */
-    size_t size;		/* Total valid data in the block.  */
-    size_t offset;		/* Current offset into the block.  */
-    off_t filepos;		/* Position of next entry to read.  */
-#if defined(HAVE_PTHREAD_H)
-    pthread_mutex_t lock;	/* Mutex lock for this structure.  */
-#endif
-};
-
-#if !defined(DT_DIR)
-# define DT_UNKNOWN	0
-# define DT_FIFO	1
-# define DT_CHR		2
-# define DT_DIR		4
-# define DT_BLK		6
-# define DT_REG		8
-# define DT_LNK		10
-# define DT_SOCK	12
-# define DT_WHT		14
-typedef struct __dirstream *	FTPDIR;
-#else
-typedef DIR *			FTPDIR;
-#endif
-
-/*@unchecked@*/
-static int ftpmagicdir = 0x8440291;
-#define	ISFTPMAGIC(_dir) (!memcmp((_dir), &ftpmagicdir, sizeof(ftpmagicdir)))
-
 /*@-boundswrite@*/
 /*@null@*/
 static DIR * ftpOpendir(const char * path)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
 {
-    FTPDIR mydir;
+    AVDIR avdir;
     struct dirent * dp;
     size_t nb;
     const char * s, * sb, * se;
@@ -1160,23 +1177,29 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 	}
     }
 
-    nb += sizeof(*mydir) + sizeof(*dp) + ((ac + 1) * sizeof(*av)) + (ac + 1);
-    mydir = xcalloc(1, nb);
+    nb += sizeof(*avdir) + sizeof(*dp) + ((ac + 1) * sizeof(*av)) + (ac + 1);
+    avdir = xcalloc(1, nb);
     /*@-abstract@*/
-    dp = (struct dirent *) (mydir + 1);
+    dp = (struct dirent *) (avdir + 1);
     av = (const char **) (dp + 1);
     dt = (char *) (av + (ac + 1));
     t = (char *) (dt + ac + 1);
     /*@=abstract@*/
 
-    mydir->fd = ftpmagicdir;
+    avdir->fd = avmagicdir;
 /*@-usereleased@*/
-    mydir->data = (char *) dp;
+    avdir->data = (char *) dp;
 /*@=usereleased@*/
-    mydir->allocation = nb;
-    mydir->size = ac;
-    mydir->offset = -1;
-    mydir->filepos = 0;
+    avdir->allocation = nb;
+    avdir->size = ac;
+    avdir->offset = -1;
+    avdir->filepos = 0;
+
+#if defined(HAVE_PTHREAD_H)
+/*@-moduncon -noeffectuncon@*/
+    (void) pthread_mutex_init(&avdir->lock, NULL);
+/*@=moduncon =noeffectuncon@*/
+#endif
 
     ac = 0;
     /*@-dependenttrans -unrecog@*/
@@ -1241,78 +1264,11 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
     av[ac] = NULL;
 
 /*@-kepttrans@*/
-    return (DIR *) mydir;
+    return (DIR *) avdir;
 /*@=kepttrans@*/
 }
 /*@=boundswrite@*/
 
-/*@dependent@*/ /*@null@*/
-static struct dirent * ftpReaddir(DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/
-{
-    FTPDIR mydir = (FTPDIR)dir;
-    struct dirent * dp;
-    const char ** av;
-    unsigned char * dt;
-    int ac;
-    int i;
-
-    if (mydir == NULL || !ISFTPMAGIC(mydir) || mydir->data == NULL) {
-	/* XXX TODO: EBADF errno. */
-	return NULL;
-    }
-
-    dp = (struct dirent *) mydir->data;
-    av = (const char **) (dp + 1);
-    ac = mydir->size;
-    dt = (char *) (av + (ac + 1));
-    i = mydir->offset + 1;
-
-/*@-boundsread@*/
-    if (i < 0 || i >= ac || av[i] == NULL)
-	return NULL;
-/*@=boundsread@*/
-
-    mydir->offset = i;
-
-    /* XXX glob(3) uses REAL_DIR_ENTRY(dp) test on d_ino */
-/*@-type@*/
-    dp->d_ino = i + 1;		/* W2DO? */
-    dp->d_reclen = 0;		/* W2DO? */
-
-#if !defined(hpux) && !defined(sun)
-    dp->d_off = 0;		/* W2DO? */
-/*@-boundsread@*/
-    dp->d_type = dt[i];
-/*@=boundsread@*/
-#endif
-/*@=type@*/
-
-    strncpy(dp->d_name, av[i], sizeof(dp->d_name));
-if (_ftp_debug)
-fprintf(stderr, "*** ftpReaddir(%p) %p \"%s\"\n", (void *)mydir, dp, dp->d_name);
-    
-    return dp;
-}
-
-static int ftpClosedir(/*@only@*/ DIR * dir)
-	/*@globals fileSystem @*/
-	/*@modifies dir, fileSystem @*/
-{
-    FTPDIR mydir = (FTPDIR)dir;
-
-if (_ftp_debug)
-fprintf(stderr, "*** ftpClosedir(%p)\n", (void *)mydir);
-    if (mydir == NULL || !ISFTPMAGIC(mydir)) {
-	/* XXX TODO: EBADF errno. */
-	return -1;
-    }
-    free((void *)mydir);
-    mydir = NULL;
-    return 0;
-}
-
 int Stat(const char * path, struct stat * st)
 {
     const char * lpath;
@@ -1324,13 +1280,17 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
     case URL_IS_FTP:
 	return ftpStat(path, st);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davStat(path, st);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1349,13 +1309,17 @@ fprintf(stderr, "*** Lstat(%s,%p)\n", path, st);
     case URL_IS_FTP:
 	return ftpLstat(path, st);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+	return davLstat(path, st);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1372,13 +1336,21 @@ int Readlink(const char * path, char * buf, size_t bufsiz)
     case URL_IS_FTP:
 	return ftpReadlink(path, buf, bufsiz);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
+#ifdef	NOTYET
+	return davReadlink(path, buf, bufsiz);
+#else
+	return -2;
+#endif
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1396,14 +1368,16 @@ int Access(const char * path, int amode)
 if (_rpmio_debug)
 fprintf(stderr, "*** Access(%s,%d)\n", path, amode);
     switch (ut) {
-    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:		/* XXX WRONG WRONG WRONG */
     case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1439,7 +1413,7 @@ int Glob_pattern_p (const char * pattern, int quote)
 	case ']':
 	    if (open)
 		return (1);
-	    continue;      
+	    continue;
 
 	case '+':
 	case '@':
@@ -1468,6 +1442,8 @@ if (_rpmio_debug)
 fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)errfunc, pglob);
 /*@=castfcnptr@*/
     switch (ut) {
+    case URL_IS_HTTPS:
+    case URL_IS_HTTP:
     case URL_IS_FTP:
 /*@-type@*/
 	pglob->gl_closedir = Closedir;
@@ -1479,15 +1455,13 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
 	flags |= GLOB_ALTDIRFUNC;
 	flags &= ~GLOB_TILDE;
 	break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
-	flags &= ~GLOB_TILDE;
-	/*@fallthrough@*/
     case URL_IS_PATH:
 	pattern = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return -2;
 	/*@notreached@*/ break;
@@ -1513,13 +1487,17 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
     case URL_IS_FTP:
 	return ftpOpendir(path);
 	/*@notreached@*/ break;
-    case URL_IS_HTTP:		/* XXX WRONG WRONG WRONG */
+    case URL_IS_HTTPS:	
+    case URL_IS_HTTP:
+	return davOpendir(path);
+	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
 	break;
     case URL_IS_DASH:
+    case URL_IS_HKP:
     default:
 	return NULL;
 	/*@notreached@*/ break;
@@ -1533,8 +1511,12 @@ struct dirent * Readdir(DIR * dir)
 {
 if (_rpmio_debug)
 fprintf(stderr, "*** Readdir(%p)\n", (void *)dir);
-    if (dir == NULL || ISFTPMAGIC(dir))
-	return ftpReaddir(dir);
+    if (dir == NULL)
+	return NULL;
+    if (ISAVMAGIC(dir))
+	return avReaddir(dir);
+    if (ISDAVMAGIC(dir))
+	return davReaddir(dir);
     return readdir(dir);
 }
 
@@ -1542,7 +1524,11 @@ int Closedir(DIR * dir)
 {
 if (_rpmio_debug)
 fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
-    if (dir == NULL || ISFTPMAGIC(dir))
-	return ftpClosedir(dir);
+    if (dir == NULL)
+	return 0;
+    if (ISAVMAGIC(dir))
+	return avClosedir(dir);
+    if (ISDAVMAGIC(dir))
+	return davClosedir(dir);
     return closedir(dir);
 }
