@@ -1,4 +1,5 @@
 #include "system.h"
+const char *__progname;
 
 #define	_AUTOHELP
 
@@ -74,6 +75,14 @@ static struct poptOption optionsTable[] = {
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmVerifyPoptTable, 0,
 	N_("Verify options (with -V or --verify):"),
 	NULL },
+#ifdef	NOTYET
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliQVSourcePoptTable, 0,
+        N_("Source options (with --query or --verify):"),
+        NULL },
+#endif
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliFtsPoptTable, 0,
+        N_("File tree walk options (with --ftswalk):"),
+        NULL },
 #endif	/* IAM_RPMQV */
 
 #ifdef	IAM_RPMK
@@ -101,7 +110,7 @@ static struct poptOption optionsTable[] = {
 #endif	/* IAM_RPMEIU */
 
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliAllPoptTable, 0,
-	N_("Common options for all rpm modes:"),
+	N_("Common options for all rpm modes and executables:"),
 	NULL },
 
    POPT_AUTOALIAS
@@ -400,8 +409,9 @@ int main(int argc, const char ** argv)
 	    break;
 	case RPMSIGN_ADD_SIGNATURE:
 	case RPMSIGN_NEW_SIGNATURE:
+	case RPMSIGN_DEL_SIGNATURE:
 	    bigMode = MODE_RESIGN;
-	    ka->sign = 1;
+	    ka->sign = (ka->qva_mode != RPMSIGN_DEL_SIGNATURE);
 	    break;
 	}
   }
@@ -782,7 +792,8 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 
 #ifdef	IAM_RPMQV
     case MODE_QUERY:
-	if (qva->qva_source != RPMQV_ALL && !poptPeekArg(optCon))
+	if (!poptPeekArg(optCon)
+	 && !(qva->qva_source == RPMQV_ALL || qva->qva_source == RPMQV_HDLIST))
 	    argerror(_("no arguments given for query"));
 
 	qva->qva_specQuery = rpmspecQuery;
@@ -796,7 +807,8 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 	verifyFlags &= ~qva->qva_flags;
 	qva->qva_flags = (rpmQueryFlags) verifyFlags;
 
-	if (qva->qva_source != RPMQV_ALL && !poptPeekArg(optCon))
+	if (!poptPeekArg(optCon)
+	 && !(qva->qva_source == RPMQV_ALL || qva->qva_source == RPMQV_HDLIST))
 	    argerror(_("no arguments given for verify"));
 	ec = rpmcliVerify(ts, qva, (const char **) poptGetArgs(optCon));
     }	break;
